/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __ANIMATION_H__
#define __ANIMATION_H__

#include "framebuffer.h"
#include "mclip.h"

/** This class is instantiated whenever we need to play back an animation (or clip).
 *  This is an abstract base class. The different output devices derive their own classes
 *  from this */
class Animation {
 protected:
  /// The frame buffer to be used for output.
  FrameBuffer *frameBuffer;
  /// The clip to play back.
  MediaClip   *source;

  /** Initialize the frame buffer & vars.
   *  This function should be overridden by the derived classes to set up any vars &
   *  frame buffer pointers */
  virtual void Init () = NULL;
  /** The frame buffer has stopped its playback.
   *  This function is so windowed apps can prepare any pixmaps for repaints */
  virtual void StopPlayback() = NULL;
  /** Display the next frame.
   *  This function is called whenever a new frame is ready to be displayed in the
   *  Frame buffer - it is typically used to flip screens (or blit to a window). */
  virtual void NextFrame () = NULL;

 public:
  virtual ~Animation ();

  /// Main entry point
  void Animate (int startFrame, int endFrame);

  /// Show a specific frame
  void ShowFrame (int frame);
  
  /// Set the source clip to play back
  void SetSource (MediaClip *newSrc);
};


#endif  //  __ANIMATION_H__
