/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or lied. You use this program at your own risk.
 */

/* This file contains all the code neccessary to get the main window running.
 */
#include <gtk/gtk.h>
#include <gdk_imlib.h>
#include <gnome.h>

#include "config.h"

#include "codeccentral.h"
#include "codec.h"
#include "gtkanimwindow.h"
#include "application.h"
#include "clipview.h"
#include "misc.h"

// clip types
#include "colorclip.h"
#include "singleimage.h"
#include "imagesequence.h"
#include "timeline.h"
#include "trisequence.h"

Application *app;

Application::Application (int argc, char **argv) {
  app = this;
  project = NULL;
  clipboard = NULL;
  //  gtk_init (&argc, &argv);
  gnome_init ("WhatIsThis", "AndWhatIsThis", argc, argv);

  ShowSplashScreen ();
  prefs = new AppSettings ();

  animation = new GtkAnimationWindow (320, 240);
  
  CodecCentral::InitCodecs ();

  mainWindow = new ProjectWindow ();

  NewProject ();

  gdk_window_destroy (splashScreen);
  mainWindow->Show ();
}

/// The window has been closed, or the user has select Quit. 
void Application::OpenProject () {
  if (project->ChangedSinceSave ()) {
    switch (ShowDialog ("Close project", "The current project has not been saved.\n\nDo you wish to save the current changes ?", YesNoCancel)) {
    case Yes:
      if (!SaveProject ())
	return;
    case Cancel:
      return;
    }
  }
  
  char *fName = ShowFileSelector ("Save project", project->GetFileName ());
  if (!CheckFile (fName)) {
    ShowDialog ("Open project", "There was an error opening the specified file.", Error);
    return;
  }

  Project *prj = new Project();
  if (prj->Load (fName)) {
    delete project;
    project = prj;
    mainWindow->BuildProjectTreeView ();
    mainWindow->SetCurrentFolder (project->GetRoot ());
  } else 
    delete prj;
}
int Application::SaveProject () {
  if (!project->GetFileName ()) {
    return SaveProjectAs ();
  }
  return project->Save ();
}

int Application::SaveProjectAs () {
  char *temp = ShowFileSelector ("Save project", project->GetFileName ());

  if (temp) {
    printf ("%s\n", temp); 
    project->SetFileName (temp);
    delete temp;
    return project->Save ();
  }
  return FALSE;
}

/// Close the current project
int Application::CloseProject () {
  if (!project) {
    return TRUE;
  }

  int retVal = TRUE;

  if (project->ChangedSinceSave ()) {
    switch (ShowDialog ("Close project", "The current project has not been saved.\n\nDo you wish to save the current changes ?", YesNoCancel)) {
    case Yes:
      retVal = SaveProject ();
      break;
    case Cancel:
      retVal = FALSE;
    }
  }
  if (retVal) {
    if (project)
      delete project;
    project=NULL;
  }
  return retVal;
}

/// Make a new project. TODO: Load a default project from a number of locations
void Application::NewProject () {
  if (!CloseProject ())
    return;

  project = new Project ();
  mainWindow->BuildProjectTreeView ();
  mainWindow->SetCurrentFolder (project->GetRoot ());
}

void Application::ShowSplashScreen () {
  GdkWindowAttr         attr;
  GdkPixmap             *p,*m;
  GdkImlibImage         *im;
  gint w,h;
  
  gdk_imlib_init();
  
  gtk_widget_push_visual(gdk_imlib_get_visual());
  gtk_widget_push_colormap(gdk_imlib_get_colormap());
       
  /* Load the image specified as the first argument */
  im=gdk_imlib_load_image(DATADIR"icons/trinity.jpeg");
  /* Suck[B the image's original width and height out of the Image structure */
  w=im->rgb_width;h=im->rgb_height;
  /* Set up attributes for GDK to create a Window */
  attr.window_type=GDK_WINDOW_TOPLEVEL;
  attr.wclass=GDK_INPUT_OUTPUT;
  attr.event_mask=GDK_STRUCTURE_MASK;
  attr.width=w;
  attr.height=h;
  /* Create a Window to display in */
  splashScreen=gdk_window_new(NULL,&attr,0);
    
  /* Render the original 24-bit Image data into a pixmap of size w * h */
  gdk_imlib_render(im,w,h);
  /* Extract the Image and mask pixmaps from the Image */
  p=gdk_imlib_move_image(im);
  /* Put the Image pixmap in the background of the window */
  gdk_window_set_back_pixmap(splashScreen,p,0);
  /* Actually display the window */
  gdk_window_show(splashScreen);
  /* Flush the GDK buffer */
  gdk_flush();
  /* We are finished with the imlib version, so just kill it */
  gdk_imlib_kill_image (im);
};

void Application::MakeNewFolder (char *name) {
  switch (mainWindow->GetCurrentFolder()->MakeSubFolder (name)) {
  case 1:
    ShowDialog ("Make new folder", "A folder with that name already exists.\n\nPlease specify a unique name", Error);
    return;
  case 2:
    ShowDialog ("Make new folder", "The characters /, >, < are not permitted in folder or clip names.", Error);
    return;
  case 0:
    mainWindow->GetCurrentFolder()->SetOpened (TRUE);
    mainWindow->BuildProjectTreeView ();
  }
}

int Application::RenameCurrentFolder(char *newName) {
  if (!strcmp (mainWindow->GetCurrentFolder()->GetName(), newName))
    return 0;
  if (mainWindow->GetCurrentFolder()->GetParent())
    switch (mainWindow->GetCurrentFolder()->GetParent()->CheckFolderName (newName)) {
    case 1:
      ShowDialog ("Rename folder", "A folder with that name already exists.\n\nPlease specify a unique name", Error);
      return 1;
    case 2:
      ShowDialog ("Rename folder", "The characters /, >, < are not permitted in folder or clip names.", Error);
      return 2;
    }
  mainWindow->GetCurrentFolder()->SetName (newName);
  mainWindow->BuildProjectTreeView ();
  return 0;
}

void Application::EraseCurrentFolder() {
  if (mainWindow->GetCurrentFolder() == project->GetRoot()) {
    ShowDialog ("Delete folder", "Error\n\nYou cannot delete a projects root folder.", 
		Error);
    return;
  }

  mainWindow->GetCurrentFolder()->GetParent()->DeleteSubFolder (mainWindow->GetCurrentFolder());

  mainWindow->GetCurrentFolder()->Erase ();
  mainWindow->BuildProjectTreeView ();
}

void Application::PlayClip (MediaClip *clip, int startFrame, int endFrame) {
  animation->SetSource (clip);
  animation->Animate (startFrame, endFrame);
}

void Application::RegisterListener (SignalListener *l) {
  listeners = g_slist_append (listeners, l);
};
void Application::DeRegisterListener (SignalListener *l) {
  listeners = g_slist_remove (listeners, l);
};
static void _EmitSignal (gpointer obj, gpointer signal) {
  ((SignalListener *)obj)->Signal ((AppSignal *)signal);
}
void Application::EmitSignal (AppSignal *sig) {
  g_slist_foreach (listeners, _EmitSignal, sig);
}

Clip *Application::BuildClip (XmlToken *src) {
  if (!g_strcasecmp (src->GetKey (), "folder")) {
    Folder *fold = new Folder ();
    fold->Load (src);
    return fold;
  }
  if (!g_strcasecmp (src->GetKey (), "ImageSequence")) {
    ImageSequenceClip *is = new ImageSequenceClip ();
    is->Load (src);
    return is;
  }
  if (!g_strcasecmp (src->GetKey (), "SingleImage")) {
    SingleImageClip *si = new SingleImageClip ();
    si->Load (src);
    return si;
  }
  if (!g_strcasecmp (src->GetKey (), "SolidColor")) {
    ColorClip *c = new ColorClip ();
    c->Load (src);
    return c;
  }
  if (!g_strcasecmp (src->GetKey (), "TimelineClip")) {
    TimelineClip *tc = new TimelineClip ();
    tc->Load (src);
    return (Clip *)tc;
  }
  if (!g_strcasecmp (src->GetKey (), "TriSequence")) {
    TriSequenceClip *tc = new TriSequenceClip ();
    tc->Load (src);
    return (Clip *)tc;
  }

  if (!g_strcasecmp (src->GetKey (), "Timeline")) {
    Timeline *tl = new Timeline ();
    tl->Load (src);
    return tl;
  }
  
  return NULL;
}

void Application::SetClipboard (MediaClip *src) {
  clipboard = src;
  clipboardMediaClip = TRUE;
}
void Application::SetClipboard (Clip *src) {
  clipboard = src;
  clipboardMediaClip = FALSE;
}
MediaClip *Application::GetClipboardMediaClip () {
  assert (clipboardMediaClip);
  return (MediaClip *)clipboard;
}
Clip *Application::GetClipboard () {
  return clipboard;
}

int Application::IsClipboardMediaClip () {
  return clipboardMediaClip;
}

char *Application::GetClipboardDesc () {
  if (clipboardMediaClip)
    return ((MediaClip *)clipboard)->GetTypeName();
  return "Clip";
}

