/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or lied. You use this program at your own risk.
 */

#include "appsettings.h"
#include <gnome.h>
#include "application.h"

AppSettings::AppSettings () {
  editSettings = NULL;
  SetDefault ();
}

void AppSettings::SetDefault () {
  clipTimeBG = 0x008080FF;
  clipTimeInOut = 0x40c0c0FF;
  clipTimeMarker = 0xFF;
  panelClipViewWidth = 120;
  panelClipPlayerButtons = 0;
  timeLineBG = 0x808080FF;
  timeLineActiveFrame = 0xFF80;
  timeLineVideoColor = 0x408080FF;
  timeLineAudioColor = 0x508050FF;
  timeLineClipEdges = 0xFF;
}

void AppSettings::Save (XmlToken *dest) {
  dest->Clear ();
  dest->SetKey ("AppSettings");
  dest->AppendInt ("ClipTimeBG", clipTimeBG);
  dest->AppendInt ("ClipTimeInOut", clipTimeInOut);
  dest->AppendInt ("ClipTimeMarker", clipTimeMarker);
  dest->AppendInt ("PanelClipWidth", panelClipViewWidth);
  dest->AppendInt ("panelClipPlayerButtons", panelClipPlayerButtons);
  dest->AppendInt ("timeLineBG", timeLineBG);
  dest->AppendInt ("timeLineActiveFrame", timeLineActiveFrame);
  dest->AppendInt ("timeLineVideoColor", timeLineVideoColor);
  dest->AppendInt ("timeLineAudioColor", timeLineAudioColor);
  dest->AppendInt ("timeLineClipEdges", timeLineClipEdges);
};

void AppSettings::Load (XmlToken *src) {
  clipTimeBG = src->GetInt ("ClipTimeBG");
  clipTimeInOut = src->GetInt ("ClipTimeInOut");
  clipTimeMarker = src->GetInt ("ClipTimeMarker");
  panelClipViewWidth = src->GetInt ("PanelClipWidth");
  panelClipPlayerButtons = src->GetInt ("panelClipPlayerButtons");
  timeLineBG = src->GetInt ("timeLineBG");
  timeLineVideoColor = src->GetInt ("timeLineVideoColor");
  timeLineAudioColor = src->GetInt ("timeLineAudioColor");
  timeLineClipEdges = src->GetInt ("timeLineClipEdges");
}

void AppSettings::BuildEditWidget () {
  GtkWidget *dialog_vbox1;
  GtkWidget *notebook1;
  GtkWidget *vbox7;
  GtkWidget *hbox7;
  GtkWidget *frame4;
  GtkWidget *table2;
  GtkWidget *label14;
  GtkWidget *label12;
  GtkWidget *entry1;
  GtkWidget *hbox9;
  GtkObject *spinbutton1_adj;
  GtkWidget *spinbutton1;
  GtkWidget *label20;
  GtkWidget *checkbutton8;
  GtkWidget *checkbutton7;
  GtkWidget *frame5;
  GtkWidget *table5;
  GtkWidget *label21;
  GtkWidget *hbox10;
  GtkObject *spinbutton2_adj;
  GtkWidget *spinbutton2;
  GtkWidget *label23;
  GtkWidget *label17;
  GtkWidget *label1;
  GtkWidget *vbox4;
  GtkWidget *frame2;
  GtkWidget *table4;
  GtkWidget *label11;
  GtkWidget *combo1;
  GList *combo1_items = NULL;
  GtkWidget *optionmenu2;
  GtkWidget *optionmenu2_menu;
  GtkWidget *glade_menuitem;
  GtkWidget *label10;
  GtkWidget *checkbutton2;
  GtkWidget *checkbutton5;
  GtkWidget *optionmenu6;
  GtkWidget *optionmenu6_menu;
  GtkWidget *checkbutton4;
  GtkWidget *frame3;
  GtkWidget *table3;
  GtkWidget *checkbutton9;
  GtkWidget *label18;
  GtkWidget *optionmenu4;
  GtkWidget *optionmenu4_menu;
  GtkWidget *label19;
  GtkWidget *optionmenu5;
  GtkWidget *optionmenu5_menu;
  GtkWidget *checkbutton6;
  GtkWidget *optionmenu7;
  GtkWidget *optionmenu7_menu;
  GtkWidget *label2;
  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *label4;
  GtkWidget *optionmenu1;
  GtkWidget *optionmenu1_menu;
  GtkWidget *frame1;
  GtkWidget *vbox2;
  GtkWidget *label3;
  GtkWidget *dialog_action_area1;
  GtkWidget *button1;
  GtkWidget *button2;
  GtkWidget *button3;
  GtkAccelGroup *accel_group;
  GtkTooltips *tooltips;

  if (editSettings)
    delete editSettings;

  editSettings = new AppSettings (*this);

  tooltips = gtk_tooltips_new ();

  accel_group = gtk_accel_group_new ();

  dialog1 = gnome_dialog_new (NULL, NULL);
  gtk_widget_ref (dialog1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "dialog1", dialog1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_window_set_policy (GTK_WINDOW (dialog1), FALSE, FALSE, FALSE);

  dialog_vbox1 = GNOME_DIALOG (dialog1)->vbox;
  gtk_widget_ref (dialog_vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "dialog_vbox1", dialog_vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_vbox1);

  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (dialog_vbox1), notebook1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (notebook1), 4);

  vbox7 = gtk_vbox_new (FALSE, 2);
  gtk_widget_ref (vbox7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox7", vbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox7);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox7);
  gtk_container_set_border_width (GTK_CONTAINER (vbox7), 4);

  hbox7 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox7", hbox7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox7);
  gtk_box_pack_start (GTK_BOX (vbox7), hbox7, TRUE, TRUE, 0);

  frame4 = gtk_frame_new (" Playback cache: ");
  gtk_widget_ref (frame4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame4", frame4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame4);
  gtk_box_pack_start (GTK_BOX (hbox7), frame4, TRUE, TRUE, 0);

  table2 = gtk_table_new (4, 2, TRUE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (frame4), table2);
  gtk_container_set_border_width (GTK_CONTAINER (table2), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table2), 4);

  label14 = gtk_label_new ("Cache size:");
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_table_attach (GTK_TABLE (table2), label14, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (label14, FALSE);
  gtk_misc_set_alignment (GTK_MISC (label14), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label14), 22, 0);

  label12 = gtk_label_new ("Cache location:");
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table2), label12, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (label12, FALSE);
  gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label12), 22, 0);

  entry1 = gtk_entry_new ();
  gtk_widget_ref (entry1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "entry1", entry1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (entry1);
  gtk_table_attach (GTK_TABLE (table2), entry1, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (entry1, FALSE);
  gtk_tooltips_set_tip (tooltips, entry1, "Set the location of the cache files", NULL);
  gtk_entry_set_text (GTK_ENTRY (entry1), "/tmp/");

  hbox9 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox9", hbox9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox9);
  gtk_table_attach (GTK_TABLE (table2), hbox9, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);

  spinbutton1_adj = gtk_adjustment_new (1, 0, 10000, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_box_pack_start (GTK_BOX (hbox9), spinbutton1, TRUE, TRUE, 0);
  gtk_widget_set_usize (spinbutton1, 92, -2);
  gtk_widget_set_sensitive (spinbutton1, FALSE);
  gtk_tooltips_set_tip (tooltips, spinbutton1, "Set the program cache size", NULL);

  label20 = gtk_label_new ("MB");
  gtk_widget_ref (label20);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label20", label20,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label20);
  gtk_box_pack_start (GTK_BOX (hbox9), label20, FALSE, FALSE, 0);

  checkbutton8 = gtk_check_button_new_with_label ("Each Project has its own cache");
  gtk_widget_ref (checkbutton8);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton8", checkbutton8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton8);
  gtk_table_attach (GTK_TABLE (table2), checkbutton8, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 22, 0);
  gtk_widget_set_sensitive (checkbutton8, FALSE);

  checkbutton7 = gtk_check_button_new_with_label ("Enable");
  gtk_widget_ref (checkbutton7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton7", checkbutton7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton7);
  gtk_table_attach (GTK_TABLE (table2), checkbutton7, 0, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (checkbutton7, FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton7), TRUE);

  frame5 = gtk_frame_new (" Memory buffer: ");
  gtk_widget_ref (frame5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame5", frame5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame5);
  gtk_box_pack_start (GTK_BOX (vbox7), frame5, FALSE, FALSE, 0);

  table5 = gtk_table_new (1, 2, TRUE);
  gtk_widget_ref (table5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table5", table5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table5);
  gtk_container_add (GTK_CONTAINER (frame5), table5);
  gtk_container_set_border_width (GTK_CONTAINER (table5), 4);

  label21 = gtk_label_new ("Cache size:");
  gtk_widget_ref (label21);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label21", label21,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label21);
  gtk_table_attach (GTK_TABLE (table5), label21, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

  hbox10 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox10", hbox10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox10);
  gtk_table_attach (GTK_TABLE (table5), hbox10, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);

  spinbutton2_adj = gtk_adjustment_new (64, 0, 1000, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_box_pack_start (GTK_BOX (hbox10), spinbutton2, TRUE, TRUE, 0);
  gtk_widget_set_usize (spinbutton2, 92, -2);
  gtk_widget_set_sensitive (spinbutton2, FALSE);
  gtk_tooltips_set_tip (tooltips, spinbutton2, "Set the program cache size", NULL);

  label23 = gtk_label_new ("MB");
  gtk_widget_ref (label23);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label23", label23,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label23);
  gtk_box_pack_start (GTK_BOX (hbox10), label23, FALSE, FALSE, 0);

  label17 = gtk_label_new ("The cache is used to store all effects in the playback\nresolution. Unless you have a monster machine, it is\nrecommended to use it.\n\nThe buffer is used to minimze frame skipping. If you \nsuffer from this, try to increase its size.");
  gtk_widget_ref (label17);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label17", label17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label17);
  gtk_box_pack_start (GTK_BOX (vbox7), label17, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label17), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label17), 0, 0.5);

  label1 = gtk_label_new ("Cache");
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox4);

  frame2 = gtk_frame_new (" Project window: ");
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_box_pack_start (GTK_BOX (vbox4), frame2, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame2), 4);

  table4 = gtk_table_new (5, 2, TRUE);
  gtk_widget_ref (table4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table4", table4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table4);
  gtk_container_add (GTK_CONTAINER (frame2), table4);
  gtk_container_set_border_width (GTK_CONTAINER (table4), 4);

  label11 = gtk_label_new ("Preview size:");
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table4), label11, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label11), 24, 0);

  combo1 = gtk_combo_new ();
  gtk_widget_ref (combo1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "combo1", combo1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (combo1);
  gtk_table_attach (GTK_TABLE (table4), combo1, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (GTK_COMBO (combo1)->entry, 84, -2);
  gtk_widget_set_usize (combo1, 100, -2);
  gtk_widget_set_sensitive (combo1, FALSE);
  gtk_combo_set_use_arrows_always (GTK_COMBO (combo1), TRUE);
  combo1_items = g_list_append (combo1_items, "160x120");
  combo1_items = g_list_append (combo1_items, "200x150");
  combo1_items = g_list_append (combo1_items, "240x180");
  combo1_items = g_list_append (combo1_items, "320x240");
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);

  optionmenu2 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu2", optionmenu2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu2);
  gtk_table_attach (GTK_TABLE (table4), optionmenu2, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (optionmenu2, 100, -2);
  gtk_widget_set_sensitive (optionmenu2, FALSE);
  gtk_tooltips_set_tip (tooltips, optionmenu2, "Set how many panel buttons are visible", NULL);
  gtk_widget_add_accelerator (optionmenu2, "pressed", accel_group,
                              GDK_z, 0,
                              GTK_ACCEL_VISIBLE);
  optionmenu2_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Play");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Play + IN/OUT control");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu2_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu2), optionmenu2_menu);

  label10 = gtk_label_new ("Panel Buttons:");
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_table_attach (GTK_TABLE (table4), label10, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label10), 24, 0);

  checkbutton2 = gtk_check_button_new_with_label ("Main Toolbar");
  gtk_widget_ref (checkbutton2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton2", checkbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton2);
  gtk_table_attach (GTK_TABLE (table4), checkbutton2, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_sensitive (checkbutton2, FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton2), TRUE);

  checkbutton5 = gtk_check_button_new_with_label ("Show mini timeline");
  gtk_widget_ref (checkbutton5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton5", checkbutton5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton5);
  gtk_table_attach (GTK_TABLE (table4), checkbutton5, 0, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 20, 0);
  gtk_widget_set_usize (checkbutton5, 65, -2);
  gtk_widget_set_sensitive (checkbutton5, FALSE);

  optionmenu6 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu6", optionmenu6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu6);
  gtk_table_attach (GTK_TABLE (table4), optionmenu6, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (optionmenu6, 100, -2);
  gtk_widget_set_sensitive (optionmenu6, FALSE);
  optionmenu6_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Icons");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu6_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Text");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu6_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Text + Icons");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu6_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu6), optionmenu6_menu);

  checkbutton4 = gtk_check_button_new_with_label ("Clip Viewer panel");
  gtk_widget_ref (checkbutton4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton4", checkbutton4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton4);
  gtk_table_attach (GTK_TABLE (table4), checkbutton4, 0, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_sensitive (checkbutton4, FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton4), TRUE);

  frame3 = gtk_frame_new (" Timeline window:");
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox4), frame3, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (frame3), 4);

  table3 = gtk_table_new (4, 2, TRUE);
  gtk_widget_ref (table3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "table3", table3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table3);
  gtk_container_add (GTK_CONTAINER (frame3), table3);
  gtk_container_set_border_width (GTK_CONTAINER (table3), 4);
  gtk_table_set_row_spacings (GTK_TABLE (table3), 2);

  checkbutton9 = gtk_check_button_new_with_label ("Show output frames.");
  gtk_widget_ref (checkbutton9);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton9", checkbutton9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton9);
  gtk_table_attach (GTK_TABLE (table3), checkbutton9, 0, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (checkbutton9, FALSE);

  label18 = gtk_label_new ("Show video clips as:");
  gtk_widget_ref (label18);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label18", label18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label18);
  gtk_table_attach (GTK_TABLE (table3), label18, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label18), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label18), 0, 0.5);

  optionmenu4 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu4", optionmenu4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu4);
  gtk_table_attach (GTK_TABLE (table3), optionmenu4, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (optionmenu4, 100, -2);
  gtk_widget_set_sensitive (optionmenu4, FALSE);
  gtk_tooltips_set_tip (tooltips, optionmenu4, "Set how to display video clips", NULL);
  optionmenu4_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Blocks");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu4_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Frames");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu4_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu4), optionmenu4_menu);

  label19 = gtk_label_new ("Show audio clips as:");
  gtk_widget_ref (label19);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label19", label19,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label19);
  gtk_table_attach (GTK_TABLE (table3), label19, 0, 1, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label19), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label19), 0, 0.5);

  optionmenu5 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu5);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu5", optionmenu5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu5);
  gtk_table_attach (GTK_TABLE (table3), optionmenu5, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (optionmenu5, 100, -2);
  gtk_widget_set_sensitive (optionmenu5, FALSE);
  gtk_tooltips_set_tip (tooltips, optionmenu5, "Set how to display audio clips", NULL);
  optionmenu5_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Blocks");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Sample curves");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("VU-meters");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu5_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu5), optionmenu5_menu);

  checkbutton6 = gtk_check_button_new_with_label ("Toolbar");
  gtk_widget_ref (checkbutton6);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "checkbutton6", checkbutton6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton6);
  gtk_table_attach (GTK_TABLE (table3), checkbutton6, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_widget_set_sensitive (checkbutton6, FALSE);
  gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (checkbutton6), TRUE);

  optionmenu7 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu7);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu7", optionmenu7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu7);
  gtk_table_attach (GTK_TABLE (table3), optionmenu7, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_widget_set_usize (optionmenu7, 100, -2);
  gtk_widget_set_sensitive (optionmenu7, FALSE);
  optionmenu7_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Icons");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu7_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Text");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu7_menu), glade_menuitem);
  glade_menuitem = gtk_menu_item_new_with_label ("Text + Icons");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu7_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu7), optionmenu7_menu);

  label2 = gtk_label_new ("View");
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox1);
  gtk_container_set_border_width (GTK_CONTAINER (vbox1), 4);

  hbox1 = gtk_hbox_new (TRUE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  label4 = gtk_label_new ("Select colors to work with:");
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox1), label4, TRUE, TRUE, 0);

  optionmenu1 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "optionmenu1", optionmenu1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu1);
  gtk_box_pack_start (GTK_BOX (hbox1), optionmenu1, TRUE, TRUE, 0);
  optionmenu1_menu = gtk_menu_new ();
  glade_menuitem = gtk_menu_item_new_with_label ("Timeline colors");
  gtk_widget_show (glade_menuitem);
  gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);

  frame1 = gtk_frame_new (" Hi there ");
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);

  vbox2 = gtk_vbox_new (FALSE, 4);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (frame1), vbox2);
  gtk_container_set_border_width (GTK_CONTAINER (vbox2), 4);

  AddColor (vbox2, "Clip timer - background", &editSettings->clipTimeBG);
  AddColor (vbox2, "Clip timer - active region", &editSettings->clipTimeInOut);
  AddColor (vbox2, "Clip timer - markers", &editSettings->clipTimeMarker);
  
  GtkWidget *hseparator1 = gtk_hseparator_new ();
  gtk_widget_show (hseparator1);
  gtk_box_pack_start (GTK_BOX (vbox2), hseparator1, FALSE, FALSE, 0);
  
  AddColor (vbox2, "Timeline - background", &editSettings->timeLineBG);
  AddColor (vbox2, "Timeline - video tracks", &editSettings->timeLineVideoColor);
  AddColor (vbox2, "Timeline - audio tracks", &editSettings->timeLineAudioColor);
  AddColor (vbox2, "Timeline - track edges", &editSettings->timeLineClipEdges);
  AddColor (vbox2, "Timeline - active frame", &editSettings->timeLineActiveFrame,
	    true);

  label3 = gtk_label_new ("Colors");
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);

  dialog_action_area1 = GNOME_DIALOG (dialog1)->action_area;
  gtk_widget_ref (dialog_action_area1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "dialog_action_area1", dialog_action_area1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (dialog_action_area1);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (dialog_action_area1), 8);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_OK);
  button1 = GTK_WIDGET(g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_APPLY);
  button2 = GTK_WIDGET(g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gnome_dialog_append_button (GNOME_DIALOG (dialog1), GNOME_STOCK_BUTTON_CANCEL);
  button3 = GTK_WIDGET(g_list_last (GNOME_DIALOG (dialog1)->buttons)->data);
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (dialog1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  GTK_WIDGET_SET_FLAGS (button3, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (OkPressed),
                      this);
  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (ApplyPressed),
                      this);
  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (CancelPressed),
                      this);

  gtk_object_set_data (GTK_OBJECT (dialog1), "tooltips", tooltips);

  gtk_window_add_accel_group (GTK_WINDOW (dialog1), accel_group);

  gtk_window_set_modal (GTK_WINDOW (dialog1), TRUE);

  gtk_widget_show (dialog1);
}

void AppSettings::OkPressed (GtkButton *btn, gpointer user_data) {
  ((AppSettings *)user_data)->Apply ();
  ((AppSettings *)user_data)->Close ();
}

void AppSettings::ApplyPressed (GtkButton *btn, gpointer user_data) {
  ((AppSettings *)user_data)->Apply ();
}

void AppSettings::CancelPressed (GtkButton *btn, gpointer user_data) {
  ((AppSettings *)user_data)->Close ();
}

void AppSettings::Close () {
  gtk_widget_destroy (dialog1);
}

void AppSettings::Apply () {
  clipTimeBG = editSettings->clipTimeBG;
  clipTimeInOut = editSettings->clipTimeInOut;
  clipTimeMarker = editSettings->clipTimeMarker;
  panelClipViewWidth = editSettings->panelClipViewWidth;
  panelClipPlayerButtons = editSettings->panelClipPlayerButtons;
  timeLineBG = editSettings->timeLineBG;
  timeLineActiveFrame = editSettings->timeLineActiveFrame;
  timeLineVideoColor = editSettings->timeLineVideoColor;
  timeLineAudioColor = editSettings->timeLineAudioColor;
  timeLineClipEdges = editSettings->timeLineClipEdges;
  AppSignal temp (APP_SETTINGS, (void *)this);
  app->EmitSignal (&temp);
}

void AppSettings::AddColor (GtkWidget *parent, char *desc, guint32 *color, 
			    gboolean showAlpha) {
  GtkWidget *hbox2;
  GtkWidget *label24;
  GtkWidget *colorpicker1;

  hbox2 = gtk_hbox_new (TRUE, 16);
  gtk_widget_show (hbox2);
  gtk_box_pack_start (GTK_BOX (parent), hbox2, FALSE, FALSE, 0);

  label24 = gtk_label_new (desc);
  gtk_widget_show (label24);
  gtk_box_pack_start (GTK_BOX (hbox2), label24, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label24), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label24), 1, 0.5);

  colorpicker1 = gnome_color_picker_new ();
  gtk_widget_show (colorpicker1);
  gtk_box_pack_start (GTK_BOX (hbox2), colorpicker1, TRUE, TRUE, 0);
  gnome_color_picker_set_title (GNOME_COLOR_PICKER (colorpicker1), desc);
  gnome_color_picker_set_use_alpha (GNOME_COLOR_PICKER (colorpicker1), showAlpha);

  gnome_color_picker_set_i8 (GNOME_COLOR_PICKER (colorpicker1),
			     *color >> 24,
			     (*color & 0xFF0000) >> 16,
			     (*color & 0xFF00) >> 8,
			     *color & 0xFF);
  gtk_signal_connect (GTK_OBJECT (colorpicker1), "color_set",
                      GTK_SIGNAL_FUNC (ColorSet),
                      color);
}

void AppSettings::ColorSet (GnomeColorPicker *gnomecolorpicker,
			    guint            arg1,
			    guint            arg2,
			    guint            arg3,
			    guint            arg4,
			    gpointer         user_data) {
  guint32 color = ((arg1 & 0xFF00) << 16) +
    ((arg2 & 0xFF00) << 8) +
    (arg3 & 0xFF00) +
    (arg4 >> 8);

  printf ("%d\n", color);
  *((guint32 *)user_data) = color;
}

