/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or lied. You use this program at your own risk.
 */

#ifndef __APPSETTINGS_H__
#define __APPSETTINGS_H__


#include <gnome.h>
#include "xml.h"

/** The main settings for the application.
 *  This class enacpsulates all application settings (Color, etc) */
class AppSettings {
 public:
  /// The background for clip timelines (RGBA).
  guint32 clipTimeBG;
  /// The color for IN-OUT in clip timelines (RGBA).
  guint32 clipTimeInOut;
  /// The Color of the IN & OUT markers in clip timelines (RGBA).
  guint32 clipTimeMarker;
  /** The size of the clip viewer plane (RGBA).
   *  The height is deduced from the aspect ratio of the output. */
  guint panelClipViewWidth;
  /** The amount of buttons to show in the clip pane
   *  0 = just play & movement buttons
   *  1 = IN & OUT control as well
   *  2 = play, movement, timeline (default)
   *  3 = IN/OUT play, movement, timeline */
  guint panelClipPlayerButtons;
  /// The background color for timelines.
  guint32 timeLineBG;
  /// The line showing the active frame in a timeline.
  guint32 timeLineActiveFrame;  
  /// The color of video clips.
  guint32 timeLineVideoColor;
  /// The color of audio clips.
  guint32 timeLineAudioColor;
  /// The color of time line clips.
  guint32 timeLineClipEdges;

  /// Default constructor.
  AppSettings ();
  /// Reset to default settings.
  void SetDefault ();
  /// Save the settings to an XML token.
  void Save (XmlToken *dest);
  /// Load the settings from a token.
  void Load (XmlToken *src);
  /** Build the 'preferences' widget . */
  void BuildEditWidget ();
 private:
  /// Add a color selector to parent
  void AddColor (GtkWidget *parent, char *desc, guint32 *color, 
		 gboolean showAlpha = FALSE);
  /// Apply the canges of the dialog
  void Apply ();
  /// Close the dialog box.
  void Close ();

  /// Main dialog buttons callbacks
  static void OkPressed (GtkButton *btn, gpointer user_data);
  static void ApplyPressed (GtkButton *btn, gpointer user_data);
  static void CancelPressed (GtkButton *btn, gpointer user_data);
  /// Callback for the color selector.
  static void ColorSet (GnomeColorPicker *gnomecolorpicker,
			guint            arg1,
			guint            arg2,
			guint            arg3,
			guint            arg4,
			gpointer         user_data);
  /// Values for working with in the prefs box.
  AppSettings *editSettings;
  /// The prefs dialog widget
  GtkWidget *dialog1;
};

#endif
