/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */


#ifndef __APPSIGNAL_H__
#define __APPSIGNAL_H__

#include <memory.h>
/// The list of signals the app can emit.
enum AppSignalType {
  /** A clip has been changed.
   *  This signal is emitted whenever a clip is changed. The data member
   *  gets set to a pointer to the specified clip */
  CLIP_CHANGED,
  /** The main application preferences have changed.
   *  This signal is emitted whenever the app prefs are changed. The data 
   *  member is a pointer to the new app prefs */
  APP_SETTINGS,
  /** A clip has had a new active frame set
   *  This signal is emitted whenever a ClipTimer or a ClipPlayerButtons object 
   *  Changes the shown frame. */
  CLIP_ACTIVE_FRAME_CHANGED,
  /// A clip has it's IN point moved
  CLIP_IN_POINT_MOVED,
  /// A clip has it's OUT point moved
  CLIP_OUT_POINT_MOVED,
  /** All windows should reset themselves */
  RESET
};

/** A generic class for signals emitted by the app object.
 *  This is made so any objects can get notified when something changes.
 *  If a class wishes to be notified, it should derive from SignalListener.
 *  When the app emits a signal, all registered objects get a call to Signal */
class AppSignal {
 public:
  /// The type of signal
  AppSignalType type;
  /// Generic data for a signal
  void *data;

  AppSignal () {};
  AppSignal (AppSignalType t, void *d) { type = t; data = d;}
};


/** Abstract base class for all objects that needs to recieve signal from the main
 *  app */
class SignalListener {
 public:
  /** Constructor.
   *  This function registers the clip user with the main application */
  SignalListener ();

  /** Destructor.
   *  This function removes this object from the applications list of clip
   *  users */
  virtual ~SignalListener ();

  /** Notify this clip user that a clip has changed in another part of the application.*/
  virtual void Signal (AppSignal *signal) = NULL;
};

#endif




