/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __CLIP_H__
#define __CLIP_H__

#include <memory.h> // get NULL
#include "xml.h"    // get Save & load functionality

/** This is the generic clip.
 *  It has functionality that is used by most data objects. */
class Clip { 

  /** The reference counter in the project. 
   *  NOTE: This is <i>not</i> a reference counter in how many objects uses this clip, but
   *  how many times the clip is used in a project. */
  int refCount;

  /// This clips name
  char  *name;

 public:
  Clip () {refCount = 0; name = NULL;}
  ~Clip ();
  /// Save this clip into a token.
  virtual void Save (XmlToken *dest);
  /** Load this clip from a token.
   *  This function assumes that a clip object of the correct subclass has already been
   *  Allocated. */
  virtual void Load (XmlToken *sec);

  /** Set this clips name 
   *  @param newName the new name of the clip.   */
  virtual void SetName (char *newName);

  /** Get this clip's name 
   *  @return the name of the clip    */
  char *GetName () {return name;}

  /** Return a textual description of this clip type */
  virtual char *GetTypeName () = NULL;

  /// Add 1 to the reference count of this clip.
  void AddRef ();

  /** Remove a reference count to this clip. If the reference count
   *  reaches zero, the object is destroyed */
  void RemoveRef ();

};

inline void Clip::AddRef () {refCount++;}
inline void Clip::RemoveRef () {if (! (--refCount)) delete (this);}
#endif
