/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "clipinfo.h"
#include "application.h"
#include "misc.h"

ClipInfo::ClipInfo (MediaClip *src, GtkWidget *parent, int fps) {
  if (fps == -1)
    this->fps = app->GetProject()->GetFPS();
  else
    this->fps = fps;

  BuildWidgets (parent);
  SetSource (src);
  if(GTK_IS_CONTAINER(parent)) gtk_container_add(GTK_CONTAINER(parent), table);
  else gtk_box_pack_start(GTK_BOX(parent), table, TRUE, TRUE, 0);

  gtk_widget_show (table);
}

ClipInfo::~ClipInfo () {
  gtk_widget_destroy (table);
}

void ClipInfo::Update () {
  gtk_label_set_text (GTK_LABEL(location), src->GetFileLocation ());
  gtk_label_set_text (GTK_LABEL(name), src->GetName ());
  switch (src->GetCacheStatus ()) {
  case 0:
    gtk_label_set_text (GTK_LABEL(cached), "yes");
    break;
  case 1:
    gtk_label_set_text (GTK_LABEL(cached), "no");
    break;
  case 2:
    gtk_label_set_text (GTK_LABEL(cached), "N/A");
    break;
  }

  char *temp = BuildLengthString (src->GetLength (fps), fps);
  gtk_label_set_text (GTK_LABEL(duration), temp);
  delete temp;

  temp = BuildLengthString (src->GetOutPoint (fps) - src->GetInPoint (fps), fps);
  gtk_label_set_text (GTK_LABEL(outin), temp);
  delete temp;

  gtk_label_set_text (GTK_LABEL(type), src->GetTypeName());
}

void ClipInfo::Signal (AppSignal *signal) {
  if (src == signal->data)
    Update ();
}

void ClipInfo::SetSource (MediaClip *newSource) {
  if (src == newSource)
    return;

  src = newSource;
  Update ();
}

void ClipInfo::BuildWidgets (GtkWidget *parent) {
  GtkTooltips *tooltips;
  GtkWidget *temp;

  tooltips = gtk_tooltips_new ();
  table = gtk_table_new (6, 2, FALSE);
  gtk_widget_set_usize (table, -1, -1);

  temp = gtk_label_new ("Location:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 4, 5,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);  gtk_tooltips_set_tip (tooltips, temp, "Shows the clip location", NULL);
  gtk_misc_set_alignment (GTK_MISC (temp), 0.0, 0.5);

  temp = gtk_label_new ("Duration:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_tooltips_set_tip (tooltips, temp, "The duration of the clip", NULL);
  gtk_misc_set_alignment (GTK_MISC (temp), 0.0, 0.5);

  temp = gtk_label_new ("Name:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_tooltips_set_tip (tooltips, temp, "The name of the clip (surprise)", NULL);
  gtk_label_set_justify (GTK_LABEL (temp), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (temp), 0.0, 0.5);

  temp = gtk_label_new ("Cached:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 5, 6,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_tooltips_set_tip (tooltips, temp, "Shows wheter this clip is in the cache", NULL);
  gtk_label_set_justify (GTK_LABEL (temp), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (temp), 0, 0.5);

  temp = gtk_label_new ("OUT-IN:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 3, 4,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (temp), 0, 0.5);

  temp = gtk_label_new ("Type:");
  gtk_widget_show (temp);
  gtk_table_attach (GTK_TABLE (table), temp, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_label_set_justify (GTK_LABEL (temp), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (temp), 0, 0.5);

  name = gtk_label_new ("");
  gtk_widget_show (name);
  gtk_table_attach (GTK_TABLE (table), name, 1, 2, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (name), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (name), 8, 0);

  cached = gtk_label_new ("");
  gtk_widget_show (cached);
  gtk_table_attach (GTK_TABLE (table), cached, 1, 2, 5, 6,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (cached), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (cached), 8, 0);

  location = gtk_label_new ("");
  gtk_widget_show (location);
  gtk_table_attach (GTK_TABLE (table), location, 1, 2, 4, 5,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (location), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (location), 8, 0);

  duration = gtk_label_new ("");
  gtk_widget_show (duration);
  gtk_table_attach (GTK_TABLE (table), duration, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (duration), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (duration), 8, 0);

  outin = gtk_label_new ("");
  gtk_widget_show (outin);
  gtk_table_attach (GTK_TABLE (table), outin, 1, 2, 3, 4,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (outin), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (outin), 8, 0);

  type = gtk_label_new ("");
  gtk_widget_show (type);
  gtk_table_attach (GTK_TABLE (table), type, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (type), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (type), 8, 0);

}
