/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __CLIPINFO_H__
#define __CLIPINFO_H__

#include "appsignal.h"
#include "mclip.h"
#include <gtk/gtk.h>

/** The class used for showing textual clip info.
 *  This class makes a series of widgets that display clip info (length, FPS, etc.) */
class ClipInfo : public SignalListener {
  /// The source clip
  MediaClip *src;
  /// The clip info widgets
  GtkWidget *table, *name, *cached, *location, *duration, *outin, *type;
  /// How many frames/sec to use in time calculations
  int fps;

  /// Build the clip info widgets
  void BuildWidgets (GtkWidget *parent);
  /// Update the clip info
  void Update ();
 public:
  /// Build a new clip info panel
  ClipInfo (MediaClip *src, GtkWidget *parent, int fps = -1);
  /// Destroy this panel
  virtual ~ClipInfo ();
  /// Handle a clipchanged signal from main app
  virtual void Signal (AppSignal *signal); 
  /// Set the clip that the viewer displays info from
  void SetSource (MediaClip *newSource);
};

#endif
