/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */
#include "clippanel.h"
#include "projectwindow.h"

// Get all clip... icon files
#include "pixmaps/clipstart.xpm"
#include "pixmaps/clipin.xpm"
#include "pixmaps/clipplay.xpm"
#include "pixmaps/clippause.xpm"
#include "pixmaps/clipstop.xpm"
#include "pixmaps/clipout.xpm"
#include "pixmaps/clipend.xpm"
#include "pixmaps/clipsetin.xpm"
#include "pixmaps/clipsetout.xpm"
#include "pixmaps/clipclearinout.xpm"

ClipPanel::ClipPanel (MediaClip *srcClip, GtkWidget *parent) {
  src = srcClip;
  BuildWidget (parent);
  viewer->Invalidate ();
}

ClipPanel::~ClipPanel () {
  delete viewer;
  delete info;
}

void ClipPanel::SetSource (MediaClip *newSource) {
  if (newSource != src) {
    src = newSource;
    viewer->SetSource (newSource);
    info->SetSource (newSource);
  }
}

void ClipPanel::BuildWidget (GtkWidget *parent) {
  GtkWidget *clipViewerHandle = gtk_handle_box_new ();
  gtk_box_pack_start (GTK_BOX (parent), clipViewerHandle, FALSE, FALSE, 0);

  widget = gtk_hbox_new (FALSE, 4);
  gtk_container_add (GTK_CONTAINER (clipViewerHandle), widget);
  gtk_widget_show (widget);
  gtk_widget_set_usize (widget, -1, 150);
  gtk_container_border_width (GTK_CONTAINER (widget), 4);
  
  viewer = new ClipViewer (src, widget, 120,90, 
			   (ClipViewerStyle) (SIMPLE | TIMER | FLAT_BUTTONS));
  
  GtkWidget *temp = gtk_vseparator_new ();
  gtk_widget_show (temp);
  gtk_box_pack_start (GTK_BOX (widget), temp, FALSE, FALSE, 2);
  
  temp = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (temp);
  gtk_box_pack_start (GTK_BOX (widget), temp, TRUE, TRUE, 0);
  
  info = new ClipInfo (src, temp);
  
  GtkWidget *hbuttonbox4 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox4);
  gtk_box_pack_start (GTK_BOX (temp), hbuttonbox4, FALSE, FALSE, 0);
  gtk_widget_set_usize (hbuttonbox4, -1, 33);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox4), GTK_BUTTONBOX_SPREAD);
  gtk_button_box_set_child_size (GTK_BUTTON_BOX (hbuttonbox4), 85, 19);
  gtk_button_box_set_child_ipadding (GTK_BUTTON_BOX (hbuttonbox4), 0, 0);
  
  GtkWidget *EditClip = gtk_button_new_with_label ("Edit clip");
  gtk_widget_show (EditClip);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), EditClip);
  gtk_signal_connect (GTK_OBJECT (EditClip), "clicked",
		      GTK_SIGNAL_FUNC (ProjectWindow::on_EditClip_clicked),
		      this);
  
  GtkWidget *DeleteClip = gtk_button_new_with_label ("Delete Clip");
  gtk_widget_show (DeleteClip);
  gtk_container_add (GTK_CONTAINER (hbuttonbox4), DeleteClip);
  gtk_signal_connect (GTK_OBJECT (DeleteClip), "clicked",
		      GTK_SIGNAL_FUNC (ProjectWindow::on_DeleteClip_clicked),
		      this);
  gtk_widget_show (clipViewerHandle);
};


