/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "cliptimer.h"
#include "application.h"
void ClipTimer::Build () {
  bg = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (),
			      "width_pixels", 0, NULL);
  gtk_signal_connect (GTK_OBJECT (bg), "event",
		      (GtkSignalFunc) _ItemEvent,
		      this);
  inOutTime = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
			       "width_pixels", 0, NULL);
  gtk_signal_connect (GTK_OBJECT (inOutTime), "event",
		      (GtkSignalFunc) _ItemEvent,
		      this);
  in1 = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
			       "width_pixels", 0, NULL);
  in2 = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
			       "width_pixels", 0, NULL);
  out1 = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
				"width_pixels", 0, NULL);
  out2 = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
				"width_pixels", 0, NULL);

  frameIndicator = FALSE;

  gnome_canvas_item_raise_to_top (in1);
  gnome_canvas_item_raise_to_top (in2);
  gnome_canvas_item_raise_to_top (out1);
  gnome_canvas_item_raise_to_top (out2);
  gnome_canvas_item_lower_to_bottom (bg);
  Regen ();
}

gint ClipTimer::_ItemEvent (GnomeCanvasItem *item, GdkEvent *event, 
			    gpointer data) {
  return ((ClipTimer *)data)->ItemEvent (event);
}

gint ClipTimer::ItemEvent (GdkEvent *event) {
  switch (event->type) {
  case GDK_BUTTON_PRESS:
    if (event->button.button == 1) {
      src->SetCurrentFrame ((int)(event->button.x / scale), src->GetFps());
      return TRUE;
    }
  case GDK_MOTION_NOTIFY:
    if (event->button.state & GDK_BUTTON1_MASK) {
      src->SetCurrentFrame ((int)(event->button.x / scale), src->GetFps());
      return TRUE;
    }
    break;
  }
  return FALSE;
}


void ClipTimer::Regen () {
  scale = (float) width / (float)(src->GetLength(app->GetProject()->GetFPS ()));

  double in = (double)(src->GetInPoint(app->GetProject()->GetFPS ())) * scale;
  double out = ((double)(src->GetOutPoint(app->GetProject()->GetFPS ()) + 1) * scale) - 1;

  gnome_canvas_item_set (inOutTime, 
			 "x1", (double) in, "y1", (double) 0, 
			 "x2", (double) out - 1, "y2", (double) height,
			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeInOut,
			 NULL);
  gnome_canvas_item_set (bg, 
			 "x1", (double) 0, "y1", (double) 0, 
			 "x2", (double) width-1, "y2", (double) height,
       			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeBG,
			 NULL);
  gnome_canvas_item_set (in1, 
			 "x1", (double) in-1, "y1", (double) 0,
			 "x2", (double) in, "y2", (double) (height),
			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeMarker,
			 NULL);
  gnome_canvas_item_set (in2, 
			 "x1", (double) (in-3), "y1", (double) ((height >> 1) - 1),
			 "x2", (double) in-1, "y2", (double) ((height >> 1) + 1),
			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeMarker,
			 NULL);
  gnome_canvas_item_set (out1,
			 "x1", (double) out, "y1", (double) 0,
			 "x2", (double) out+1, "y2", (double) (height),
			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeMarker,
			 NULL);
  gnome_canvas_item_set (out2,
			 "x1", (double) out, "y1", (double) ((height >> 1) - 1),
			 "x2", (double) (out+2), "y2", (double) ((height >> 1) + 1),
			 "fill_color_rgba", (guint) app->GetPrefs()->clipTimeMarker,
			 NULL);

    if (frameIndicator)
      gnome_canvas_item_set (indicator,
	    "x1", (double) src->GetCurrentFrame(src->GetFps()) * scale, 
	    "y1", (double) -1.0,
	    "x2", (double) src->GetCurrentFrame(src->GetFps()) * scale + scale, 
	    "y2", (double) (height + 1),
	    "fill_color_rgba", (guint) app->GetPrefs()->timeLineActiveFrame,
	    NULL);
}

// TODO: Get width & height from parent
ClipTimer::ClipTimer (GnomeCanvasGroup *parent, MediaClip *clip) {
  width = 100;
  height = 12;

  src = clip;
  root = parent;

  cb = NULL;

  Build ();
}

ClipTimer::ClipTimer (GnomeCanvasGroup *parent, MediaClip *clip, int w, int h) {
  width = w;
  height = h;

  src = clip;
  root = parent;

  cb = NULL;

  Build ();
}

void ClipTimer::SetClickCallback (GtkSignalFunc callback, gpointer user_data) {
  cb = callback;
  uData = user_data;
}

void ClipTimer::SetSize (int newWidth, int newHeight) {
  width = newWidth;
  height = newHeight;
  Regen ();
}

void ClipTimer::Signal (AppSignal *signal) {
  switch (signal->type) {
  case APP_SETTINGS:
    Regen ();
    break;
  case CLIP_ACTIVE_FRAME_CHANGED:
    if (frameIndicator)
      gnome_canvas_item_set (indicator,
			     "x1", (double) src->GetCurrentFrame(src->GetFps()) * scale, "y1", (double) -1.0,
			     "x2", (double) src->GetCurrentFrame(src->GetFps()) * scale + scale, "y2", (double) (height + 1),
			     "fill_color_rgba", (guint) app->GetPrefs()->timeLineActiveFrame,
			     NULL);
    break;
  case CLIP_CHANGED:
    if (src->Uses ((MediaClip*) signal->data))
      Regen ();
  }
}

void ClipTimer::SetSource (MediaClip *newSource) {
  if (src == newSource)
    return;

  src = newSource;
  
  Regen ();
}

void ClipTimer::SetFrameIndicatorVisible (gboolean visible) {
  if (visible == frameIndicator)
    return;

  if (visible) {
    indicator = gnome_canvas_item_new (root, gnome_canvas_rect_get_type (), 
				       "x1", (double) src->GetCurrentFrame(src->GetFps()) * scale, 
				       "y1", (double) -1.0,
				       "x2", (double) src->GetCurrentFrame(src->GetFps()) * scale + scale, 
				       "y2", (double) (height + 1),
				       "fill_color_rgba", (guint) app->GetPrefs()->timeLineActiveFrame,
				       "width_pixels", 1,
				       "outline_color", "black",
				       NULL);
  } else 
    gtk_object_destroy (GTK_OBJECT(indicator));

  frameIndicator = visible;
};




