/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __CLIPTIMER_H__
#define __CLIPTIMER_H__

#include "appsignal.h"
#include "mclip.h"

#include <gnome.h>

/** A clip timing info 
 *  This class makes a timeline object that shows a clip's in & out points. 
 *  It is used by the clip info panel & timeline objects. */
class ClipTimer : public SignalListener {
  /// The size (in pixels) of the clip widget.
  int width, height;
  /// How many pixels to user for each frame.
  float scale;
  /// The source Media Clip
  MediaClip *src;
  /// The container for all other canvas items.
  GnomeCanvasGroup *root;
  /// The backround solid color.
  GnomeCanvasItem *bg;    
  /// The rect specifying the distance between IN & OUT points.
  GnomeCanvasItem *inOutTime;
  /// The rects used to show the IN point.
  GnomeCanvasItem *in1, *in2;
  /// The rects used to show the OUT point.
  GnomeCanvasItem *out1, *out2;
  /// The active frame indicator
  GnomeCanvasItem *indicator;
  /// Is the indicator visible ?
  gboolean frameIndicator;
  
  /// Info for callbacks.
  GtkSignalFunc cb;
  gpointer      uData;

  /// Build all GnomeCanvas items.
  void Build ();
  /// recalculate all coordinates.
  void Regen ();

  /// click callback
  static gint _ItemEvent (GnomeCanvasItem *item, GdkEvent *event, 
			  gpointer data);

  /// Click callback - sets current frame
  gint ItemEvent (GdkEvent *event);

 public:
  /** Create a new clip timing info inside a gnome Canvas.
   *  It will fill it's parent area.
   *  @param parent the group canvas item for showing the info in
   *  @param clip the clip to get IN, OUT & length from. */
  ClipTimer (GnomeCanvasGroup *parent, MediaClip *clip);
  /** Create a new clip timing info inside a gnome Canvas.
   *  It will fill it's parent area.
   *  @param parent the group canvas item for showing the info in
   *  @param clip the clip to get IN, OUT & length from.
   *  @param w the width of the timeline
   *  @param h the height of the timeline */
  ClipTimer (GnomeCanvasGroup *parent, MediaClip *clip, int w, int h);
  /// Sets whether the active frame indicator should be visible
  void SetFrameIndicatorVisible (gboolean visible);
  /// Set the callback function for whenever the user clicks in the timeline
  void SetClickCallback (GtkSignalFunc callback, gpointer user_data);
  /// Set the new size of the canvas item
  void SetSize (int newWidth, int newHeight);
  /// Set a new source clip that this clip info gets its data from.
  void SetSource (MediaClip *newSource);
  /// Handle a clipchanged signal from main app
  virtual void Signal (AppSignal *signal);
};

#endif
