/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <gtk/gtk.h>
#include "clipview.h"
#include "playbuttons.h"
#include "application.h"

PlayButtonsCallback ClipViewer::cb = {
  (GtkSignalFunc)_GoStart,
  (GtkSignalFunc)_GoEnd,
  (GtkSignalFunc)_Stop,
  (GtkSignalFunc)_Play,
  (GtkSignalFunc)_PlayInOut,
  (GtkSignalFunc)_Pause,
  (GtkSignalFunc)_GoForward,
  (GtkSignalFunc)_GoBack,
  (GtkSignalFunc)_SetIn,
  (GtkSignalFunc)_SetOut,
  (GtkSignalFunc)_ClearInOut
};

ClipViewer::ClipViewer (MediaClip *srcClip, GtkWidget *parent, int w, int h, 
			ClipViewerStyle buttons) {
  gdk_rgb_init ();
  anim = NULL;
  src = srcClip;
  MakeWidget (parent, w, h, buttons);
  anim->SetSource (srcClip);

  SetPosition (src->GetInPoint (app->GetProject ()->GetFPS ()));
}

ClipViewer::ClipViewer (MediaClip *srcClip, GtkWidget *parent, Animation *dest,
			ClipViewerStyle buttons) {
  gdk_rgb_init ();
  anim = dest;
  src = srcClip;
  MakeWidget (parent, 0, 0, buttons);
  anim->SetSource (srcClip);

  SetPosition (src->GetInPoint (app->GetProject ()->GetFPS ()));
  
}

void ClipViewer::SetSource (MediaClip *newSource) {
  if (newSource == src)
    return;

  src = newSource;

  clipTimer->SetSource (newSource);
  anim->SetSource (src);
}

void ClipViewer::SetPosition (int newPosition) {
  src->SetCurrentFrame (newPosition, app->GetProject ()->GetFPS ());
  Invalidate ();
}

void ClipViewer::Invalidate () {
  anim->ShowFrame (src->GetCurrentFrame (app->GetProject ()->GetFPS ()));
}

void ClipViewer::MakeWidget (GtkWidget *parent, int width, int height, 
			     ClipViewerStyle buttons) {
  GtkWidget *table15;

  // Make container if needed
  if (!anim || (buttons & TIMER)) {
    table15 = gtk_vbox_new (false, 4);
    gtk_widget_show (table15);
    gtk_box_pack_start (GTK_BOX (parent), table15, FALSE, FALSE, 0);
  } else
    table15 = parent;   // If not, we just insert into parent

  if (!anim) {
    GtkWidget *table16 = gtk_hbox_new (false, 0);
    gtk_widget_show (table16);
    gtk_box_pack_start (GTK_BOX (table15), table16, FALSE, FALSE, 0);

    anim = new GtkAnimation (table16, width, height);
  }

  if (buttons & TIMER) {
    GtkWidget *viewport24;
    viewport24 = gtk_viewport_new (NULL, NULL);
    gtk_widget_show (viewport24);
    gtk_widget_set_usize (viewport24, -1, 16);
    gtk_box_pack_start (GTK_BOX (table15), viewport24, FALSE, FALSE, 0);

    timeLineWidget = gnome_canvas_new_aa ();
    gtk_widget_show (timeLineWidget);
    gtk_container_add (GTK_CONTAINER (viewport24), timeLineWidget);
    gtk_widget_set_usize (timeLineWidget, -1, 12);  
    clipTimer = new ClipTimer 
      (gnome_canvas_root (GNOME_CANVAS(timeLineWidget)), src);
    clipTimer->SetFrameIndicatorVisible (TRUE);

    gtk_signal_connect (GTK_OBJECT (timeLineWidget), "expose_event",
			GTK_SIGNAL_FUNC (_Timeline_configure),
			this);
  }

  toolbar = PlayButtons::Build (&cb, table15, this, buttons, GTK_TOOLBAR_ICONS);
  if (!GTK_IS_TOOLBAR (table15)) {
    gtk_widget_show (toolbar);
    gtk_box_pack_start (GTK_BOX (table15), toolbar, FALSE, FALSE, 0);
    gtk_widget_set_usize (toolbar, -1, 20);
  }
}


void ClipViewer::_GoStart (GtkWidget *widget, ClipViewer *cv) {
  cv->GoStart ();
}
void ClipViewer::_GoBack (GtkWidget *widget, ClipViewer *cv) {
  cv->GoBack ();
}
void ClipViewer::_Stop (GtkWidget *widget, ClipViewer *cv) {
  cv->Stop ();
}
void ClipViewer::_Play (GtkWidget *widget, ClipViewer *cv) {
  cv->Play ();
}
void ClipViewer::_PlayInOut (GtkWidget *widget, ClipViewer *cv) {
  cv->PlayInOut ();
}
void ClipViewer::_Pause (GtkWidget *widget, ClipViewer *cv) {
  cv->Pause ();
}
void ClipViewer::_GoForward (GtkWidget *widget, ClipViewer *cv) {
  cv->GoForward ();
}
void ClipViewer::_GoEnd (GtkWidget *widget, ClipViewer *cv) {
  cv->GoEnd ();
}
void ClipViewer::_Timeline_configure (GnomeCanvas *widget, GdkEventExpose *event, ClipViewer *cv) {
  cv->ConfigureTimeline ();
}

void ClipViewer::_SetIn (GtkWidget *widget, ClipViewer *cv) {}
void ClipViewer::_SetOut (GtkWidget *widget, ClipViewer *cv) {}
void ClipViewer::_ClearInOut (GtkWidget *widget, ClipViewer *cv) {}

void ClipViewer::Signal (AppSignal *signal) {
  switch (signal->type) {
  case CLIP_CHANGED:
    if (src->Uses ((MediaClip*) signal->data))
      Invalidate ();
    break;
  case CLIP_ACTIVE_FRAME_CHANGED:
    if (src == signal->data)
      Invalidate ();
    break;
  }
}

/// Go 1 frame forward.
void ClipViewer::GoForward () {
  if (src->GetCurrentFrame (app->GetProject ()->GetFPS ()) < src->GetLength (app->GetProject ()->GetFPS ()))
    SetPosition (src->GetCurrentFrame (app->GetProject ()->GetFPS ()) + 1);
}
/// Go 1 frame backwards.
void ClipViewer::GoBack () {
  if (src->GetCurrentFrame (app->GetProject ()->GetFPS ()))
    SetPosition (src->GetCurrentFrame (app->GetProject ()->GetFPS ()) - 1);
}
/// Go to IN point (or start frame)
void ClipViewer::GoStart () {
  SetPosition (src->GetInPoint (app->GetProject ()->GetFPS ()));
}

/// Go to OUT point (or end frame)
void ClipViewer::GoEnd () {
  SetPosition (src->GetOutPoint (app->GetProject ()->GetFPS ()));
}
/// Start playback
void ClipViewer::Play () {
  app->PlayClip (src, src->GetCurrentFrame (app->GetProject ()->GetFPS ()), 
		 src->GetLength (app->GetProject ()->GetFPS ())-1);
}
/// Start playback
void ClipViewer::PlayInOut () {
  app->PlayClip (src, src->GetInPoint (app->GetProject ()->GetFPS ()), 
		 src->GetOutPoint (app->GetProject ()->GetFPS ())-1);
}

/// Stop playback
void ClipViewer::Stop () {}
/// Pause playback
void ClipViewer::Pause () {}

void ClipViewer::SetIn () {
  src->SetInPoint (src->GetCurrentFrame (app->GetProject ()->GetFPS ()), 
		   app->GetProject ()->GetFPS ());
}

void ClipViewer::SetOut () {
  src->SetOutPoint (src->GetCurrentFrame (app->GetProject ()->GetFPS ()), 
		    app->GetProject ()->GetFPS ());
}
void ClipViewer::ClearInOut () {}

int ClipViewer::GetPosition () {
  return src->GetCurrentFrame (app->GetProject ()->GetFPS ());
}

void ClipViewer::ConfigureTimeline () {
  printf ("Config\n");
  gnome_canvas_set_scroll_region (GNOME_CANVAS(timeLineWidget), 0.0, 0.0, 
				  (double)(GTK_WIDGET(timeLineWidget))->allocation.width, 
				  (double)(GTK_WIDGET(timeLineWidget))->allocation.height);
  clipTimer->SetSize (GTK_WIDGET(timeLineWidget)->allocation.width, GTK_WIDGET(timeLineWidget)->allocation.height);
  Invalidate ();
}

