/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <errno.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <dirent.h>
#include <sys/types.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <gnome.h>
#include <gmodule.h>

#include "codeccentral.h"

struct Codec **CodecCentral::codec;

int numberOfCodecs;

struct Codec * CodecCentral::GetCodecWithID (char *codecID) {
   for (int i=0; i< numberOfCodecs; i++) {
     if (!strcmp(codec[i]->id, codecID)) {
	    return codec[i];
	  }
	}
	return NULL;
}

GtkWidget *CodecCentral::GetVideoCodecSelectionWidget(GtkWidget *parent, char *defaultCodecID, void (*signalfunc)(GtkWidget *entry, Codec *codec))
{
  GtkWidget *hbox2;
  GtkWidget *label4;
  GtkWidget *optionmenu1;
  GtkWidget *optionmenu1_menu;
  GtkWidget *glade_menuitem;
  int codecIdx = 0;


  hbox2 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox2);
  gtk_object_set_data_full (GTK_OBJECT (parent), "hbox2", hbox2,
		                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox2);

  label4 = gtk_label_new (_("compression CODEC :"));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (parent), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox2), label4, FALSE, FALSE, 0);

  optionmenu1 = gtk_option_menu_new ();
  gtk_widget_ref (optionmenu1);
  gtk_object_set_data_full (GTK_OBJECT (parent), "optionmenu1", optionmenu1,
	                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (optionmenu1);
  gtk_box_pack_start (GTK_BOX (hbox2), optionmenu1, TRUE, TRUE, 0);

  optionmenu1_menu = gtk_menu_new ();

  for (int i=0; i< numberOfCodecs; i++) {
    if (!strcmp(codec[i]->id, defaultCodecID)) {
	   codecIdx = i;
	 }
    glade_menuitem = gtk_menu_item_new_with_label (_(codec[i]->name));
    gtk_widget_show (glade_menuitem);
    gtk_menu_append (GTK_MENU (optionmenu1_menu), glade_menuitem);
    gtk_signal_connect(GTK_OBJECT(glade_menuitem), "activate",
                        GTK_SIGNAL_FUNC(signalfunc), codec[i]);
  }

  gtk_option_menu_set_menu (GTK_OPTION_MENU (optionmenu1), optionmenu1_menu);

  gtk_option_menu_set_history(GTK_OPTION_MENU(optionmenu1), codecIdx);


  return hbox2;
  
}

int CodecCentral::InitCodecs () {

  int mode = 0;
  char funcname[128];
  char fileName[128];
  void *(*op)();
  struct dirent **namelist;
  int direntries;
  struct GModule *gmodule;
  Codec *currentCodec;

  if (!g_module_supported()) {
    printf("Modules not supported...\n");
	 return 0;
  }

  direntries = scandir( PLUGINDIR "codecs/", &namelist, NULL, NULL);
  codec = (Codec *[direntries]) new (Codec *)[direntries];

  sprintf (funcname, "GetCodecInfo__Fv");

  numberOfCodecs = 0;

  for (int i=0; i<direntries; i++) {
    sprintf (fileName, PLUGINDIR "codecs/%s",namelist[i]->d_name);

    gmodule = g_module_open (fileName, (GModuleFlags) 0);
    if (gmodule) {
      if (g_module_symbol (gmodule, funcname, (gpointer *) &op)) {
			
        currentCodec = (Codec *)op();
		  if (!GetCodecWithID(currentCodec->id)) {
          codec[numberOfCodecs] = (Codec *)op();
	
          printf("codec %i: %s initialized ID=%s\n", numberOfCodecs, codec[numberOfCodecs]->name, codec[numberOfCodecs]->id);

		    numberOfCodecs++;
		  }
		}
      else printf("error, not a CODEC\n");
    }
  }

  return 1;
}

