
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "assert.h"
#include "triframe.h"

TriFrame::TriFrame(int w, int h, int smallestx, int pixelSize) {
	int i,j, k, l;
	int smallesty;
	int count=0;
	unsigned char *content;
	int pixels;

	height = h;
	width = w;

   this->pixelSize = pixelSize;

   numbuffers = 1;
	for (j=80; j< width; j*=2) {
		numbuffers++;
	}
   smallesty = (int)(((float)height)/((float)width/(float)smallestx));

	buffer = new unsigned char[width*height];

	offsets = new unsigned long[numbuffers*sizeof(long)];
	offsetsbuf = new unsigned long[numbuffers*sizeof(long)];
	lengths = new unsigned long[numbuffers*sizeof(long)];
	heights = new unsigned long[numbuffers*sizeof(long)];
	widths = new unsigned long[numbuffers*sizeof(long)];

	lengths[numbuffers-1]= width*height;
	memset(buffer, numbuffers-1 , width*height);

	widths[numbuffers-1] = width;
	heights[numbuffers-1] = height;

	for (k=1, l=1; k<numbuffers; k++, l*=2) {
		widths[k-1] = smallestx*l;
		heights[k-1] = smallesty*l;
		lengths[k-1] = 0;
		for (j=0; j<(smallesty*l);j++) {
			pixels=0;
			for (i=0; i<(smallestx*l);i++) {
				content = (buffer+((width*i)/(smallestx*l))+width*((height*j)/(smallesty*l)));
				assert (content < buffer+width*height);
				if (*content > k-1) {
					*content = k-1;
					lengths[k-1]++;
					lengths[numbuffers-1]--;
					pixels++;
				}
			}
		}
	}
	offsets[0] = 0;
	for (i=1; i<numbuffers; i++) {
		 offsets[i] = offsets[i-1]+lengths[i-1];
	}
	
}

TriFrame::~TriFrame() {
  if (buffer)
    delete buffer;
  if (offsets)
    delete offsets;
  if (offsetsbuf)
    delete offsetsbuf;
  if (lengths)
    delete lengths;
  if (heights)
    delete heights;
  if (widths)
    delete widths;
}

int  TriFrame::GetWidth(int scale)
{
	return widths[scale];
}

int  TriFrame::GetHeight(int scale)
{
	return heights[scale];
}

int  TriFrame::GetSize(int scale)
{
	return widths[scale]*heights[scale];
}


void TriFrame::MakeFrame(unsigned char *datain, unsigned char *dataout)
{
   switch (pixelSize) {
	  case 1:
	    break;
	  case 2:
	    MakeFrame16(datain, dataout);
	    break;
	  case 3:
	    MakeFrame24(datain, dataout);
	    break;
	  case 4:
	    MakeFrame32(datain, dataout);
	    break;
	  default:
	    MakeFrame32(datain, dataout);
	    break;
	}
}

void TriFrame::RestoreFrame(unsigned char *datain, unsigned char *dataout, int scale)
{
	switch (pixelSize) {
	  case 1:
	    break;
	  case 2:
	    RestoreFrame16(datain, dataout, scale);
	    break;
	  case 3:
	    RestoreFrame24(datain, dataout, scale);
	    break;
	  case 4:
	    RestoreFrame32(datain, dataout, scale);
	    break;
	  default:
	    RestoreFrame32(datain, dataout, scale);
	    break;
	}
}

void TriFrame::MakeFrame16(unsigned char *datain, unsigned char *dataout)
{
	int i;
	unsigned short *target=(unsigned short *)dataout;

	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
			*(target+((offsetsbuf[(int)(*(buffer+i))]++))) 
					  = *((unsigned short *)datain+i);
	}

}		  

void TriFrame::RestoreFrame16(unsigned char *datain, unsigned char *dataout, int scale) 
{
	int i;
	unsigned short *target=(unsigned short *)dataout;

	if (scale == -1) scale = numbuffers;
	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
			if (scale >= *(buffer+i)) {
				*(target) = *((unsigned short *)datain+((offsetsbuf[(int)(*(buffer+i))]++)));
				target++;
			}
	}
}

void TriFrame::MakeFrame24(unsigned char *datain, unsigned char *dataout)
{
	int i;
	unsigned char *target=dataout;
	unsigned char *target2;

	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
	      target2 = target+3*((offsetsbuf[(int)(*(buffer+i))]++)); 
			*(target2) = *(datain++);
			*(target2+1) = *(datain++);
			*(target2+2) = *(datain++);
	}

}		  

void TriFrame::RestoreFrame24(unsigned char *datain, unsigned char *dataout, int scale) 
{
	int i;
	unsigned char *target=dataout;
	unsigned char *source2;

	if (scale == -1) scale = numbuffers;
	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
			if (scale >= *(buffer+i)) {
	         source2 = datain+3*((offsetsbuf[(int)(*(buffer+i))]++)); 
			   *(target++) = *(source2);
			   *(target++) = *(source2+1);
			   *(target++) = *(source2+2);
			}
	}

}

void TriFrame::MakeFrame32(unsigned char *datain, unsigned char *dataout)
{
	int i;
	unsigned long *target=(unsigned long *)dataout;

	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
			*(target+((offsetsbuf[(int)(*(buffer+i))]++))) 
					  = *((unsigned long *)datain+i);
	}

}		  

void TriFrame::RestoreFrame32(unsigned char *datain, unsigned char *dataout, int scale) 
{
	int i;
	unsigned long *target=(unsigned long *)dataout;

	if (scale == -1) scale = numbuffers;
	memcpy(offsetsbuf, offsets, numbuffers*sizeof(unsigned long));

	for (i=0; i<height*width; i+=1) {
			if (scale >= *(buffer+i)) {
				*(target) = *((unsigned long *)datain+((offsetsbuf[(int)(*(buffer+i))]++)));
				target++;
			}
	}

}

