/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */
#ifndef __TRIFRAME_H
#define __TRIFRAME_H

/** This is the class to handle the variable image size on disk
 *  <p>Image data is stored like this:
 *  <p>
 *  00 02 20 22    01 03 10 11 12 13 21 23 30 31 32 33
 *  ***********    ***********************************
 *   block 1             block 2
 *  <p>
 *  we can read just a small amount of data to get a small
 *  resolution picture using block 1, wich will be 
 *  reconstructed as:
 *  <p>
 *  00 02 
 *  20 22
 *  <p>
 *  or a full resolution version by reading block 1 and block 2 like:
 *  <p> 
 *  00 01 02 03 
 *  10 11 12 13 
 *  20 21 22 23 
 *  30 31 32 33
 *
 * This algorithm has to make full use of the underlaying harddrive geometry
 * to achieve its full speed. TODO
 **/

class TriFrame {
  // a buffer to hold the lookup data
  unsigned char *buffer;

  // the number of buffers we have. Each buffer will hold a scaled version
  // of the original image
  int numbuffers;

  // the size of the pixels
  int pixelSize;

  // the dimension of the original image and the maximum size of the buffer
  int width;
  int height;

  // the offsets in the source image
  unsigned long *offsets;

  // a temporary copy for speedup
  unsigned long *offsetsbuf;

  // an array with the lenght of each buffer
  unsigned long *lengths;

  // an array with the widths of each buffer
  unsigned long *widths;

  // an array with the heights of each buffer
  unsigned long *heights;

 protected:
 public:
  TriFrame (int width, int height, int minx, int pixelSize);
  virtual ~TriFrame ();

  virtual int GetWidth(int scale);
  virtual int GetHeight(int scale);
  virtual int GetSize(int scale);

  void MakeFrame(unsigned char *, unsigned char *);
  void RestoreFrame(unsigned char *, unsigned char *, int);

  void MakeFrame16(unsigned char *datain, unsigned char *dataout);
  void RestoreFrame16(unsigned char *datain, unsigned char *dataout, int scale);
  void MakeFrame24(unsigned char *datain, unsigned char *dataout);
  void RestoreFrame24(unsigned char *datain, unsigned char *dataout, int scale);
  void MakeFrame32(unsigned char *datain, unsigned char *dataout);
  void RestoreFrame32(unsigned char *datain, unsigned char *dataout, int scale);


};

#endif

