
/*
 * Trinity file layout:
 * 
 * tri_header
 * 
 * {
 * tri_codec_header 
 *  codec_header 
 * } x tri_header.channels
 * 
 * tri_channel_header
 *  channel data
 * tri_channel_header
 *  channel_data
 * ...
 *
 */

/* ----------------------------------------------------------------------- */

#ifndef __TRISTREAMCODEC_H__
#define __TRISTREAMCODEC_H__

#include "codec.h"
#include <stdio.h>
#include <endian.h>

/* to make these files byte order independant */
#if __BYTE_ORDER == __BIG_ENDIAN
#define TRI_SWAP2(a) SWAP2((a))
#define TRI_SWAP4(a) SWAP4((a))
#else
#define TRI_SWAP2(a) (a)
#define TRI_SWAP4(a) (a)
#endif

typedef unsigned int   uint32;
typedef unsigned short uint16;

  // Some strings for the info

  // The structure containing all CODEC info. A pointer to this is sent back to the app.

  /* this is the start of the stream */
  struct TRI_HDR {
    int size;					/* the size of this header */
    uint32 flags;					/* flags for future use */
    uint32 frames;            /* # of frames (all) */
    uint32 channels;
    uint32 bufsize;           /* suggested buffer size */
	 /* add extra data here */
  };

  /* for each channel in the stream there is a codecheader, descibing
   * the codec used for encoding this channel 
   * this header is followed by the codecs header */
  struct TRI_CODEC_HDR {
    uint32 size;					/* the size of this header */
    char   codecID[8];           /* codec identifier */
    uint32 flags;					/* flags for future use */
    uint32 priority;				/* the priority of this channel */
    uint32 codech_size;			/* the size of the codec header */
	 /* add extra data here */
  };

  struct TRI_CHANNEL_HDR { 
    uint32 size;					/* the size of this header */
	 uint32 timestamp;			/* the timestamp for this data */
    uint32 channelnr;			/* the channelnumber */
    uint32 channeldata_size;	/* the size of this the data */
	 /* add extra data here */
  };


Codec *GetCodecInfo();


#endif
