/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <stdio.h>
#include <memory.h>
#include "color.h"

int Color::initialized = 0;

// YCC<->RGB code taken from IJG's jpeglib-v6b
// Copyright (C) 1991-1996, Thomas G. Lane.

#define NO_C            -128
#define CBCR_OFFSET     ((int) 128 << 16)
#define ONE_HALF        ((int) 1 << 15)
#define FIX(x)          ((int) ((x) * (double)(1L<<16) + 0.5))
#define R_Y_OFF         0                       /* offset to R => Y section */
#define G_Y_OFF         (1*256)      /* offset to G => Y section */
#define B_Y_OFF         (2*256)      /* etc. */
#define R_CB_OFF        (3*256)
#define G_CB_OFF        (4*256)
#define B_CB_OFF        (5*256)
#define R_CR_OFF        B_CB_OFF                /* B=>Cb, R=>Cr are the same */
#define G_CR_OFF        (6*256)
#define B_CR_OFF        (7*256)
#define TABLE_SIZE      (8*256)
static int convert_rgb_ycc_tab[TABLE_SIZE];
static int convert_Crrtab[256];
static int convert_Cbbtab[256];
static int convert_Crgtab[256];
static int convert_Cbgtab[256];
static unsigned char convert_range[5*256+128];
static unsigned char *convert_rangetable = NULL;

void Color::Init(){
  if (initialized) return;

  int i, x;

  // YCbCr <-> 24bit conversion
  for (i = 0; i < 256; i++) {
    convert_rgb_ycc_tab[i+R_Y_OFF] = FIX(0.29900) * i;
    convert_rgb_ycc_tab[i+G_Y_OFF] = FIX(0.58700) * i;
    convert_rgb_ycc_tab[i+B_Y_OFF] = FIX(0.11400) * i + ONE_HALF;
    convert_rgb_ycc_tab[i+R_CB_OFF] = (-FIX(0.16874)) * i;
    convert_rgb_ycc_tab[i+G_CB_OFF] = (-FIX(0.33126)) * i;
    convert_rgb_ycc_tab[i+B_CB_OFF] = FIX(0.50000) * i + CBCR_OFFSET + ONE_HALF-1;
    convert_rgb_ycc_tab[i+G_CR_OFF] = (-FIX(0.41869)) * i;
    convert_rgb_ycc_tab[i+B_CR_OFF] = (-FIX(0.08131)) * i;
  }
  for (i = 0, x = -128; i < 256; i++, x++) {
    convert_Crrtab[i] = ((int)(FIX(1.40200) * x + ONE_HALF) >> 16);
    convert_Cbbtab[i] = ((int)(FIX(1.77200) * x + ONE_HALF) >> 16);
    convert_Crgtab[i] = (- FIX(0.71414)) * x;
    convert_Cbgtab[i] = (- FIX(0.34414)) * x + ONE_HALF;
  }
  convert_rangetable = convert_range+256;
  memset(convert_range,0,256);
  for (i = 0; i < 256; i++) convert_rangetable[i] = i;
  memset(convert_rangetable+256,255,384);
  memset(convert_rangetable+256+384,0,384);
  memcpy(convert_rangetable+256+384+384,convert_rangetable,128);

  initialized = 1;
}

void Color::RGB16_to_RGB32(unsigned short *source, unsigned long *dest, int pixels) 
{
	int i;

	for (i=0; i<pixels;i++) {
		*(dest++) = ((*source & 0xF800) << 8) |
						((*source & 0x07E0) << 5) |
						((*source++ & 0x001f) << 3); 
	}
}

void Color::RGB_to_YUV422Pixel(int r, int g, int b, int *y, int *u, int *v) 
{
  //make sure we have the tables
  Init();

  *y = ((convert_rgb_ycc_tab[r+R_Y_OFF] + convert_rgb_ycc_tab[g+G_Y_OFF] + convert_rgb_ycc_tab[b+B_Y_OFF]) >> 16); 
  *u = ((convert_rgb_ycc_tab[r+R_CB_OFF] + convert_rgb_ycc_tab[g+G_CB_OFF] + convert_rgb_ycc_tab[b+B_CB_OFF]) >> 16); 
  *v = ((convert_rgb_ycc_tab[r+R_CR_OFF] + convert_rgb_ycc_tab[g+G_CR_OFF] + convert_rgb_ycc_tab[b+B_CR_OFF]) >> 16);
}

void Color::YUV422_to_RGBPixel(int y, int cb, int cr, int *r, int *g, int *b) 
{
  //make sure we have the tables
  Init();

  *r = convert_rangetable[y + convert_Crrtab[cr]]; \
  *g = convert_rangetable[y + ((int) (convert_Cbgtab[cb] + convert_Crgtab[cr]) >> 16)]; \
  *b = convert_rangetable[y + convert_Cbbtab[cb]];

}


