/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "colorclip.h"
#include "application.h"
#include "misc.h"

#include <gnome.h>

void ColorClip::Add (GtkMenuItem *menuitem, gpointer user_data) {
  char *name = g_strdup ("New color");

  if (!ShowInputBox ("Add solid color clip", "Enter the name of the clip", &name))
    return;
  ColorClip *temp = new ColorClip ();
  temp->SetName (name);

  if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (temp))
    delete temp;   
  else 
    app->GetProjectWindow()->BuildProjectListView ();
}


ColorClip::ColorClip () {
  color = 0;   // full black
}

ColorClip::~ColorClip () {
}

char *ColorClip::GetTypeName () {
  return "Solid color";
}

int ColorClip::Uses (MediaClip *clip) {
  if (clip == this)
    return 1;
  return 0;
}

int ColorClip::Erase () {
  return 0;
}

int ColorClip::GetSize () {
  return 0;
}

char *ColorClip::GetFileLocation () {
  return "N/A";
}

int ColorClip::GetCacheStatus () {
  return 2;
}

void ColorClip::Render (int frame, int fps, FrameBuffer *dest) {
  for (int y = dest->GetHeight ();y--;) 
    for (int x = dest->GetWidth ();x--;)
      dest->SetPixel (x, y, color);
}

void ColorClip::Save (XmlToken *dest) {
  StillClip::Save (dest);

  dest->SetKey ("SolidColor");
  dest->AppendInt ("Red", color >> 16);
  dest->AppendInt ("Green", (color >> 8) & 0xFF);
  dest->AppendInt ("Blue", color & 0xFF);
}

void ColorClip::Load (XmlToken *src) {
  StillClip::Load (src);

  color = (src->GetInt ("red") << 16) +
          (src->GetInt ("green") << 8) +
          src->GetInt ("blue");
}

static void on_colorselection_changed (GtkColorSelection *colSel, ColorClip *clip) {
  double col[4];
  gtk_color_selection_get_color (colSel, col);
  clip->EditChangeColor (GDKtoRGB (col));
}

void ColorClip::EditChangeColor (guint32 newColor) {
  if (color == newColor) 
    return;
  
  color = newColor;

  app->ClipChanged (this);
}

void ColorClip::EditCancelClicked () {
  EditRevertClicked ();
  CloseEditWindow ();
}
void ColorClip::EditRevertClicked () {
  double *gtkCol;
  color = revertColor;

  gtkCol = RGBtoGDK (color);

  gtk_color_selection_set_color (GTK_COLOR_SELECTION(colorSel), gtkCol);
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(colorSel), gtkCol);

  StillClip::EditRevertClicked ();
  app->ClipChanged (this);
}

void ColorClip::BuildEditWidget (GtkNotebook *parent) {
  revertColor = color;

  double *gtkCol;
  
  // Build the edit window
  GtkWidget *label106;

  colorSel = gtk_color_selection_new ();
  gtk_widget_show (colorSel);
  gtk_signal_connect (GTK_OBJECT (colorSel), "color_changed",
                      GTK_SIGNAL_FUNC (on_colorselection_changed),
                      this);

  label106 = gtk_label_new ("Color");
  gtk_widget_show (label106);
  gtk_notebook_append_page (parent, colorSel, label106);

  StillClip::BuildEditWidget (parent);


  // end build window
  gtkCol = RGBtoGDK(color);

  gtk_color_selection_set_color (GTK_COLOR_SELECTION(colorSel), gtkCol);
  gtk_color_selection_set_color (GTK_COLOR_SELECTION(colorSel), gtkCol);
}


