/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "stillclip.h"

/** A solid color.
 *  This clip is just a sold color. It also has an edit window.*/
class ColorClip : public StillClip {
  /// The color of the clip (in 32bit RGB)
  guint32 color;
  /// The color to revert to on Revet and Cancel.
  guint32 revertColor;
  /// Pointer to the color selector
  GtkWidget *colorSel;
 protected:
  /** Edit this clip.
   *  This function opens the standard GTK color selection dialog for editing of the 
   *  color */
  virtual void BuildEditWidget (GtkNotebook *parent);
 public:
  // Create a black color clip
  ColorClip ();
  // Delete this clip & an eventual edit window 
  virtual ~ColorClip ();

  /** Return "Solid color" */
  virtual char *GetTypeName ();

  /** Query if this clip makes use of another clip.
   *  @return 1 if [this clip] = [clip], otherwise 0; */
  virtual int Uses (MediaClip *clip);

  /** Erase the color clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int Erase ();

  /** Get the disk size of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetSize ();

  /** Get the location of this clips file(s)
   *  This function is a stub. It doesn't do anything, and always returns "" */ 
  virtual char *GetFileLocation ();

  /** Get cache status of this clip
   *  This function is a stub. It doesn't do anything, and always returns 2 (N/A) */
  virtual int GetCacheStatus ();

  /** Render this clip's data into the current stream.
   * This function is a stud. It doesnt do anything. */
  virtual void Render (int frame, int fps, FrameBuffer *dest);

  /// Cancel has been pressed in the edit window 
  virtual void EditCancelClicked ();
  /// Revert has been pressed in the edit window 
  virtual void EditRevertClicked ();
  /// The color has been changed in the edit window
  virtual void EditChangeColor (guint32 newColor);
  /// callback from main menu item
  static void Add (GtkMenuItem *menuitem, gpointer user_data);

  /// Save this clip into a token.
  virtual void Save (XmlToken *dest);
  /** Load this clip from a token.
   *  This function assumes that a clip object of the correct subclass has already been
   *  Allocated. */
  virtual void Load (XmlToken *sec);
};

// Entry-level function when 'add color clip' is selected from menus
void on_Add_color_clip_activate (GtkMenuItem *menuitem, gpointer user_data);
