/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "misc.h"
#include "mclip.h"
#include "imagesequence.h"
#include "trisequence.h"
#include "assert.h"
#include "converter.h"
#include "application.h"

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>
#include <gnome.h>

static GtkWidget *frame1;
static GtkWidget *window;
static GtkWidget *radiobutton1;
static ConvertInfo conv;

int xsize;
int ysize;

/// File/import/image seq.
MediaClip *Converter::Convert (MediaClip *source, MediaClip *dest) 
{

  conv.source = (ImageSequenceClip *)source;
  conv.dest = (TriSequenceClip *)dest;
  window = BuildImportWidget(&conv);

  gtk_widget_show(window);

}
void Converter::on_radiobutton1_toggled  (GtkToggleButton *togglebutton,
                                               gpointer         user_data)
{
  gtk_widget_set_sensitive((GtkWidget *)user_data, 
         !(togglebutton->active));
}

void Converter::on_button1_clicked  (GtkButton *button,
                                               gpointer         user_data)
{
  int convertneeded = !GTK_TOGGLE_BUTTON(radiobutton1)->active;
  gtk_widget_destroy(window);

  MediaClip *clipToAdd;
  
  ImageSequenceClip *temp = ((ConvertInfo *)user_data)->source;
  TriSequenceClip *tseq = ((ConvertInfo *)user_data)->dest;

  if (convertneeded) {

    tseq->SetName ("importing clip");
    // TODO make this configurable.
    xsize = 320;
    ysize = 224;
    // we now have the imagesequence
    // we import this into the trisequenceclip 
    FrameBuffer *buf = new FrameBuffer(xsize, ysize, RGB24);
    buf->SetVideoData(new unsigned char[xsize*ysize*3]);

    if (tseq->ConvertFrom(temp, tmpnam(NULL), TRUE, buf)) {
      delete tseq;
      delete temp;
	   return;
    }
    tseq->SetName ("imported clip");
	 clipToAdd = tseq;
  }
  else {
	 clipToAdd = temp;
  }
	    

  printf("adding to project\n");
  if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (clipToAdd)) {
    delete tseq;
    delete temp;   
	 return;
  }
  app->GetProjectWindow()->BuildProjectListView();
  
  printf("import ended\n");

}

void Converter::on_button2_clicked  (GtkButton *togglebutton,
                                               gpointer         user_data)
{
  gtk_widget_destroy(window);
}

GtkWidget *Converter::BuildImportWidget (ConvertInfo *convert) {

  GtkWidget *importWindow;
  GtkWidget *vbox2;
  GtkWidget *notebook1;
  GtkWidget *empty_notebook_page;
  GtkWidget *label1;
  GtkWidget *label4;
  GtkWidget *combo1;
  GtkWidget *vbox3;
  GtkWidget *table3;
  GSList *table3_group = NULL;
  GtkWidget *radiobutton2;
  GtkWidget *fileentry2;
  GtkWidget *label2;
  GtkWidget *label3;
  GtkWidget *hbuttonbox1;
  GtkWidget *button1;
  GtkWidget *button2;

  GtkWidget *sourceWidget;
  GtkWidget *destWidget;
  GList *combo1_items;

  importWindow = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_widget_ref (importWindow);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "importWindow", importWindow,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_window_set_title (GTK_WINDOW (importWindow), _("Import"));

  vbox2 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox2);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "vbox2", vbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox2);
  gtk_container_add (GTK_CONTAINER (importWindow), vbox2);

  notebook1 = gtk_notebook_new ();
  gtk_widget_ref (notebook1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "notebook1", notebook1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (notebook1);
  gtk_box_pack_start (GTK_BOX (vbox2), notebook1, TRUE, TRUE, 0);

  // configure the source 
  sourceWidget = convert->source->BuildImportWidget(importWindow);
  gtk_container_add (GTK_CONTAINER (notebook1), sourceWidget);
  
  label1 = gtk_label_new (_("import"));
  gtk_widget_ref (label1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "label1", label1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label1);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 0), label1);

  vbox3 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox3);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "vbox3", vbox3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox3);
  gtk_container_add (GTK_CONTAINER (notebook1), vbox3);

  table3 = gtk_table_new (2, 2, FALSE);
  gtk_widget_ref (table3);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "table3", table3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table3);
  gtk_box_pack_start (GTK_BOX (vbox3), table3, FALSE, FALSE, 0);

  radiobutton1 = gtk_radio_button_new_with_label (table3_group, _("Work on Images"));
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton1));
  gtk_widget_ref (radiobutton1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "radiobutton1", radiobutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton1);
  gtk_table_attach (GTK_TABLE (table3), radiobutton1, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (radiobutton1), 4);

  radiobutton2 = gtk_radio_button_new_with_label (table3_group, _("Convert to stream"));
  table3_group = gtk_radio_button_group (GTK_RADIO_BUTTON (radiobutton2));
  gtk_widget_ref (radiobutton2);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "radiobutton2", radiobutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (radiobutton2);
  gtk_table_attach (GTK_TABLE (table3), radiobutton2, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_container_set_border_width (GTK_CONTAINER (radiobutton2), 5);

/*
  fileentry2 = gnome_file_entry_new (NULL, NULL);
  gtk_widget_ref (fileentry2);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "fileentry2", fileentry2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (fileentry2);
  gtk_table_attach (GTK_TABLE (table3), fileentry2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
                    (GtkAttachOptions) (0), 7, 0);
						  */


  label4 = gtk_label_new (_("Image size :"));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (vbox3), label4, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label4), 4, 0);

  combo1 = gtk_combo_new ();
  gtk_widget_ref (combo1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "combo1", combo1,
                            (GtkDestroyNotify) gtk_widget_unref);
  combo1_items = NULL;
  gtk_widget_show (combo1);
  gtk_box_pack_start (GTK_BOX (vbox3), combo1, TRUE, TRUE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (combo1), 4);
  combo1_items = g_list_append (combo1_items, _("Original size"));
  combo1_items = g_list_append (combo1_items, _("Half size"));
  combo1_items = g_list_append (combo1_items, _("Fourth size"));
  gtk_combo_set_popdown_strings (GTK_COMBO (combo1), combo1_items);
  g_list_free (combo1_items);


  // configure the destination 
  destWidget = convert->dest->BuildConvertWidget(importWindow);
  gtk_container_add (GTK_CONTAINER (vbox3), destWidget);
  gtk_widget_set_sensitive(destWidget, FALSE);

  gtk_signal_connect (GTK_OBJECT (radiobutton1), "toggled",
                      GTK_SIGNAL_FUNC (on_radiobutton1_toggled),
                      (gpointer) destWidget);

  label2 = gtk_label_new (_("conversion"));
  gtk_widget_ref (label2);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "label2", label2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label2);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 1), label2);

  empty_notebook_page = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (empty_notebook_page);
  gtk_container_add (GTK_CONTAINER (notebook1), empty_notebook_page);

  // for statistics about the conversion process
  label3 = gtk_label_new (_("statistics"));
  gtk_widget_ref (label3);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "label3", label3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label3);
  gtk_notebook_set_tab_label (GTK_NOTEBOOK (notebook1), gtk_notebook_get_nth_page (GTK_NOTEBOOK (notebook1), 2), label3);

  hbuttonbox1 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "hbuttonbox1", hbuttonbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox1);
  gtk_box_pack_start (GTK_BOX (vbox2), hbuttonbox1, FALSE, TRUE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox1), 0);

  button1 = gnome_stock_button (GNOME_STOCK_BUTTON_OK);
  gtk_widget_ref (button1);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "button1", button1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button1);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button1);
  GTK_WIDGET_SET_FLAGS (button1, GTK_CAN_DEFAULT);

  button2 = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_ref (button2);
  gtk_object_set_data_full (GTK_OBJECT (importWindow), "button2", button2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button2);
  gtk_container_add (GTK_CONTAINER (hbuttonbox1), button2);
  GTK_WIDGET_SET_FLAGS (button2, GTK_CAN_DEFAULT);

  gtk_signal_connect (GTK_OBJECT (button1), "clicked",
                      GTK_SIGNAL_FUNC (Converter::on_button1_clicked),
                      convert);

  gtk_signal_connect (GTK_OBJECT (button2), "clicked",
                      GTK_SIGNAL_FUNC (Converter::on_button2_clicked),
                      NULL);

  return importWindow;
}
