/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

/* This file contains the code needed to get the folders in a project working */

#include "folder.h"
#include <string.h>
#include "misc.h"
#include "application.h"

#include "imagesequence.h"
#include "singleimage.h"
#include "colorclip.h"
#include <stdio.h>


Folder::Folder () {
  clip = NULL;
  parent = NULL;
  subFolder = NULL;
  opened = FALSE;
}

// TODO: free subelements
Folder::~Folder () {
  g_slist_free (clip);
  g_slist_free (subFolder);
}

char *Folder::GetTypeName () {
  return "Folder";
}

int Folder::Insert (XmlToken *token) {
  char *str = NULL;
  int error = 0;

  if (!g_strcasecmp (token->GetKey (), "ImageSequence") || 
      !g_strcasecmp (token->GetKey (), "SingleImage") ||
      !g_strcasecmp (token->GetKey (), "SolidColor") ||
      !g_strcasecmp (token->GetKey (), "TriSequence")) {
    MediaClip *src = (MediaClip *)app->BuildClip (token);
    return AddClip (src);
  }

  if (!g_strcasecmp (token->GetKey (), "Collection")) {
    token->Rewind ();
    XmlToken *temp;
    while (temp = token->GetNext ()) {
      error = Insert (temp);
      delete temp;
      if (error)
	return error;
    }
    return 0;
  }

  if (!g_strcasecmp (token->GetKey (), "folder")) {
    Folder *src = (Folder *)app->BuildClip (token);
    return AddSubFolder (src);
  }
  return -1;
}

void Folder::Load (XmlToken *src) {
  Clip::Load (src);

  opened = src->GetInt ("Opened");

  src->Rewind ();
  XmlToken *temp;
  while (temp = src->GetNext ()) {
    Insert (temp);
  }
}

void Folder::Save (XmlToken *dest) {
  Clip::Save (dest);

  dest->AppendInt ("Opened", opened);

  GSList *temp = subFolder;
  while (temp) {
    XmlToken *token = new XmlToken;
    ((Folder *)(temp->data))->Save (token);
    dest->AppendToken (token);
    temp = temp->next;
  }

  temp = clip;
  while (temp) {
    XmlToken *token = new XmlToken;
    ((MediaClip *)(temp->data))->Save (token);
    dest->AppendToken (token);
    temp = temp->next;
  }
  dest->SetKey ("Folder");
}

static gint Compare (Clip *c1, Clip *c2) {
  return g_strcasecmp (c1->GetName (), c2->GetName ());
}

/// Add a sub-folder to this folder
int Folder::AddSubFolder (Folder *fold) {
  int status = CheckName (subFolder, fold->GetName());
  if (status)
    return status;

  subFolder = g_slist_insert_sorted (subFolder, fold, (GCompareFunc)Compare);
  fold->parent = this;
  return 0;
}

/// Add a new clip to this folder
int Folder::AddClip (MediaClip *newClip) {  
  int status = CheckName (clip, newClip->GetName());
  if (status == 1) {
    char *origname = g_strdup (newClip->GetName ());
    int counter = 1;
    while (status == 1) {
      counter++;
      char *temp = g_strdup_printf ("%s (%d)", origname, counter);
      status = CheckName (clip, temp);
      if (!status)
	newClip->SetName (temp);

      delete temp;
    }
    delete origname;
  }

  if (status == 2)
    return 2;
  
  clip = g_slist_insert_sorted (clip, newClip, (GCompareFunc)Compare);
  return 0;
}

/// Delete a clip from this folder
void Folder::DeleteClip (MediaClip *src) {
  clip = g_slist_remove (clip, src);
}

/// Delete a subfolder from this folder
void Folder::DeleteSubFolder (Folder *folder) {
  subFolder = g_slist_remove (subFolder, folder);
}

static int compareFunc (Clip *c1, char *c2) {
  return g_strcasecmp (c1->GetName(), c2);
}

int Folder::CheckName (GSList *list, char *clipName) {
  // Check if name exists.
  if (g_slist_find_custom (list, clipName, (GCompareFunc)compareFunc)) {
    return 1;
  }
  // Check if clipname contains an illegal char
  char *temp = clipName;
  while (*temp) {
    if (*temp < 32)
      return 2;

    switch (*temp) {
    case '>':
    case '<':
      return 2;
    }
    temp++;
  }
  return 0;
}

void Folder::Erase () {};






