/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "gtkanimation.h"

GtkAnimation::~GtkAnimation () {
  if (backBuffer)
    delete backBuffer;
};

GtkAnimation::GtkAnimation (int w, int h) {
  dest          = NULL;
  width         = w;
  height        = h;
  Init ();
}

GtkAnimation::GtkAnimation (GtkWidget *dest, int w, int h) {
  width = w;
  height = h;

  this->dest = dest->window;
  gc = dest->style->white_gc;
  BuildWidget (dest);
  Init ();
}

void GtkAnimation::Init () {
  frameBuffer = new FrameBuffer (width, height, RGB24);

  backBuffer = new guchar[frameBuffer->GetVideoDataSize ()];
  frameBuffer->SetVideoData (backBuffer);
}

void GtkAnimation::NextFrame () {
  gdk_draw_rgb_image (dest, gc, 0, 0, width, height, 
		      GDK_RGB_DITHER_NORMAL, backBuffer, frameBuffer->GetStride ());
}

void GtkAnimation::StopPlayback () {}

void GtkAnimation::BuildWidget (GtkWidget *parent) {
  GtkWidget *viewport27;
  GtkWidget *video;

  viewport27 = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (viewport27);
  gtk_widget_set_usize (viewport27, width + 4, height +4);

  video = gtk_drawing_area_new ();
  gtk_widget_show (video);
  gtk_container_add (GTK_CONTAINER (viewport27), video);
  gtk_widget_set_usize (video, width, height);

  if(GTK_IS_BOX(parent)) {
    gtk_box_pack_start(GTK_BOX(parent), viewport27, TRUE, FALSE, 0);
  }  else {
    gtk_container_add(GTK_CONTAINER(parent), viewport27);
  }
  dest = video->window;
  gc = video->style->white_gc;
}
