/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __GTKANIMATION_H__
#define __GTKANIMATION_H__

#include "animation.h"
#include <gtk/gtk.h>
#include <glib.h>

/** Play an animation in a gtk window. */
class GtkAnimation : public Animation {
  /// A graphics context for doing the blits
  GdkGC *gc;
  /// The drawable to draw on.
  GdkWindow *dest;
  /// The back buffer memory.
  guchar *backBuffer;
  /// The dimensions of the playback window
  int width, height;
  /** The frame buffer has stopped its playback.
   *  This function is so windowed apps can prepare any pixmaps for repaints. */
  virtual void StopPlayback();

  /** Display the next frame.
   *  This function blits the back buffer onto the front buffer. */
  virtual void NextFrame ();
 protected:
  /** Initialize the frame buffer & vars. 
   *  This function allocates the memory needed for back buffer*/
  virtual void Init ();

 public:
  /// Build the widget to play back in
  void BuildWidget (GtkWidget *parent);

  /// Destroy the playback object.
  ~GtkAnimation ();
  /** Play back an animation.
   *  @param w the width of the playback window.
   *  @param h the height of the playback window. */
  GtkAnimation (int w, int h);
  /** Play back an animation.
   *  @param widget a GdkDrawable to use for playback. Width & height are taken from this
   *                widget. */
  GtkAnimation (GtkWidget *dest, int w, int h);
};

#endif  // __GTKANIMATION_H__
