/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "trisequence.h"
#include "converter.h"
#include "imagesequence.h"
#include "importsettings.h"
#include "application.h"
#include "misc.h"
#include "assert.h"

#include <stdlib.h>
#include <stdio.h>
#include <gtk/gtk.h>

/// File/import/image seq.
void ImageSequenceClip::Import (GtkMenuItem *menuitem, gpointer user_data) {
  
  char *fileName; // = ShowFileSelector ("Select the first image in the sequence", "");
  ImageSequenceClip *temp = new ImageSequenceClip ();
  TriSequenceClip *tseq = new TriSequenceClip ();

  MediaClip *target = Converter::Convert(temp, tseq);
/*
  MediaClip *target = ImportSettings::Import(temp, tseq);

  if (target) { 
    if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (tseq)) {
      delete tseq;
      delete temp;   
      return;
    }
    app->GetProjectWindow()->BuildProjectListView ();
  }


  if (fileName) {
    temp->SetName (g_basename (fileName));
    
	 printf("setting filename\n");
    if (temp->SetFileName (fileName)) {
      delete temp;
      return;
    }

    printf("making trisequence\n");

    // we now have the imagesequence
	 // we import this into thr trisequenceclip 
	 if (tseq->ConvertFrom(temp, tmpnam(NULL))) {
      delete tseq;
      delete temp;
      return;
	 }
    tseq->SetName (g_basename (fileName));
	 
	 printf("adding to project\n");
    if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (tseq)) {
      delete tseq;
      delete temp;   
      return;
    }
    app->GetProjectWindow()->BuildProjectListView ();
  }
*/
}

void ImageSequenceClip::BuildEditWidget (GtkNotebook *parent) {
  
}

ImageSequenceClip::ImageSequenceClip () {
  startFrame = length = 0;
  prefix = postfix = NULL;
}

ImageSequenceClip::~ImageSequenceClip () {
  if (prefix)
    delete prefix;

  if (postfix)
    delete postfix;
}

char *ImageSequenceClip::GetTypeName () {
  return "Image sequence";
}
int ImageSequenceClip::Uses (MediaClip *clip) {
  if (clip == this)
    return 1;
  return 0;
}

int ImageSequenceClip::Erase () {
  return 0;
}
void ImageSequenceClip::Save (XmlToken *dest) {
  MediaClip::Save (dest);

  dest->AppendInt ("Decimals", decimals);
  dest->AppendInt ("StartFrame", startFrame);
  dest->AppendInt ("Length", length);
  if (strlen (prefix))
    dest->AppendString ("Prefix", prefix);
  if (strlen (postfix))
    dest->AppendString ("Postfix", postfix);

  dest->SetKey ("ImageSequence");
}

void ImageSequenceClip::Load (XmlToken *src) {
  MediaClip::Load (src);

  decimals = src->GetInt ("Decimals");
  startFrame = src->GetInt ("StartFrame");
  length = src->GetInt ("Length");
  prefix = src->GetString ("Prefix");
  if (!prefix)
    prefix = g_strdup ("");
  postfix = src->GetString ("PostFix");
  if (!postfix)
    postfix = g_strdup ("");
}

int ImageSequenceClip::GetLength (int fps) {
  return length * fps / clipFps;
}

int ImageSequenceClip::GetSize () {
  return 0;
}
char *ImageSequenceClip::GetFileLocation () {
  return "";
}
int ImageSequenceClip::GetCacheStatus () {
  return 2;
}

void ImageSequenceClip::Render (int frame, int fps, FrameBuffer *dest) {
  GdkImlibImage *src;
  frame = frame * clipFps / fps;

  printf ("frame: %d ", frame);

  if (frame < 0)
    frame = 0;

  if (frame > length)
    frame = length;

  printf ("frame: %d\n", frame);

  if (decimals == -1) {
    src = gdk_imlib_load_image (prefix);
  } else {
    src = gdk_imlib_load_image (BuildFileName (frame + startFrame));
  }

  GdkRectangle srcRect = {0, 0, src->rgb_width, src->rgb_height};
  GdkRectangle destRect = {0, 0,  dest->GetWidth(), dest->GetHeight()};

  FrameBuffer *temp = new FrameBuffer (src->rgb_width, src->rgb_height, RGB24);
  temp->SetVideoData (src->rgb_data);
  
  dest->StretchBlit (temp, &srcRect, &destRect);

  gdk_imlib_destroy_image(src);       
  delete temp;
}

char *ImageSequenceClip::BuildFileName (int fileNum) {
  static char tempstr[255];
  strcpy (tempstr, prefix);
  strncat (tempstr, "0000000000000", decimals);
  specialitoa (&tempstr[strlen(tempstr)-1], fileNum);
  strcat (tempstr, postfix);
}
int ImageSequenceClip::SetFileName (char *newFileName) {
  if (prefix)
    delete prefix;

  if (postfix)
    delete postfix;

  // attempt to load this image
  GdkImlibImage *temp = gdk_imlib_load_image (newFileName);
  if (!temp)
    return 1;

  gdk_imlib_kill_image (temp);

  // Parse the file name to get the prefix & postfix.
  // First, we work backwards from the end an find the first numeric char
  int len = strlen (newFileName);
  char *scanchar = &newFileName[len - 1];
  
  while (*scanchar < '0' || *scanchar > '9') {
    if (scanchar == newFileName) {
      // No deciamals found, we have a single image
      decimals = -1;
      startFrame = length = 1;
      prefix = g_strdup (newFileName);
      return 0;
    }
    scanchar--;
  }
  int deciend = scanchar - newFileName;
  
  postfix = g_strdup (scanchar+1);
  // Find prefix
  while ((*scanchar >= '0' && *scanchar <= '9') && scanchar != newFileName) {
    scanchar--;
  }

  int decistart = scanchar - newFileName + 1;
  prefix = g_strndup (newFileName, decistart);
  startFrame = atoi (&newFileName[decistart]);
  decimals = deciend - decistart + 1;

  // Okay, now we should have the basics done...
  // Its time to find out how many there are...
  FILE *fil = NULL;
  int i = startFrame + 1;
  do {
    if (fil)
      fclose (fil);
    i++;
    fil = fopen (BuildFileName (i),"r");
  } while (fil != NULL);

  length = i - 1 - startFrame;
  inPoint = 3;
  outPoint = length - 3;
  return 0;
}

void ImageSequenceClip::BuildEditWidget (GtkWidget *parent) {}



void ImageSequenceClip::on_button3_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  char *fileName = ShowFileSelector ("Select the first image in the sequence", "");

  ImageSequenceClip *clip = (ImageSequenceClip *)user_data;

  if (!clip->SetFileName(fileName)) {
    clip->SetName(g_basename(fileName));

    // TODO needs a static char[]
    gtk_label_set_text(GTK_LABEL(clip->label14), fileName);

    clip->spinbutton1_adj = gtk_adjustment_new (1, 0, clip->length, 1, 10, 10);
	 gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(clip->spinbutton1), GTK_ADJUSTMENT(clip->spinbutton1_adj));

    clip->spinbutton2_adj = gtk_adjustment_new (1, 0, clip->length, 1, 10, 10);
	 gtk_spin_button_set_adjustment(GTK_SPIN_BUTTON(clip->spinbutton2), GTK_ADJUSTMENT(clip->spinbutton2_adj));

	 gtk_spin_button_set_value(GTK_SPIN_BUTTON(clip->spinbutton1), (gfloat) clip->startFrame);
	 gtk_spin_button_set_value(GTK_SPIN_BUTTON(clip->spinbutton2), (gfloat) clip->length);
    gtk_widget_set_sensitive (clip->frame1, TRUE);

    gtk_label_set_text(GTK_LABEL(clip->label11), fileName);

	 
  }
  else {
    gtk_widget_set_sensitive (clip->frame1, FALSE);
  }

}

GtkWidget *ImageSequenceClip::BuildImportWidget (GtkWidget *window1) {

  GtkWidget *vbox1;
  GtkWidget *hbox1;
  GtkWidget *label4;
  GtkWidget *button3;
  GtkWidget *table1;
  GtkWidget *label6;
  GtkObject *spinbutton3_adj;
  GtkWidget *label7;
  GtkWidget *label5;
  GtkWidget *label8;
  GtkWidget *label9;
  GtkWidget *frame2;
  GtkWidget *table2;
  GtkWidget *label13;
  GtkWidget *label10;
  GtkWidget *label12;
  GtkWidget *checkbutton1;

  vbox1 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "vbox1", vbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox1);

  hbox1 = gtk_hbox_new (FALSE, 0);
  gtk_widget_ref (hbox1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "hbox1", hbox1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbox1);
  gtk_box_pack_start (GTK_BOX (vbox1), hbox1, FALSE, FALSE, 0);

  label4 = gtk_label_new (_("select file:"));
  gtk_widget_ref (label4);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label4", label4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label4);
  gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE, 0);
  gtk_misc_set_padding (GTK_MISC (label4), 3, 0);

  label14 = gtk_label_new ("");
  gtk_widget_ref (label14);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label14", label14,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label14);
  gtk_box_pack_start (GTK_BOX (hbox1), label14, TRUE, FALSE, 0);

  button3 = gtk_button_new_with_label (_("browse"));
  gtk_widget_ref (button3);
  gtk_object_set_data_full (GTK_OBJECT (window1), "button3", button3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button3);
  gtk_box_pack_start (GTK_BOX (hbox1), button3, FALSE, FALSE, 0);
  gtk_container_set_border_width (GTK_CONTAINER (button3), 4);

  frame1 = gtk_frame_new (_("Sequence info"));
  gtk_widget_ref (frame1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "frame1", frame1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame1);
  gtk_box_pack_start (GTK_BOX (vbox1), frame1, TRUE, TRUE, 0);
  gtk_widget_set_sensitive (frame1, FALSE);

  table1 = gtk_table_new (5, 2, FALSE);
  gtk_widget_ref (table1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "table1", table1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table1);
  gtk_container_add (GTK_CONTAINER (frame1), table1);

  spinbutton2_adj = gtk_adjustment_new (1, 0, 100, 1, 10, 10);
  spinbutton2 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton2_adj), 1, 0);
  gtk_widget_ref (spinbutton2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "spinbutton2", spinbutton2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton2);
  gtk_table_attach (GTK_TABLE (table1), spinbutton2, 1, 2, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  label6 = gtk_label_new (_("Last Frame :"));
  gtk_widget_ref (label6);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label6", label6,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label6);
  gtk_table_attach (GTK_TABLE (table1), label6, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label6), 0, 7.45058e-09);
  gtk_misc_set_padding (GTK_MISC (label6), 10, 0);

  spinbutton3_adj = gtk_adjustment_new (50, 0, 10000, 1, 10, 10);
  spinbutton3 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton3_adj), 1, 0);
  gtk_widget_ref (spinbutton3);
  gtk_object_set_data_full (GTK_OBJECT (window1), "spinbutton3", spinbutton3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton3);
  gtk_table_attach (GTK_TABLE (table1), spinbutton3, 1, 2, 2, 3,
                    (GtkAttachOptions) (GTK_EXPAND),
                    (GtkAttachOptions) (0), 0, 0);

  label7 = gtk_label_new (_("Frames/second :"));
  gtk_widget_ref (label7);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label7", label7,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label7);
  gtk_table_attach (GTK_TABLE (table1), label7, 0, 1, 2, 3,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label7), 0, 7.45058e-09);
  gtk_misc_set_padding (GTK_MISC (label7), 10, 0);

  label5 = gtk_label_new (_("First Frame :"));
  gtk_widget_ref (label5);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label5", label5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label5);
  gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_RIGHT);
  gtk_misc_set_alignment (GTK_MISC (label5), 0, 7.45058e-09);
  gtk_misc_set_padding (GTK_MISC (label5), 10, 0);

  spinbutton1_adj = gtk_adjustment_new (1, 0, 1e+06, 1, 10, 10);
  spinbutton1 = gtk_spin_button_new (GTK_ADJUSTMENT (spinbutton1_adj), 1, 0);
  gtk_widget_ref (spinbutton1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "spinbutton1", spinbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (spinbutton1);
  gtk_table_attach (GTK_TABLE (table1), spinbutton1, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (spinbutton1), TRUE);

  label8 = gtk_label_new (_("total number of frames :"));
  gtk_widget_ref (label8);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label8", label8,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label8);
  gtk_table_attach (GTK_TABLE (table1), label8, 0, 1, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label8), 0, 7.45058e-09);
  gtk_misc_set_padding (GTK_MISC (label8), 10, 5);

  label9 = gtk_label_new ("");
  gtk_widget_ref (label9);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label9", label9,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label9);
  gtk_table_attach (GTK_TABLE (table1), label9, 1, 2, 4, 5,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  frame2 = gtk_frame_new (_("Frame properties"));
  gtk_widget_ref (frame2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "frame2", frame2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame2);
  gtk_table_attach (GTK_TABLE (table1), frame2, 0, 2, 3, 4,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 6, 5);

  table2 = gtk_table_new (3, 2, FALSE);
  gtk_widget_ref (table2);
  gtk_object_set_data_full (GTK_OBJECT (window1), "table2", table2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (table2);
  gtk_container_add (GTK_CONTAINER (frame2), table2);

  label11 = gtk_label_new ("");
  gtk_widget_ref (label11);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label11", label11,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label11);
  gtk_table_attach (GTK_TABLE (table2), label11, 1, 2, 0, 1,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label13 = gtk_label_new ("");
  gtk_widget_ref (label13);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label13", label13,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label13);
  gtk_table_attach (GTK_TABLE (table2), label13, 1, 2, 1, 2,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 0, 0);

  label10 = gtk_label_new (_("width :"));
  gtk_widget_ref (label10);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label10", label10,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label10);
  gtk_table_attach (GTK_TABLE (table2), label10, 0, 1, 0, 1,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 0);
  gtk_misc_set_alignment (GTK_MISC (label10), 7.45058e-09, 0.5);
  gtk_misc_set_padding (GTK_MISC (label10), 5, 3);

  label12 = gtk_label_new (_("height :"));
  gtk_widget_ref (label12);
  gtk_object_set_data_full (GTK_OBJECT (window1), "label12", label12,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label12);
  gtk_table_attach (GTK_TABLE (table2), label12, 0, 1, 1, 2,
                    (GtkAttachOptions) (GTK_FILL),
                    (GtkAttachOptions) (0), 5, 0);
  gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label12), 5, 3);

  checkbutton1 = gtk_check_button_new_with_label (_("interlaced"));
  gtk_widget_ref (checkbutton1);
  gtk_object_set_data_full (GTK_OBJECT (window1), "checkbutton1", checkbutton1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (checkbutton1);
  gtk_table_attach (GTK_TABLE (table2), checkbutton1, 0, 2, 2, 3,
                    (GtkAttachOptions) (0),
                    (GtkAttachOptions) (0), 5, 0);

  gtk_signal_connect (GTK_OBJECT (button3), "clicked",
                      GTK_SIGNAL_FUNC (on_button3_clicked),
                      (gpointer) this);

  return vbox1;

}
