/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __IMAGESEQUENCE_H__
#define __IMAGESEQUENCE_H__

#include "mclip.h"

/// This is a dummy clip used for test purposes only.
class ImageSequenceClip : public MediaClip {
  /** The number of digits in the numbering.
   *  if 0, no padding is done. If -1 no numbering is used (we are dealing with 1 image)*/
  int decimals;
  /// The start frame number of this imported clip.
  int startFrame;
  /// The length (in frames) of this clip
  int length;
  /// the pre & postfix of the filename string.
  char *prefix, *postfix;

  GtkWidget * frame1;
  GtkWidget * spinbutton1;
  GtkObject * spinbutton1_adj;
  GtkWidget * spinbutton2;
  GtkObject * spinbutton2_adj;
  GtkWidget * spinbutton3;
  GtkWidget * label14;
  GtkWidget * label11;

  /** Build the filename from prefix, decimals, startFrame, endFrame 
   *  @return the filename. This is a pointer to a static string, so don't free it 
   *  after use. */
  char *BuildFileName (int fileNum);
 protected:
  virtual void BuildEditWidget (GtkWidget *parent);
 public:
  ImageSequenceClip ();
  virtual ~ImageSequenceClip ();
  /** Return a textual description of this clip type */
  virtual char *GetTypeName ();

  /** Query if this clip makes use of another clip.
   *  @return 1 if [this clip] = [clip], otherwise 0; */
  virtual int Uses (MediaClip *clip);
  /// Save this clip into a token
  virtual void Save (XmlToken *dest);
  /// Load this clip from a token
  virtual void Load (XmlToken *src);

  /** Erase the dummy clip
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int Erase ();

  /** Get the length of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetLength (int fps);

  /** Get the disk size of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetSize ();

  /** Get the location of this clips file(s)
   *  This function is a stub. It doesn't do anything, and always returns "" */ 
  virtual char *GetFileLocation ();

  /** Get cache status of this clip
   *  This function is a stub. It doesn't do anything, and always returns 2 (N/A) */
  virtual int GetCacheStatus ();

  /* Render this clip's data into the current stream.
   * This function is a stud. It doesnt do anything. */
  virtual void Render (int frame, int fps, FrameBuffer *dest);

  /// Build the notebook page for this widget
  virtual void BuildEditWidget (GtkNotebook *parent);

  /// Build the notebook page for this widget
  virtual GtkWidget *BuildImportWidget (GtkWidget *parent);

  static void on_button3_clicked(GtkButton *button, gpointer user_data);

  /** Set the first image file used by this clip.
   *  This function calculates the files used by this clip & sets up all neccessary vars.
   *  @param newFileName the new file to use 
   *  @return error flag: 0 = this file is ok, 1 = Imlib can not load this file*/
  int SetFileName (char *newFileName);
  /// Import menu callback
  static void Import (GtkMenuItem *menuitem, gpointer user_data);
};

#endif
