/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "mclip.h"
#include "application.h"

MediaClip::MediaClip () {
  currentFrame = 0;
  clipFps = app->GetProject()->GetFPS();
  mediaClipEdit = NULL;
}

MediaClip::~MediaClip () {
  if (mediaClipEdit)
    gtk_widget_destroy (mediaClipEdit);
}

void MediaClip::ChangeName (GtkEntry *src) {
  char *temp = gtk_entry_get_text (src);

  SetName (temp);
  //h  delete temp;
  app->ClipChanged (this);
}

void MediaClip::SetCurrentFrame (int frameNo, int fps) {
  currentFrame = frameNo * clipFps / fps;
  AppSignal sig (CLIP_ACTIVE_FRAME_CHANGED, (void *)this);
  app->EmitSignal (&sig);
}

void MediaClip::Load (XmlToken *src) {
  Clip::Load (src);

  currentFrame = src->GetInt ("CurrentFrame");
  clipFps = src->GetInt ("fps");
  inPoint = src->GetInt ("IN");
  outPoint = src->GetInt ("OUT");
}

void MediaClip::Save (XmlToken *dest) {
  Clip::Save (dest);

  dest->AppendInt ("CurrentFrame", currentFrame);
  dest->AppendInt ("fps", clipFps);
  dest->AppendInt ("IN", inPoint);
  dest->AppendInt ("OUT", outPoint);
}

void MediaClip::ChangeSpeed (GtkSpinButton *src) {
  clipFps = gtk_spin_button_get_value_as_int (src);
  app->ClipChanged (this);
}

void MediaClip::on_revert_clicked (GtkButton *button, gpointer user_data) {
  ((MediaClip *)user_data)->EditRevertClicked ();
}
void MediaClip::on_ok_clicked (GtkButton *button, gpointer user_data) {
  ((MediaClip *)user_data)->EditOkClicked ();
}
void MediaClip::on_cancel_clicked (GtkButton *button, gpointer user_data) {
  ((MediaClip *)user_data)->EditCancelClicked ();
}
void MediaClip::on_speed_changed (GtkEditable *editable, gpointer user_data) {
  ((MediaClip *)user_data)->ChangeSpeed (GTK_SPIN_BUTTON (editable));
}
void MediaClip::on_name_changed (GtkEditable *editable, gpointer user_data) {
  ((MediaClip *)user_data)->ChangeName (GTK_ENTRY(editable));
}

void MediaClip::CloseEditWindow () { 
  delete revertName;
  gtk_widget_destroy (mediaClipEdit);
  mediaClipEdit = NULL;
}

void MediaClip::EditOkClicked () {
  app->GetProjectWindow()->BuildProjectListView ();
  CloseEditWindow ();
}
void MediaClip::EditCancelClicked () {
  SetName (revertName);
  app->ClipChanged (this);
  CloseEditWindow ();
}
void MediaClip::EditRevertClicked () {
  SetName (revertName);
  
  gtk_entry_set_text(GTK_ENTRY(clipname), GetName ());
  app->ClipChanged (this);
  clipFps = revertFps;
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(clipSpeed), clipFps);
}
void MediaClip::Edit () {
  // Only open a new window if we don't have one already
  if (mediaClipEdit)
    return;

  revertName = g_strdup (GetName ());
  revertFps = clipFps;

  char *temp = g_strconcat ("Edit ", GetTypeName (), NULL);
  GtkWidget *temp2 = BuildEditWindow (temp);
  delete temp;

  gtk_widget_show (mediaClipEdit);
}

GtkWidget *MediaClip::BuildEditWindow (char *title) {
  GtkWidget *vbox32;
  GtkWidget *table17;
  GtkWidget *label114;
  GtkObject *clipSpeed_adj;
  GtkWidget *label116;
  GtkWidget *label115;
  GtkWidget *hseparator14;
  GtkWidget *hseparator12;
  GtkWidget *hbuttonbox15;
  GtkWidget *editColorRevert;
  GtkWidget *editColorOk;
  GtkWidget *editColorCancel;

  mediaClipEdit = gtk_window_new (GTK_WINDOW_DIALOG);
  gtk_widget_set_usize (mediaClipEdit, 450, 323);
  gtk_container_border_width (GTK_CONTAINER (mediaClipEdit), 4);
  gtk_signal_connect (GTK_OBJECT (mediaClipEdit), "delete_event",
                      GTK_SIGNAL_FUNC (on_colorEditWindow_delete_event),
                      this);
  gtk_window_set_title (GTK_WINDOW (mediaClipEdit), title);
  gtk_window_set_wmclass (GTK_WINDOW (mediaClipEdit), "coloredit", "coloredit");

  vbox32 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox32);
  gtk_container_add (GTK_CONTAINER (mediaClipEdit), vbox32);

  table17 = gtk_table_new (2, 3, FALSE);
  gtk_widget_show (table17);
  gtk_box_pack_start (GTK_BOX (vbox32), table17, FALSE, FALSE, 0);

  label114 = gtk_label_new ("Clip Name:");
  gtk_widget_show (label114);
  gtk_table_attach (GTK_TABLE (table17), label114, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label114), 0, 0.5);

  clipSpeed_adj = gtk_adjustment_new (clipFps, 1, 1000, 1, 10, 10);
  clipSpeed = gtk_spin_button_new (GTK_ADJUSTMENT (clipSpeed_adj), 1, 0);
  gtk_widget_show (clipSpeed);
  gtk_table_attach (GTK_TABLE (table17), clipSpeed, 1, 2, 1, 2,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 0);
  gtk_widget_set_usize (clipSpeed, 80, -1);
  gtk_signal_connect (GTK_OBJECT (clipSpeed), "changed",
                      GTK_SIGNAL_FUNC (on_speed_changed),
                      this);
  gtk_spin_button_set_numeric (GTK_SPIN_BUTTON (clipSpeed), TRUE);
  gtk_spin_button_set_update_policy (GTK_SPIN_BUTTON (clipSpeed), GTK_UPDATE_IF_VALID);

  label116 = gtk_label_new ("Frames/second.");
  gtk_widget_show (label116);
  gtk_table_attach (GTK_TABLE (table17), label116, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label116), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label116), 4, 0);

  clipname = gtk_entry_new ();
  gtk_entry_set_text(GTK_ENTRY(clipname), GetName ());
  gtk_widget_show (clipname);
  gtk_table_attach (GTK_TABLE (table17), clipname, 1, 3, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
		    (GtkAttachOptions) 0, 0, 0);
  gtk_signal_connect (GTK_OBJECT (clipname), "changed",
                      GTK_SIGNAL_FUNC (on_name_changed),
                      this);

  label115 = gtk_label_new ("Recording speed: ");
  gtk_widget_show (label115);
  gtk_table_attach (GTK_TABLE (table17), label115, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 8);

  hseparator14 = gtk_hseparator_new ();
  gtk_widget_show (hseparator14);
  gtk_box_pack_start (GTK_BOX (vbox32), hseparator14, FALSE, FALSE, 4);

  hbuttonbox15 = gtk_hbutton_box_new ();
  gtk_widget_show (hbuttonbox15);
  gtk_box_pack_end (GTK_BOX (vbox32), hbuttonbox15, FALSE, FALSE, 0);
  gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox15), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing (GTK_BUTTON_BOX (hbuttonbox15), 16);

  editColorRevert = gtk_button_new_with_label ("Revert");
  gtk_widget_show (editColorRevert);
  gtk_container_add (GTK_CONTAINER (hbuttonbox15), editColorRevert);
  gtk_signal_connect (GTK_OBJECT (editColorRevert), "clicked",
                      GTK_SIGNAL_FUNC (on_revert_clicked),
                      this);

  editColorOk = gtk_button_new_with_label ("Ok");
  gtk_widget_show (editColorOk);
  gtk_container_add (GTK_CONTAINER (hbuttonbox15), editColorOk);
  gtk_signal_connect (GTK_OBJECT (editColorOk), "clicked",
                      GTK_SIGNAL_FUNC (on_ok_clicked),
                      this);

  editColorCancel = gtk_button_new_with_label ("Cancel");
  gtk_widget_show (editColorCancel);
  gtk_container_add (GTK_CONTAINER (hbuttonbox15), editColorCancel);
  gtk_signal_connect (GTK_OBJECT (editColorCancel), "clicked",
                      GTK_SIGNAL_FUNC (on_cancel_clicked),
                      this);

  hseparator12 = gtk_hseparator_new ();
  gtk_widget_show (hseparator12);
  gtk_box_pack_end (GTK_BOX (vbox32), hseparator12, FALSE, FALSE, 4);


  GtkWidget *notebook6 = gtk_notebook_new ();
  gtk_widget_show (notebook6);
  gtk_box_pack_start (GTK_BOX(vbox32), notebook6, TRUE, TRUE, 0);

  BuildEditWidget (GTK_NOTEBOOK(notebook6));
}

gboolean MediaClip::on_colorEditWindow_delete_event (GtkWidget *widget, 
						     GdkEvent *event, 
						     gpointer user_data) {
  ((MediaClip *)user_data)->CloseEditWindow ();
  return FALSE;
}
