/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "misc.h"
#include <gnome.h>

GtkWidget *GetPixmap (gchar **xpmdata, GtkWidget *window) {
 GtkStyle *internal0;
 GdkPixmap *internal1;
 GdkBitmap *mask;
 GtkWidget *internal2;
 internal0 = gtk_widget_get_style(window);
 internal1 = gdk_pixmap_create_from_xpm_d(window->window, 
					  &mask, 
					  &internal0->bg[GTK_STATE_NORMAL], 
					  xpmdata);
 internal2 = gtk_pixmap_new(internal1, mask);
 return internal2;
}

int ShowInputBox (char *title, char *desc, char **text) {
  char *temp = gtk_dialog_cauldron (desc, GTK_CAULDRON_GRAB | GTK_CAULDRON_DIALOG,
				    " ( %L / %Eod)          / %S / ( %Bgrqxfp || %Bgqxfp )",
				    desc, text, GNOME_STOCK_BUTTON_OK, 
				    GNOME_STOCK_BUTTON_CANCEL);
  if (!temp)
    return 0;
  if (strcmp (temp,"GTK_CAULDRON_ENTER") && strcmp (temp,"Button_Ok"))
    return 0;
  return 1;
}

static int fileSelActive = FALSE, fileSelOk;
static void on_fileSelectorCancel_clicked (GtkButton *button,GtkWidget *fileSel){
  fileSelActive = FALSE;
  fileSelOk = FALSE;
}
static void on_fileSelectorOk_clicked (GtkButton *button, GtkFileSelection *fileSel) {
  fileSelActive = FALSE;
  fileSelOk = TRUE;
}
char *ShowFileSelector (char *title, char *fileName) {
  GtkWidget *fileSel = gtk_file_selection_new (title);

  if (fileName)
    gtk_file_selection_set_filename (GTK_FILE_SELECTION(fileSel), fileName);

  /* Connect the ok_button to file_ok_sel function */
  gtk_signal_connect (GTK_OBJECT (fileSel), "delete_event",
		      GTK_SIGNAL_FUNC (on_fileSelectorCancel_clicked), NULL);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fileSel)->ok_button),
		      "clicked", (GtkSignalFunc) on_fileSelectorOk_clicked, fileSel);
  gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fileSel)->cancel_button),
  		      "clicked", (GtkSignalFunc) on_fileSelectorCancel_clicked, fileSel);
  gtk_window_set_modal (GTK_WINDOW (fileSel), TRUE);

  fileSelActive = TRUE;
  fileSelOk = TRUE;
  gtk_widget_show (fileSel);

  while (fileSelActive) {
    while (gtk_events_pending ())
      gtk_main_iteration ();
    gdk_flush ();
  }

  char *retStr = fileSelOk ? 
                 g_strdup (gtk_file_selection_get_filename (GTK_FILE_SELECTION(fileSel)))
                 : NULL;
  gtk_widget_destroy (GTK_WIDGET(fileSel));
  return retStr;
}

DialogReturn ShowDialog (char *title, char *text, DialogType type) {
  GtkWidget *temp;
  switch (type) {
  case Info:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_INFO, GNOME_STOCK_BUTTON_OK,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 0);
    break;
  case YesNo:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_QUESTION, 
				  GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 1);
    break;
  case YesNoCancel:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_QUESTION, 
				  GNOME_STOCK_BUTTON_YES, GNOME_STOCK_BUTTON_NO,
				   GNOME_STOCK_BUTTON_CANCEL,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 0);
    break;
  case OkCancel:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_QUESTION, 
				  GNOME_STOCK_BUTTON_OK,GNOME_STOCK_BUTTON_CANCEL,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 0);
    break;
  case Warning:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_WARNING, 
				  GNOME_STOCK_BUTTON_YES,GNOME_STOCK_BUTTON_NO,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 1);
    break;
  case Error:
    temp = gnome_message_box_new (text, GNOME_MESSAGE_BOX_ERROR, 
				  GNOME_STOCK_BUTTON_OK,NULL);
    gnome_dialog_set_default (GNOME_DIALOG(temp), 0);
    break;
  }
  gtk_window_position (GTK_WINDOW (temp), GTK_WIN_POS_CENTER);
  gtk_window_set_title (GTK_WINDOW(temp), title);
  gtk_window_set_modal (GTK_WINDOW(temp), TRUE);
  int result = gnome_dialog_run_and_close (GNOME_DIALOG(temp));

  if (result == -1)
    result = 2;

  if (type == OkCancel && result == 1)
    result = 2;

  return (DialogReturn)result;

}

/* I always thought that itoa was a libc function - guess you learn
   something new every day ;) */
void specialitoa (char *dest, int src) {
  while (src > 0) {
    *dest = (src % 10) + '0';
    *dest--;
    src /= 10;
  }
}

char *BuildLengthString (int duration,int fps) {
  int frames  = duration % fps;
  duration -= frames;
  int secs    = (duration / fps) % 60;
  duration -= secs * fps;
  int minutes = (duration / fps / 60);
  char *str;
  if (minutes < 60) {
    str = g_strdup ("00:00:00");
    specialitoa (&str[7], frames);
    specialitoa (&str[4], secs);
    specialitoa (&str[1], minutes);
  } else {
    str = g_strdup ("00:00:00:00");
    specialitoa (&str[11], frames);
    specialitoa (&str[7], secs);
    specialitoa (&str[4], minutes % 60);
    specialitoa (&str[1], minutes / 60);
  }
  return str;
}

double gtkCol[4];

char *FindString (char *src, char *str) {
  int len = strlen (str);
  while (*src) {
    if (*src == *str) {
      int i;
      for (i = 1; i < len; i++) {
	if (src[i] != str[i])
	  break;
      }
      if (i == len) {
	return src;
      }
    }
    src++;
  }
}

double *ARGBtoGDK (guint32 RGB) {
  gtkCol[3] = (double)(RGB >> 24) / 255.0;
  gtkCol[0] = (double)((RGB >> 16) & 255) / 255.0;
  gtkCol[1] = (double)((RGB >> 8) & 255) / 255.0;
  gtkCol[2] = (double)(RGB & 255) / 255.0;
  return gtkCol;
}

double *RGBtoGDK (guint32 RGB) {
  RGB &= 0xFFFFFF;
  gtkCol[0] = (double)(RGB >> 16) / 255.0;
  gtkCol[1] = (double)((RGB >> 8) & 255) / 255.0;
  gtkCol[2] = (double)(RGB & 255) / 255.0;
  return gtkCol;
}

guint32 GDKtoARGB (double *GDK) {
  return ((guint32)(GDK[3] * 255) << 24) + 
    ((guint32)(GDK[0] * 255) << 16) + 
    ((guint32)(GDK[1] * 255) << 8) + 
    (guint32)(GDK[2] *255);
}

guint32 GDKtoRGB (double *GDK) {
  return ((guint32)(GDK[0] * 255) << 16) + 
    ((guint32)(GDK[1] * 255) << 8) + 
    (guint32)(GDK[2] *255);
}
int ValidFileName (char *fileName) {
  if (fileName && fileName[strlen(fileName) - 1] != '/')
    return TRUE;

  return FALSE;
}

int CheckFile (char *fileName, char *mode) {
  if (!fileName)
    return FALSE;

  FILE *temp = fopen (fileName, mode);

  if (!temp)
    return FALSE;

  fclose (temp);
  return TRUE;
}
  
  
