/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */
#include <gtk/gtk.h>

#ifndef __MISC_H__
#define __MISC_H__
/** Check if a file name is valid.
 *  At the moment this just makes sure that the fileName does NOT end with a /
 *  @param fileName the file name to check
 *  @return TRUE if file name is valid, FALSE if not. */
int ValidFileName (char *fileName);

/** Check if a file exists.
 *  This checks if a file can be opened with the given permissions
 *  @param fileName the file name can be opened
 *  @param mode an fopen() format string. Defaults to "r" */
int CheckFile (char *fileName, char *mode = "r");

/** Make a modal input box.
 *  @param title the window title
 *  @param desc  the descriptive text label
 *  @param text  a pointer to a char*. this char* is modified when the user selects OK
 *  @return 1 on OK, 0 on cancel */
int ShowInputBox (char *title, char *desc, char **text);
/** Make a modal file selection box.
 *  @param title the window title
 *  @param fileName the default filename
 *  @return the complete path of the file selected, or NULL if cancel was clicked. The
 *          string should be freed after use. */
char *ShowFileSelector (char *title, char *fileName);

/** Build a string of the form 'hrs:min:sec:fra'.
 *  @param length the length to calculate from (measured in frames)
 *  @param fps how many frames/second to use in calculatiom.
 *  @return a new allocated string containing the text */
char *BuildLengthString (int length, int fps);

/** Get a GtkPixmap from XPM data.
 *  @param xpmdata data from an XPM file
 *  @param window a widget from the window to display pixmaps in */
GtkWidget *GetPixmap (gchar **xpmdata, GtkWidget *window);

/** Convert a color from ARGB to GDK color space */
double *ARGBtoGDK (guint32 ARGB);
/** Convert a color from RGB to GDK color space */
double *RGBtoGDK (guint32 ARGB);
/// Convert a color from GDK to ARGB color space */
guint32 GDKtoARGB (double *GDK);
/// Convert a color from GDK to RGB color space */
guint32 GDKtoRGB (double *GDK);

/** Find a substring in a string.
 *  @param src the string to search
 *  @param str the substring to search <i>for</i>
 *  @return a pointer to the found substring, or NULL if not found */
char* FindString (char *src, char *str);

/** Insert an ASCII representation of val in src.
 *  This function begins in src and works its way backwards, */
void specialitoa (char *src, int val);
/** Set up the standard lookup tables. */
void initMisc ();

/** The type of dialog box to display.
 *  Theese values are used by ShowDialog to determine what buttons & icons to display. */
enum DialogType {
  /// Show an info icon & an Ok button
  Info,
  /// Show a question icon & Yes,No buttons.
  YesNo,
  /// Show a question icon & Yes,No,Cancel buttons.
  YesNoCancel,
  /// Show a question icon & Ok,Cancel buttons.
  OkCancel,
  /// Show a warning icon & Yes,No buttons.
  Warning,
  /// Show an error icon & Ok button.
  Error
};

/** Return type for dialog boxes.
 *  When the user selects one of the button on the simple dialogs (made with ShowDialog),
 *  The function returns one of theese values, depending on which button was pressed. */
enum DialogReturn {
  /// OK was pressed.
  Ok = 0,
  /// YES was pressed.
  Yes = 0,
  /// NO was pressed.
  No = 1,
  /// CANCEL was pressed.
  Cancel = 2
};


/** Show a modal input box. 
 *  @param title the title string of the window
 *  @param text the descriptive text
 *  @param type the type of dialog 
 *  @return the button that was pressed */
DialogReturn ShowDialog (char *title, char *text, DialogType type);

#endif 

