/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "playbuttons.h"
#include "misc.h"

#include "pixmaps/clipclearinout.xpm"
#include "pixmaps/clipend.xpm"
#include "pixmaps/clipin.xpm"
#include "pixmaps/clipplayinout.xpm"
#include "pixmaps/clipout.xpm"
#include "pixmaps/clippause.xpm"
#include "pixmaps/clipplay.xpm"
#include "pixmaps/clipsetin.xpm"
#include "pixmaps/clipsetout.xpm"
#include "pixmaps/clipstart.xpm"
#include "pixmaps/clipstop.xpm"


GtkWidget *PlayButtons::Build (PlayButtonsCallback *cb, GtkWidget *parent, 
			       gpointer user_data,
			       ClipViewerStyle buttons, 
			       GtkToolbarStyle style) {
  GtkWidget *toolbar;
  if (!GTK_IS_TOOLBAR (parent)) {
    toolbar = gtk_toolbar_new (GTK_ORIENTATION_HORIZONTAL, style);
    gtk_toolbar_set_style (GTK_TOOLBAR(toolbar), style);
  } else {
    toolbar = parent;
  }  

  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Start", "Go to IN point",
			   NULL, GetPixmap (clipstart_xpm, parent), 
			   GtkSignalFunc(cb->GoStart), 
			   user_data);
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Back", "Go back 1 frame",  
			   NULL, GetPixmap (clipin_xpm, parent),
			   GtkSignalFunc(cb->GoBack), user_data);
  gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Stop", "Stop playback",  
			   NULL, GetPixmap (clipstop_xpm, parent), 
			   GtkSignalFunc(cb->Stop), user_data);
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Play", "Start playback", 
			   NULL, GetPixmap (clipplay_xpm, parent), 
			   GtkSignalFunc(cb->Play), user_data);
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Play IN-OUT", 
			   "Play from IN to OUT points",  
			   NULL, GetPixmap (clipplayinout_xpm, parent), 
			   GtkSignalFunc(cb->PlayInOut), user_data);
  gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Forw.", "Skip 1 frame forward",
			   NULL, GetPixmap (clipout_xpm, parent), 
			   GtkSignalFunc(cb->GoForward), 
			   user_data);
  gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "End", "Go to OUT point",  
			   NULL, GetPixmap (clipend_xpm, parent), 
			   GtkSignalFunc(cb->GoEnd), user_data);
  if (buttons & FULL) {
    gtk_toolbar_append_space (GTK_TOOLBAR(toolbar));
    gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "IN", 
			     "Set the IN point to the current frame",
			     NULL, GetPixmap (clipsetin_xpm, parent), 
			     GtkSignalFunc(cb->SetIn), 
			     user_data);
    gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "OUT", 
			     "Set the OUT point to the current frame",
			     NULL, GetPixmap (clipsetout_xpm, parent), 
			     GtkSignalFunc(cb->SetOut), 
			     user_data);
    gtk_toolbar_append_item (GTK_TOOLBAR(toolbar), "Clear", 
			     "Clear the IN & OUT points",
			     NULL, GetPixmap (clipclearinout_xpm, parent), 
			     GtkSignalFunc(cb->ClearInOut), user_data);
  }
  return toolbar;
}
