/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

/* This is the file that handles the project class. Currently, it doesn't do much,
 * but it will eventually handle cache, project settings, etc. */

#include "project.h"
#include "application.h"
#include "config.h"

#include <stdio.h>

//Project *Project::CurrentFolder = NULL;
Project::Project () {
  // We must always have a root folder (even though it might be empty)
  root = new Folder ();

  root->SetName ("Untitled...");
  fileName = NULL;
}

void Project::SetFileName (char *newFileName) {
  if (newFileName) {
    if (fileName)
      delete fileName;
    fileName = g_strdup (newFileName);
  } else
    fileName = NULL;
}

Project::~Project () {
  if (root)
    delete root;
}

int Project::Load (char *fileName) {
  XmlToken in;
  if (!in.Load (fileName, "Project")) {
    return FALSE;
  }

  Load (&in);
  return TRUE;
}

void Project::Load (XmlToken *src) {
  XmlToken *temp = src->GetToken ("Settings");
  if (temp) {
    settings.Load (temp);
    delete temp;
  }
  /* else
    ShowDialog ("Load project", "Warning: The project settings could not be found", 
		Warning);
  */
  temp = src->GetToken ("Folder");
  if (temp) {
    if (root) 
      delete root;
    root = new Folder;
    root->Load (temp);
  } else
    ShowDialog ("Load project", "Warning: The main project info could not be found", 
		Warning);
}

int Project::Save () {
  XmlToken dest;

  Save (&dest);

  FILE *out = fopen (GetFileName (), "wb");
  if (!out) {
    ShowDialog ("Error saving project", "There was an error opening the file", Error);
    return FALSE;
  }
    
  char *text = g_strdup_printf ("<?xml version=\"%s\">\n\n", VERSION);

  dest.BuildText (&text, 0);

  fwrite (text, strlen (text), 1, out);
  fclose (out);
  delete text;
}

void Project::Save (XmlToken *dest) {
  dest->Clear ();
  dest->SetKey ("Project");

  XmlToken *temp = new XmlToken;
  settings.Save (temp);
  dest->AppendToken (temp);
  temp = new XmlToken;
  root->Save (temp);
  dest->AppendToken (temp);
}










