/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __PROJECT_H__
#define __PROJECT_H__

#include <gtk/gtk.h>
#include "misc.h"
#include "folder.h"
#include "prjsettings.h"

/** A film editor project.
 *  A project maintains a library of clips, that can be timelines, etc.
 *  They are all arranged inside folders. */
class Project {

  /// The root folder of the project
  Folder *root;

  GtkWidget *projectView, *treeView, *listView;

  char *fileName;

  /// pointer to the folder currently open inn the project view
  Folder *currentFolder;

  ProjectSettings settings;

 public:

  /// Constructor: make an empty project
  Project ();

  /// Destroy a project
  ~Project ();

  /** Load a project from a file.
   *  @return TRUE on success, FALSE on error */
  int Load (char *fileName);

  /** Load a project from src. */
  void Load (XmlToken *src);
  /** Save a project to dest
   *  @return TRUE on success, FALSE on error */
  void Save (XmlToken *dest);

  /** Get the file name of this project.
   *  This returns the filename of the project
   *  @return the filename (with path) of this project */
  char *GetFileName () {return fileName;}

  /** Set the file name of this project. */
  void SetFileName (char *fileName);

  /** Save the project to a file
   *  This function saves the project back to its file. 
   *  @return TRUE on success, otherwise FALSE. */
  int  Save ();

  /** Get the folder currently selected in the project window */
  Folder *GetCurrentFolder ();

  /// Get the project's root folder
  inline Folder *GetRoot ();

  /** Set the current Folder 
   *  This function sets a new current folder.
   *  @param folder the folder to select in the project window. If NULL,
   *                the project root folder is selected */
  void SetCurrentFolder (Folder *folder);

  /// Get how many frames/second we use in this project */
  inline int GetFPS ();

  int ChangedSinceSave () {return FALSE;}
};

inline Folder *Project::GetRoot () {
  return root;
};

inline int Project::GetFPS () {
  return 50;
}

#endif
