/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include <config.h>

#include "projectwindow.h"
#include "misc.h"
#include "clippanel.h"
#include "application.h"
#include "toolbox.h"
#include <gnome.h>


static GtkWidget *create_mainWindow (); // In mainwindow.cc

void ProjectWindow::on_EditClip_clicked (GtkButton *button, gpointer user_data) {
  app->GetProjectWindow()->GetCurrentClip()->Edit ();
}

void ProjectWindow::on_DeleteClip_clicked (GtkButton *button, gpointer user_data) {
  //  currentClip->Erase ();
}
/// quit signal handler
int ProjectWindow::Quit (GtkMenuItem *menuitem, gpointer user_data) {
  if (app->CloseProject ()){
    gtk_main_quit ();
  }
  return true;
}

// Handle clicks in the tree view
void ProjectWindow::on_projectWindowTree_selection_changed (GtkTree *tree, gpointer user_data) {
  GList *i;
  i = GTK_TREE_SELECTION(tree);
  if (i) {
    GtkWidget *item;
    item = GTK_WIDGET(i->data);
    Folder *target = (Folder *)(gtk_object_get_user_data (GTK_OBJECT(item)));
    app->GetProjectWindow()->SetCurrentFolder (target);
    app->GetProjectWindow()->SetFolderSelected (TRUE);
  }
  
}

void ProjectWindow::on_projectWindowList_select_row (GtkCList *clist, gint row, gint column, GdkEvent *event, gpointer user_data) {
  app->GetProjectWindow()->SetActiveClip 
    ((MediaClip *)gtk_clist_get_row_data (clist, row));
 
  app->GetProjectWindow()->SetFolderSelected (FALSE);
  
  // Edit if user doubleclicked
  if (event && event->type == GDK_2BUTTON_PRESS)
    app->GetProjectWindow()->GetCurrentClip()->Edit ();
}

ProjectWindow::ProjectWindow () {
  // create main window
  mainWindow = BuildWidget ();
  clipPanel = NULL;
}

void ProjectWindow::Show () {
  gtk_widget_show (mainWindow);
}

// Set the ccurrent folder to *folder, and update the 
// project list view
void ProjectWindow::SetCurrentFolder (Folder *folder) {
  currentFolder = folder;
  
  BuildProjectListView ();
}

void ProjectWindow::BuildProjectListView () {
  char *text[2];
  gtk_clist_clear (list);
  for (int i = 0; i < currentFolder->GetClipCount (); i++) {
    text[0] = currentFolder->GetClip(i)->GetName ();
    text[1] = BuildLengthString (currentFolder->GetClip(i)->GetLength (25), 25);
    gtk_clist_append (list, text);
    gtk_clist_set_row_data (list, i, currentFolder->GetClip(i));
    //    delete text[0];
    delete text[1];
  }
  gtk_clist_set_sort_type (list, GTK_SORT_ASCENDING);
  gtk_clist_set_sort_column (list, 0);
  gtk_clist_sort (list);
}

void ProjectWindow::ShowActiveClip () {
  if (!clipPanel) {
    clipPanel = new ClipPanel (currentClip, vbox);
  } else 
    clipPanel->SetSource (currentClip);
}

// Set the active clip to show in the clip viewer
void ProjectWindow::SetActiveClip (MediaClip *src) {
  if (currentClip == src)
    return;

  currentClip = src;
  
  ShowActiveClip ();
}

void ProjectWindow::BuildProjectTreeView () {
  gtk_widget_destroy (tree);

  tree = gtk_tree_new ();

  gtk_container_add (GTK_CONTAINER(treeView), tree);

  gtk_widget_set_usize (tree, 57, -1);
  gtk_signal_connect (GTK_OBJECT (tree), "selection_changed",
		      GTK_SIGNAL_FUNC (on_projectWindowTree_selection_changed),
		      NULL);
  gtk_tree_set_view_lines (GTK_TREE (tree), FALSE);
  gtk_widget_show (tree);
  gtk_object_set_data (GTK_OBJECT (mainWindow), "tree", tree);

  AddFolderToTreeView (app->GetProject()->GetRoot(), GTK_TREE(tree));
}

void ProjectWindow::on_projectWindowTree_expand (GtkTreeItem *item, gpointer user_data) {
  Folder *temp = (Folder*)gtk_object_get_user_data (GTK_OBJECT(item));
  temp->SetOpened (TRUE);
}

void ProjectWindow::on_projectWindowTree_collapse (GtkTreeItem *item, gpointer user_data) {
  Folder *temp = (Folder*)gtk_object_get_user_data (GTK_OBJECT(item));
  temp->SetOpened (FALSE);
}

void ProjectWindow::AddFolderToTreeView (Folder *source, GtkTree *tree) {
  // first, we must work out if we have any folders attatched.
  GtkTreeItem *tree_item = GTK_TREE_ITEM (gtk_tree_item_new_with_label (source->GetName ()));
  gtk_tree_append (tree, GTK_WIDGET(tree_item));

  gtk_object_set_user_data (GTK_OBJECT(tree_item), source);
  gtk_widget_show (GTK_WIDGET(tree_item));

  /* If we have, we add this folder as a subtree, and then
     do a recursive call of any subfolders */
  if (source->GetSubFolderCount ()) {
    GtkTree *subTree = GTK_TREE (gtk_tree_new ());
    gtk_tree_set_selection_mode (subTree, GTK_SELECTION_SINGLE);
    gtk_tree_set_view_mode      (subTree, GTK_TREE_VIEW_ITEM);
    gtk_tree_item_set_subtree   (tree_item, GTK_WIDGET (subTree));
    gtk_signal_connect          (GTK_OBJECT(subTree), "selection_changed", 
			         GTK_SIGNAL_FUNC(on_projectWindowTree_selection_changed), 
				 tree);
    gtk_signal_connect (GTK_OBJECT(tree_item), "expand", 
			GTK_SIGNAL_FUNC(on_projectWindowTree_expand), tree_item);
    gtk_signal_connect (GTK_OBJECT(tree_item), "collapse",
			GTK_SIGNAL_FUNC(on_projectWindowTree_collapse), tree_item);
    if (source->GetOpened())
      gtk_tree_item_expand (tree_item);

    gtk_tree_set_view_lines (subTree, FALSE);
    
    for (int i = 0; i < source->GetSubFolderCount(); i++)
      AddFolderToTreeView (source->GetSubFolder (i), subTree);
  } 
}

void ProjectWindow::Signal (AppSignal *signal) {}

void ProjectWindow::Cut () {
  Copy ();
  cutFolder = currentFolder;
  cutFlag = TRUE;
}

void ProjectWindow::Copy () {
  XmlToken *temp = new XmlToken ();
  if (folderSelected)
    app->SetClipboard (currentFolder);
  else
    app->SetClipboard (currentClip);

  cutFlag = FALSE;
}

void ProjectWindow::Paste () {
  //  app->GetClipboard()->Debug();

  // Check for empty clipboard
  if (!app->GetClipboard ())
    return;

  // Insert clip or folder into current folder
  if (app->IsClipboardMediaClip ()) {
    currentFolder->AddClip (app->GetClipboardMediaClip());
  } else {
    if (!g_strcasecmp (app->GetClipboardDesc(), "Folder")) {  // It's a folder
      // Abort on root folder cuts
      if (cutFlag) {
	if (!cutFolder->GetParent()) {
	  ShowDialog ("Cut & Paste", "Error:\n\nYou cannot cut & paste the root"
		      " folder.", Error);
	  return;
	}
	// Check that we aren't CUT/PASTEing a folder into its own subtree
	Folder *temp = currentFolder;
	while (temp) {
	  if (temp == cutFolder) {
	    ShowDialog ("Cut & Paste", "Error:\n\nYou cannot cut & paste a folder"
			" into its own subfolder.", Error);
	    return;
	  }
	  temp = temp->GetParent();
	}
      }
      currentFolder->AddSubFolder ((Folder*) app->GetClipboard());
    } else if (!g_strcasecmp (app->GetClipboardDesc(), "TimelineClip")) {
      ShowDialog ("Cut & Paste", "Error:\n\nYou cannot paste a timeline clip"
		  "into a project folder, only into another timeline\n", Error);
      return;
    }
  }
  
  // Are we cutting or copying
  if (cutFlag) {
    if (app->IsClipboardMediaClip ()) {
      cutFolder->DeleteClip (cutClip);
      cutFlag = FALSE;
    } else if (!g_strcasecmp (app->GetClipboardDesc(), "Folder")) {   // It's a folder
      cutFolder->GetParent()->DeleteSubFolder (cutFolder);
      cutFlag = FALSE;
    }
  }

  BuildProjectTreeView ();
  BuildProjectListView ();
}

/* Pointer of no return - from here onwards it's lots of messy Gnome/GTK code
 * for setting up the main window... */
GtkWidget *ProjectWindow::BuildWidget () {
  GtkWidget *appw, *hpaned1;
  GtkWidget *scrolledwindow5;
  GtkWidget *label67, *label68;

  char *title = g_strconcat ("Trinity v. ", VERSION, NULL);
  appw = gnome_app_new ("Trinity", title);
  delete title;

  Toolbox::CreateMenu (GNOME_APP(appw));
  Toolbox::CreateToolbar (GNOME_APP(appw));

  gtk_widget_set_usize (appw, 446, 369);
  gtk_signal_connect (GTK_OBJECT (appw), "delete_event",
		      GTK_SIGNAL_FUNC (Quit),
		      NULL);
  gtk_window_set_policy (GTK_WINDOW (appw), TRUE, TRUE, FALSE);
  gtk_window_set_wmclass (GTK_WINDOW (appw), "projectview", "projectview");
  
  vbox = gtk_vbox_new (FALSE, 1);
  gtk_widget_show (vbox);
  gnome_app_set_contents (GNOME_APP (appw), vbox);

  hpaned1 = gtk_hpaned_new ();
  gtk_widget_show (hpaned1);
  gtk_paned_gutter_size (GTK_PANED (hpaned1), 8);
  gtk_box_pack_start (GTK_BOX (vbox), hpaned1, true, true, 0);

  scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
  gtk_widget_show (scrolledwindow5);
  gtk_container_add (GTK_CONTAINER (hpaned1), scrolledwindow5);
  gtk_widget_set_usize (scrolledwindow5, 119, -1);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

  treeView = gtk_viewport_new (NULL, NULL);
  gtk_widget_show (treeView);
  gtk_container_add (GTK_CONTAINER (scrolledwindow5), treeView);
  gtk_widget_set_usize (treeView, 132, -1);

  tree = gtk_tree_new ();
  gtk_widget_show (tree);
  gtk_container_add (GTK_CONTAINER (treeView), tree);
  gtk_widget_set_usize (tree, 57, -1);
  gtk_signal_connect (GTK_OBJECT (tree), "selection_changed",
                      GTK_SIGNAL_FUNC (on_projectWindowTree_selection_changed),
                      NULL);

  list = (GtkCList *)gtk_clist_new (2);
  gtk_widget_show ((GtkWidget *)list);
  gtk_container_add (GTK_CONTAINER (hpaned1), (GtkWidget *)list);
  gtk_signal_connect (GTK_OBJECT (list), "select_row",
                      GTK_SIGNAL_FUNC (on_projectWindowList_select_row),
                      NULL);
  gtk_clist_set_column_width (list, 0, 220);
  gtk_clist_set_column_width (list, 1, 80);
  gtk_clist_column_titles_show (list);

  label67 = gtk_label_new ("Name:");
  gtk_widget_show (label67);
  gtk_clist_set_column_widget (list, 0, label67);
  gtk_label_set_justify (GTK_LABEL (label67), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label67), 0, 0.5);

  label68 = gtk_label_new ("Duration:");
  gtk_widget_show (label68);
  gtk_clist_set_column_widget (list, 1, label68);
  gtk_label_set_justify (GTK_LABEL (label68), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label68), 0, 0.5);

  return appw;
};
