/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __PROJECTWINDOW_H__
#define __PROJECTWINDOW_H__

#include <gtk/gtk.h>
#include "mclip.h"
#include "folder.h"
#include "clippanel.h"
#include "appsignal.h"

class Application;
/** This class encapsulates the main window. */
class ProjectWindow : public SignalListener {
  /// The main window top-level widget.
  GtkWidget *mainWindow;

  /// The current folder in the tree view
  Folder *currentFolder;

  /// The current clip in the list view
  MediaClip *currentClip;

  /// The clip viewer object.
  ClipPanel *clipPanel;
  
  /// the main container
  GtkWidget *vbox;
  /// The folder contents list view
  GtkCList *list;
  /// The folder tree & the viewport containing it.
  GtkWidget *tree, *treeView;
  /// Build the main window
  void BuildMainWindow ();
  GtkWidget *BuildWidget ();


  /// Cut, Copy & Paste: Are we cutting ?
  int cutFlag;
  /// The clip selected for cutting
  MediaClip *cutClip;
  /// The folder selected for cutting
  Folder *cutFolder;
  /// Flag: Was a folder selected last (false = clip selected)
  int folderSelected;
 public:
  ProjectWindow ();

  /// Build the project tree view
  void BuildProjectTreeView ();
  /// Build the project list view
  void BuildProjectListView ();

  /// Add a folder to a given branch in the project tree.
  void AddFolderToTreeView (Folder *source, GtkTree *tree);
  /// Get the current folder in the tree view.
  inline Folder *GetCurrentFolder ();
  /// Set the current folder of the project window.
  void SetCurrentFolder (Folder *folder);

  /// Build a clip player widget
  void SetActiveClip (MediaClip *src);
  /// Show the active clip iin the clip viewer
  void ShowActiveClip ();

  /// Show the main window.
  void Show ();
  /// Get the clip currently selected in the timeline.
  inline MediaClip *GetCurrentClip ();
  // Callback for window delete (and Exit from main menu).
  static int Quit (GtkMenuItem *menuitem, gpointer user_data);
  /// Handle an app signal
  virtual void Signal (AppSignal *signal);

  /// Callback for Cut
  void Cut ();
  /// Callback for Copy
  void Copy ();
  /// Callback for Paste
  void Paste ();
  /// Mark that a folder has been selected
  inline void SetFolderSelected (int folderSel);

  static void on_EditClip_clicked (GtkButton *button, gpointer user_data);
  static void on_DeleteClip_clicked (GtkButton *button, gpointer user_data);
 private:
  /// Signal handlers:
  static void on_projectWindowTree_selection_changed (GtkTree *tree, gpointer user_data);
  static void on_projectWindowTree_expand (GtkTreeItem *item, gpointer user_data);
  static void on_projectWindowList_select_row (GtkCList *clist, gint row, 
					       gint column, GdkEvent *event, 
					       gpointer user_data);
  static void on_projectWindowTree_collapse (GtkTreeItem *item, 
					     gpointer user_data);
};

inline Folder *ProjectWindow::GetCurrentFolder () {
  return currentFolder;
}

inline MediaClip *ProjectWindow::GetCurrentClip () {
  return currentClip;
}

inline void ProjectWindow::SetFolderSelected (int folderSel) {
  folderSelected = folderSel;
}

#endif  // __PROJECTWINDOW_H__
