/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "singleimage.h"
#include "application.h"
#include "misc.h"
#include <gtk/gtk.h>

/// File/import/single image
void SingleImageClip::Import (GtkMenuItem *menuitem, gpointer user_data) {
  char *fileName = ShowFileSelector ("Select image to import", "");
  if (fileName) {
    SingleImageClip *temp = new SingleImageClip ();
    temp->SetName (g_basename (fileName));
    
    if (temp->SetFileName (fileName)) {
      delete temp;   
      return;
    }
    if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (temp)) {
      delete temp;   
      return;
    }
    app->GetProjectWindow()->BuildProjectListView ();
  }
}

void SingleImageClip::BuildEditWidget (GtkNotebook *parent) {
  StillClip::BuildEditWidget (parent);
}


char *SingleImageClip::GetTypeName () {
  return "Still image";
}

int SingleImageClip::Uses (MediaClip *clip) {
  if (clip == this)
    return 1;
  return 0;
}

SingleImageClip::SingleImageClip () {
  fileName = NULL;
}

SingleImageClip::~SingleImageClip () {
  if (fileName)
    delete fileName;
}

void SingleImageClip::Save (XmlToken *dest) {
  StillClip::Save (dest);

  dest->AppendString ("Filename", fileName);
  dest->SetKey ("SingleImage");
}

void SingleImageClip::Load (XmlToken *src) {
  StillClip::Load (src);
  if (SetFileName (src->GetString ("Filename"))) {
    printf ("Arrgh\n");
    fileName = "Undefined";
  }
}

int SingleImageClip::Erase () {
  return 0;
}
int SingleImageClip::GetSize () {
  return 0;
}
char *SingleImageClip::GetFileLocation () {
  return fileName;
}
int SingleImageClip::GetCacheStatus () {
  return 2;
}
// TODO: Use a real scaling algorithm. For now I just use a DDA based on Breschenham's
// line drawing. In the real world, I should really use a choice of algorithms
void SingleImageClip::Render (int frame, int fps, FrameBuffer *dest) {
  GdkImlibImage *src = gdk_imlib_load_image (fileName);
  GdkRectangle srcRect = {0, 0, src->rgb_width, src->rgb_height};
  GdkRectangle destRect = {0, 0,  dest->GetWidth(), dest->GetHeight()};

  FrameBuffer *temp = new FrameBuffer (src->rgb_width, src->rgb_height, RGB24);
  temp->SetVideoData (src->rgb_data);
  
  dest->StretchBlit (temp, &srcRect, &destRect);
	       
  delete temp;

}

int SingleImageClip::SetFileName (char *newFileName) {
  if (fileName && !strcmp (newFileName, fileName))
    return 0;

  // attempt to load this image
  GdkImlibImage *temp = gdk_imlib_load_image (newFileName);
  if (!temp)
    return 1;

  gdk_imlib_kill_image (temp);
  if (fileName)
    delete fileName;

  fileName = g_strdup (newFileName);
  return 0;
}
