/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "stillclip.h"

/// This is a single (still image) clip.
class SingleImageClip : public StillClip {
  /// The image file used by this clip.
  char *fileName;
 protected:
  /** Build the class-specific bits of the edit box. */
  virtual void BuildEditWidget (GtkNotebook *parent);
  
 public:
  SingleImageClip ();
  virtual ~SingleImageClip ();

  /// Save this clip into a token
  virtual void Save (XmlToken *dest);
  /// Load this clip from a token
  virtual void Load (XmlToken *src);
    
  /** Return "single image" */
  virtual char *GetTypeName ();
  /** Query if this clip makes use of another clip.
   *  @return 1 if [this clip] = [clip], otherwise 0; */
  virtual int Uses (MediaClip *clip);

  /** Erase this dummy clip
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int Erase ();

  /** Get the disk size of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetSize ();

  /** Get the location of this clips file(s)
   *  This function is a stub. It doesn't do anything, and always returns "" */ 
  virtual char *GetFileLocation ();

  /** Get cache status of this clip
   *  This function is a stub. It doesn't do anything, and always returns 2 (N/A) */
  virtual int GetCacheStatus ();

  /* Render this clip's data into the current stream. */
  virtual void Render (int frame, int fps, FrameBuffer *dest);

  /** Set the image file used by this clip.
   *  This sets this image to use the file given in FileName.
   *  @param newFileName the new file to use 
   *  @return error flag: 0 = this file is ok, 1 = Imlib can not load this file*/
  int SetFileName (char *newFileName);

  // Entry callback from menus
  static void Import (GtkMenuItem *menuitem, gpointer user_data);
};
