/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "stillclip.h"
#include "application.h"
#include "misc.h"
StillClip::StillClip () {
  outPoint = length = clipFps * 10;
  inPoint = 0;
}

void StillClip::ChangeLength () {
  int temp = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (lengthWidget));
  if (length != temp) {
    length = temp;
    app->ClipChanged (this);
    SetRanges ();
    UpdateLengthInfo ();
  }
}

void StillClip::EditRevertClicked () {
  inPoint = revertIn;
  outPoint = revertOut;
  length = revertLength;
  SetRanges ();

  gtk_spin_button_set_value (GTK_SPIN_BUTTON(lengthWidget), length);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(in), inPoint);
  gtk_spin_button_set_value (GTK_SPIN_BUTTON(out), outPoint);

  UpdateLengthInfo ();
  UpdateInInfo     ();
  UpdateOutInInfo  ();
  UpdateOutInfo    ();

  MediaClip::EditRevertClicked ();
}

void StillClip::ChangeOutPoint () {
  int temp = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (out));
  if (inPoint != temp) {
    outPoint = temp;
    app->ClipChanged (this);
    SetRanges ();
    UpdateOutInfo ();
    UpdateOutInInfo ();
  }
}

void StillClip::ChangeInPoint () {
  int temp = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (in));
  if (outPoint != temp) {
    inPoint = temp;
    app->ClipChanged (this);
    SetRanges ();
    UpdateLengthInfo ();
    UpdateOutInInfo ();
  }
}
void StillClip::SetRanges () {
  inBound->lower = 0;
  inBound->upper = outPoint - 1;
  outBound->lower = inPoint + 1;
  outBound->upper = length;
  lengthBound->lower = outPoint;

  gtk_signal_emit_by_name (GTK_OBJECT (inBound), "changed");
  gtk_signal_emit_by_name (GTK_OBJECT (outBound), "changed");
  gtk_signal_emit_by_name (GTK_OBJECT (lengthBound), "changed");

}
void StillClip::Save (XmlToken *dest) {
  MediaClip::Save (dest);
  dest->AppendInt ("Length", length);
}
void StillClip::Load (XmlToken *src) {
  MediaClip::Load (src);
  length = src->GetInt ("Length");
}

int StillClip::GetLength (int fps) {
  return length * fps / clipFps;
}
void StillClip::UpdateLengthInfo () {
  char *temp = BuildLengthString (length, app->GetProject()->GetFPS());
  char *temp2 = g_strconcat ("Frames  (= ", temp, ")", NULL);
  gtk_label_set_text (GTK_LABEL(lengthInfo), temp2);
  delete temp;
  delete temp2;
}
void StillClip::UpdateInInfo () {
  char *temp = BuildLengthString (inPoint, app->GetProject()->GetFPS());
  char *temp2 = g_strconcat ("Frames  (= ", temp, ")", NULL);
  gtk_label_set_text (GTK_LABEL(inInfo), temp2);
  delete temp;
  delete temp2;
}
void StillClip::UpdateOutInfo () {
  char *temp = BuildLengthString (outPoint, app->GetProject()->GetFPS());
  char *temp2 = g_strconcat ("(", temp, ")", NULL);
  gtk_label_set_text (GTK_LABEL(outInfo), temp2);
  delete temp;
  delete temp2;
}
void StillClip::UpdateOutInInfo () {
  char *temp = BuildLengthString (outPoint - inPoint, app->GetProject()->GetFPS());
  gtk_label_set_text (GTK_LABEL(outInInfo), temp);
  delete temp;
}

void StillClip::on_length_changed (GtkEditable *editable, gpointer user_data) {
  ((StillClip *)user_data)->ChangeLength ();
}
void StillClip::on_in_changed (GtkEditable *editable, gpointer user_data) {
  ((StillClip *)user_data)->ChangeInPoint ();
}
void StillClip::on_out_changed (GtkEditable *editable, gpointer user_data) {
  ((StillClip *)user_data)->ChangeOutPoint ();
}

void StillClip::BuildEditWidget (GtkNotebook *parent) {
  GtkWidget *vbox34;
  GtkWidget *table18;
  GtkWidget *label119;
  GtkWidget *label118;
  GtkWidget *label117;
  GtkWidget *hbox2;
  GtkWidget *label124;
  GtkWidget *label125;
  GtkWidget *label122;
  GtkWidget *label123;
  GtkWidget *hseparator15;
  GtkWidget *label113;

  vbox34 = gtk_vbox_new (FALSE, 0);
  gtk_widget_show (vbox34);
  gtk_container_border_width (GTK_CONTAINER (vbox34), 4);

  table18 = gtk_table_new (3, 4, FALSE);
  gtk_widget_show (table18);
  gtk_box_pack_start (GTK_BOX (vbox34), table18, FALSE, FALSE, 0);

  label119 = gtk_label_new ("OUT-point:");
  gtk_widget_show (label119);
  gtk_table_attach (GTK_TABLE (table18), label119, 0, 1, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label119), 0, 0.5);

  label118 = gtk_label_new ("IN-point:");
  gtk_widget_show (label118);
  gtk_table_attach (GTK_TABLE (table18), label118, 0, 1, 1, 2,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label118), 0, 0.5);

  label117 = gtk_label_new ("Length:");
  gtk_widget_show (label117);
  gtk_table_attach (GTK_TABLE (table18), label117, 0, 1, 0, 1,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (label117), 0, 0.5);

  inInfo = gtk_label_new ("(XX:YY:ZZ)");
  gtk_widget_show (inInfo);
  gtk_table_attach (GTK_TABLE (table18), inInfo, 2, 3, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (inInfo), 0, 0.5);

  outInfo = gtk_label_new ("(XX:YY:ZZ)");
  gtk_widget_show (outInfo);
  gtk_table_attach (GTK_TABLE (table18), outInfo, 2, 3, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) 0, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (outInfo), 0, 0.5);

  lengthBound = GTK_ADJUSTMENT(gtk_adjustment_new (length, 1, 10000, 1, 10, 10));
  lengthWidget = gtk_spin_button_new (GTK_ADJUSTMENT (lengthBound), 1, 0);
  gtk_widget_show (lengthWidget);
  gtk_table_attach (GTK_TABLE (table18), lengthWidget, 1, 2, 0, 1,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 4, 2);
  gtk_widget_set_usize (lengthWidget, 80, -1);
  gtk_signal_connect (GTK_OBJECT (lengthWidget), "changed",
                      GTK_SIGNAL_FUNC (on_length_changed),
                      this);
  
  inBound = GTK_ADJUSTMENT(gtk_adjustment_new (inPoint, 1, 100, 1, 10, 10));
  in = gtk_spin_button_new (inBound, 1, 0);
  gtk_widget_show (in);
  gtk_table_attach (GTK_TABLE (table18), in, 1, 2, 1, 2,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 0, 2);
  gtk_widget_set_usize (in, 80, -1);
  gtk_signal_connect (GTK_OBJECT (in), "changed",
                      GTK_SIGNAL_FUNC (on_in_changed),
                      this);

  outBound = GTK_ADJUSTMENT(gtk_adjustment_new (outPoint, 1, 100, 1, 10, 10));
  out = gtk_spin_button_new (GTK_ADJUSTMENT (outBound), 1, 0);
  gtk_widget_show (out);
  gtk_table_attach (GTK_TABLE (table18), out, 1, 2, 2, 3,
                    (GtkAttachOptions) 0, (GtkAttachOptions) 0, 10, 2);
  gtk_widget_set_usize (out, 80, -1);
  gtk_signal_connect (GTK_OBJECT (out), "changed",
                      GTK_SIGNAL_FUNC (on_out_changed),
                      this);
  
  lengthInfo = gtk_label_new ("");
  gtk_widget_show (lengthInfo);
  gtk_table_attach (GTK_TABLE (table18), lengthInfo, 2, 4, 0, 1,
                    (GtkAttachOptions) (GTK_EXPAND|GTK_FILL), 
  		    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 0, 0);
  gtk_misc_set_alignment (GTK_MISC (lengthInfo), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (lengthInfo), 8, 0);
  
  outInInfo = gtk_label_new ("");
  gtk_widget_show (outInInfo);
  gtk_table_attach (GTK_TABLE (table18), outInInfo, 3, 4, 2, 3,
                    (GtkAttachOptions) GTK_FILL, (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_misc_set_alignment (GTK_MISC (outInInfo), 0, 0);
  
  label123 = gtk_label_new ("OUT - IN");
  gtk_table_attach (GTK_TABLE (table18), label123, 3, 4, 1, 2,
                    (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), 
		    (GtkAttachOptions) GTK_FILL, 0, 0);
  gtk_widget_show (label123);
  gtk_misc_set_alignment (GTK_MISC (label123), 0, 1);
  
  hseparator15 = gtk_hseparator_new ();
  gtk_widget_show (hseparator15);
  gtk_box_pack_start (GTK_BOX (vbox34), hseparator15, FALSE, FALSE, 4);

  label113 = gtk_label_new ("Since this is a 'timeless' clip, there are no limits on it's length.\n\nYou can set any length you wish for it in the box above...");
  gtk_widget_show (label113);
  gtk_box_pack_start (GTK_BOX (vbox34), label113, TRUE, TRUE, 0);
  gtk_label_set_justify (GTK_LABEL (label113), GTK_JUSTIFY_LEFT);
  gtk_misc_set_alignment (GTK_MISC (label113), 0, 0.5);
  gtk_misc_set_padding (GTK_MISC (label113), 8, 0);

  GtkWidget *label107 = gtk_label_new ("Timing");
  gtk_widget_show (label107);

  // End widget construction
  revertLength = length; 
  revertIn     = inPoint;
  revertOut    = outPoint;

  SetRanges ();
  UpdateLengthInfo ();
  UpdateOutInInfo ();
  UpdateInInfo ();
  UpdateOutInfo ();
  gtk_notebook_append_page (parent, vbox34, label107);
}

