/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __STILLCLIP_H__
#define __STILLCLIP_H__

#include "mclip.h"
#include <gtk/gtk.h>

/** A 'timeless' clip. 
 *  This class represents a clip that doesn't have any timing info. This is typically
 *  a still image or a solid color. This class provides the means for the user to handle
 *  The length of this clip. <p>
 *  The intresting part lies in BuildWidget & DestroyWidget.
 */
class StillClip : public MediaClip {
  /// The values to reset to on a revert click
  int revertLength, revertIn, revertOut;
  /// The length of this clip
  int length;
  /// Misc edit box widgets.
  GtkWidget *lengthWidget, *in, *out, *inInfo, *outInfo, *lengthInfo, *outInInfo;
  GtkAdjustment *inBound, *outBound, *lengthBound;
  
  /// The length of this clip has been changed from the edit window.
  void ChangeLength ();
  /// The IN point of this clip has been changed from the edit window.
  void ChangeInPoint ();
  /// The OUT point of this clip has been changed from the edit window.
  void ChangeOutPoint ();
  /** Set the bounds for the length, IN & OUT spinboxes.
   *  This function makes sure that the user can't move the IN point beyond the OUT point,
   *  the OUT point past the end, etc. */
  void SetRanges ();
  /// Update the info boxes.
  void UpdateInInfo ();
  void UpdateLengthInfo ();
  void UpdateOutInfo ();
  void UpdateOutInInfo ();
 protected:
  /// The revert button has been clicked.
  virtual void EditRevertClicked ();
 public:
  /// Load this clip's settings from a token.
  virtual void Load (XmlToken *src);
  /// Save this clip's setting to an XML token.
  virtual void Save (XmlToken *dest);

  /// Set the default length, FPS & IN-OUT points of the clip.
  StillClip ();
  
  /** Build the editing window of this clip.
   *  This function makes a takes a notebook as a parameter and appends the edit 
   *  controls to this. */
  virtual void BuildEditWidget (GtkNotebook *parent);

  /// Get the length of this clip.
  virtual int GetLength (int fps);


 private:
  /// Edit box signal callbacks
  static void on_length_changed (GtkEditable *editable, gpointer user_data);
  static void on_in_changed (GtkEditable *editable, gpointer user_data);
  static void on_out_changed (GtkEditable *editable, gpointer user_data);
};

#endif // __STILLCLIP_H__
