/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "timeline.h"
#include "application.h"
#include "misc.h"
#include "timelineedit.h"

#include <gnome.h>
TimelineClip::TimelineClip () {
  /// Dummy vars to check for valid startup
  start = layer = 10000;
  in = length = 10000;
}

void TimelineClip::Load (XmlToken *src) {
  start = src->GetInt ("Start");
  length = src->GetInt   ("Length");
  in =  src->GetInt   ("IN");
  layer = src->GetInt ("Layer");
  lock = src->GetInt  ("Locked");
}

void TimelineClip::Save (XmlToken *dest) {
  dest->AppendInt ("Start", start);
  dest->AppendInt ("Length", length);
  dest->AppendInt ("IN", in);
  dest->AppendInt ("Layer", layer);
  dest->AppendInt ("Locked", lock);
  dest->SetKey ("TimelineClip");
}

char *TimelineClip::GetTypeName () {
  return "TimelineClip";
}

void Timeline::BuildEditWidget (GtkNotebook *parent) {}

void Timeline::Add (GtkMenuItem *menuitem, gpointer user_data) {
  char *name = g_strdup ("New timeline");

  if (!ShowInputBox ("Add timeline", "Enter the name of the timeline:", &name))
    return;

  Timeline *temp = new Timeline ();
  temp->SetName (name);

  if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (temp))
    delete temp;   
  else 
    app->GetProjectWindow()->BuildProjectListView ();
}

Timeline::Timeline () {
  clips = NULL;
  outPoint = clipFps * 10;
  inPoint = 0;
  editWindow = NULL;
}

Timeline::~Timeline () {
  g_slist_free (clips);
  if (editWindow)
    delete editWindow;
}

char *Timeline::GetTypeName () {
  return "Timeline";
}

int Timeline::Erase () {
  return 0;
}

int Timeline::GetSize () {
  return 0;
}

char *Timeline::GetFileLocation () {
  return "N/A";
}

int Timeline::GetCacheStatus () {
  return 2;
}

void Timeline::Save (XmlToken *dest) {
  MediaClip::Save (dest);
}

void Timeline::Load (XmlToken *src) {
  MediaClip::Load (src);
}

void Timeline::Render (int frame, int fps, FrameBuffer *dest) {
  frame = frame * clipFps / fps;

  if (frame < 0)
    frame = 0;

  //  if (frame > length)
  //    frame = length;

  // Find all clips to go in this timeline
  GSList *temp = clips;
  GSList *list = NULL;
  while (temp) {
    TimelineClip *temp2 = (TimelineClip *)temp->data;
    if (temp2->GetStart () <= frame && 
	temp2->GetEnd () > frame) {
      list = g_slist_append (list, temp2);
    }
    temp = temp->next;
  }
  
  // Render clips in order defined by layer.
  while (list) {
    TimelineClip *minClip;
    int min = 10000;
    temp = list;
    while (temp) {
      TimelineClip *temp2 = (TimelineClip *)temp->data;
      if (temp2->GetLayer() < min) {
      	minClip = temp2;
	min = temp2->GetLayer();
      }
      temp = temp->next;
    }

    // minimum has been found - render & remove from queue
    minClip->GetSourceClip()->Render (frame - minClip->GetStart (), fps, dest);
    list = g_slist_remove (list, minClip);
  }
}
int Timeline::GetLength (int fps) {
  int length = 0;

  GSList *temp = clips;
  while (temp) {
    TimelineClip *temp2 = (TimelineClip *)temp->data;
    if (temp2->GetEnd () >= length)
      length = temp2->GetEnd ();
  }
}

int Timeline::Uses (MediaClip *clip) {
  return FALSE;
}
void Timeline::Edit () {
  if (!editWindow)
    editWindow = new TimelineWindow (this);
}

void Timeline::AddClip (MediaClip *clip) {
  TimelineClip *temp = new TimelineClip ();
  printf ("AddClip\n");
  temp->SetStart (GetCurrentFrame (clipFps));
  temp->SetIn    (clip->GetInPoint (clipFps));
  temp->SetLength (clip->GetOutPoint (clipFps) - 
		  clip->GetInPoint (clipFps));
  temp->SetLayer (0);
  temp->SetSourceClip (clip);
  AddClip (temp);
}

void Timeline::AddClip (TimelineClip *clip) {
  clips = g_slist_append (clips, clip);
}


/// Set the frame & layer of a specific clip.
void Timeline::MoveClip (TimelineClip *clip, int newFrame, int newLayer,
			 TimelineEdit *editor, GnomeCanvasItem *item) {
  if (IsFree (newLayer, newFrame, newFrame + clip->GetEnd() - clip->GetStart()), clip) {
    clip->SetStart (newFrame);
    clip->SetLayer (newLayer);
    if (editor) 
      editor->RegenPos ();
    app->ClipChanged (this);
  } else 
    printf ("not free\n");
} 

int Timeline::IsFree (int layer, int start, int end, TimelineClip *ignore) {
  GSList *temp = clips;
  
  while (temp) {
    TimelineClip *clip = (TimelineClip *)temp->data;
    if ((ignore == NULL || clip != ignore) &&
	clip->GetLayer () == layer && (
	clip->GetEnd() < end && clip->GetEnd() > start ||
	clip->GetStart() < end && clip->GetStart() > start))
	return false;
    temp = temp->next;
  }
  return true;
}












