
/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */


#include "timelineedit.h"
#include "application.h"


TimelineEdit::TimelineEdit (GnomeCanvasGroup *parent, Timeline *src) {
  root = parent;
  source = src;

  zoom = 1.0;
  bars = NULL;

  Build ();
}

void TimelineEdit::Signal (AppSignal *signal) {
  switch (signal->type) {
  case APP_SETTINGS:
    RegenColors ();
    break;
  case CLIP_CHANGED:
    //    if (source->Uses ((MediaClip *)signal->data))
      RegenPos ();
    break;
  }
}


gint TimelineEdit::_ItemEvent (GnomeCanvasItem *item, GdkEvent *event, 
			       gpointer data) {
  static double x, y; /* used to keep track of motion coordinates */
  double new_x, new_y;
  
  switch (event->type) {
  case GDK_BUTTON_PRESS:
    if (event->button.button == 1) {
      ((TimelineEdit *)data)->DragStart (item, (int)event->button.x, (int)event->button.y);
      return TRUE;
    } else if (event->button.button == 3) {
      /* Destroy the item */
      gtk_object_destroy (GTK_OBJECT (item));
      return TRUE;
    }
    break;

  case GDK_BUTTON_RELEASE:
    ((TimelineEdit *)data)->DragEnd ((int)event->button.x,(int)event->button.y);
    return TRUE;
    
  case GDK_MOTION_NOTIFY:
    if (event->button.state & GDK_BUTTON1_MASK) {
      ((TimelineEdit *)data)->Drag ((int)event->motion.x, (int)event->motion.y);
      return TRUE;
    }
    break;

  case GDK_2BUTTON_PRESS:
    ((MediaClip *)gtk_object_get_user_data (GTK_OBJECT(item)))->Edit ();
    return TRUE;
   
  case GDK_ENTER_NOTIFY:
    gnome_canvas_item_set (item, "width_pixels", 2, NULL);
    return TRUE;
  case GDK_LEAVE_NOTIFY:
    gnome_canvas_item_set (item, "width_pixels", 1, NULL);
    return TRUE;
    
  default:
    break;
  }
  
  return FALSE;
}

void TimelineEdit::Cut () {}

void TimelineEdit::Copy () {}

void TimelineEdit::Paste () {
  // Check for empty clipboard
  if (!app->GetClipboard ())
    return;

  char *clipKey = app->GetClipboardDesc ();
  
  if (!app->IsClipboardMediaClip ()) {

    if (!g_strcasecmp (clipKey, "Folder")) {   // It's a folder
      ShowDialog ("Cut, Copy & Paste", "Error:\n\nYou cannot paste a folder into a timeline",
		  Error);
      return;
    }
    // TODO: handle timelineclips
  } else {
    source->AddClip (app->GetClipboardMediaClip());
  }
  Build ();
}

void TimelineEdit::DragStart (GnomeCanvasItem *item, int x, int y) {
  /* Remember starting position */
  TimelineClip *src = (TimelineClip *)gtk_object_get_user_data (GTK_OBJECT(item));

  dragStartX = x % (int)zoom + x - (int)((double)src->GetStart() * zoom);
  dragStartY = y % 16 + y - src->GetLayer () * 16;
  dragItem = item;
}

void TimelineEdit::DragEnd (int x, int y) {
  MediaClip *clip = (MediaClip *)gtk_object_get_user_data (GTK_OBJECT(dragItem));
}

void TimelineEdit::Drag (int x, int y) {
  x -= dragStartX;
  y -= dragStartY;

  TimelineClip *clip = (TimelineClip *)gtk_object_get_user_data (GTK_OBJECT(dragItem));

  int relLayer = y / 16;
  int relFrame = (int)(((double)x) / zoom);

  source->MoveClip (clip, relFrame, relLayer, this, dragItem);
}

void TimelineEdit::Build () {
  GnomeCanvasGroup *group;
  group = GNOME_CANVAS_GROUP (gnome_canvas_item_new (
				root, gnome_canvas_group_get_type (),
				NULL));
 
  // Loop through all timelineclips contained in this timeline 
  GSList *temp = source->GetClips ();
  while (temp) {
    GnomeCanvasItem *item;
    TimelineClip *clip = (TimelineClip *)temp->data;
    item = gnome_canvas_item_new (group,
				  gnome_canvas_rect_get_type (),
				  "outline_color", "black",
				  "width_pixels", 1,
				  NULL);
    gtk_object_set_user_data (GTK_OBJECT(item), clip);
    gtk_signal_connect (GTK_OBJECT (item), "event",
			(GtkSignalFunc) _ItemEvent,
			this);
    bars = g_slist_append (bars, item);
    temp = temp->next;
  }
  RegenColors ();
  RegenPos ();
}
void TimelineEdit::RegenColors () {
  GSList *temp = bars;
  while (temp) {
    GnomeCanvasItem *item = (GnomeCanvasItem*)temp->data;
    TimelineClip *clip = 
      (TimelineClip *)gtk_object_get_user_data (GTK_OBJECT(item));
    
    gnome_canvas_item_set (item, 
			   "fill_color_rgba", 
			   (guint) app->GetPrefs()->timeLineVideoColor,
			   NULL);
    temp = temp->next;
  }
}

void TimelineEdit::BuildPos (TimelineClip *clip, GnomeCanvasItem *item) {
  RegenPos ();
}

void TimelineEdit::RegenPos () {
  GSList *temp = bars;
  while (temp) {
    GnomeCanvasItem *item = (GnomeCanvasItem *)temp->data;
    TimelineClip *clip = 
      (TimelineClip *)gtk_object_get_user_data (GTK_OBJECT(item));
    gnome_canvas_item_set (item, 
			   "x1", (double)(clip->GetStart() * zoom),
			   "y1", (double)(clip->GetLayer() * 16),
			   "x2", (double)(clip->GetEnd() * zoom),
			   "y2", (double) (clip->GetLayer() * 16 + 16),
			   NULL);


    temp = temp->next;
  }
}

void TimelineEdit::SetZoom (double newZoom) {
  zoom = newZoom;
  RegenPos ();
}




