/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __TIMELINEEDIT_H__
#define __TIMELINEEDIT_H__

#include <gnome.h>
#include "cliptimer.h"
#include "appsignal.h"
#include "timeline.h"

/** A Gnomecanvas containing a timeline.
 *  This is a completely self-contained unit for placing timelines. */
class TimelineEdit : public SignalListener {
  /// The size of this editor
  int width, height;
  /// How many pixels to use per frame.
  double zoom;
  /// The root gnome canvas. All clip items are placed on this.
  GnomeCanvasGroup *root;
  /// The timeline that is being editing
  Timeline *source;
  /// List of edit bars (should match source timeline's timelineClip
  GSList *bars;
  /** Build the edit widgets.
   *  This function stuffs all the TimelineClip's canvas items into parent. */
  void Build ();
  /// Drag start coordinates.
  int dragStartX, dragStartY;
  /// Item being dragged
  GnomeCanvasItem *dragItem;
  /// Initialize a drag operation
  void DragStart (GnomeCanvasItem *item, int x, int y);
  /// Drag handler
  void Drag (int x, int y);
  /// End a drag operation
  void DragEnd (int x, int y);
 public:
  /** Make a new timeline editor.
   *  @param parent the gnome canvas to insert this editor in.
   *  @param src the timeline to edit. */
  TimelineEdit (GnomeCanvasGroup *parent, Timeline *src);

  /// Handle Clipchanged & config signals from app
  void Signal (AppSignal *signal);

  /// Regenerate positions (typically on SetZoom)
  void RegenPos ();

  /// Regenerate color (typically on APP_SETTINGS signal)
  void RegenColors ();

  /// Set a new zoom factor
  void SetZoom (double newZoom);

  /// Get a new zoom factor
  inline double GetZoom ();

  /// Erase any selections
  void Deselect ();

  /// Returns a list of child selections
  GSList *GetSelection ();

  /// Update a clip in a timeline editor.
  void UpdateClip (TimelineClip *clip);

  /// Build the position info for one clip 
  void BuildPos (TimelineClip *clip, GnomeCanvasItem *item);

  /// Cut selection to clipboard.
  void Cut ();
  /// Copy selection to clipboard.
  void Copy ();
  /// Paste selection from clipboard.
  void Paste ();
 private:
  /// Callbacks
  static gint _ItemEvent (GnomeCanvasItem *item, GdkEvent *event, gpointer data);
  /// Main item callback
  ItemEvent (GnomeCanvasItem *item, GdkEvent *event);
};

inline double TimelineEdit::GetZoom () {
  return zoom;
}

#endif
