/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#ifndef __TIMELINEWINDOW_H__
#define __TIMELINEWINDOW_H__

#include <gnome.h>
#include "cliptimer.h"
#include "appsignal.h"

class Timeline;
class TimelineEdit;

/// Class to encapsulate an edit window for timelines.
class TimelineWindow : public SignalListener {
  /// How many pixels per frame to show
  double zoom;
  /// the source timeline being show
  Timeline *source;
  /// The clip timer along the top
  ClipTimer *clipTimer;
  /// The main edit window
  GtkWidget *window;
  /// The Timeline edit GnomeCanvas
  GtkWidget *editCanvas;
  /// The main Timeline editor
  TimelineEdit *editor;

  /// The width of the canvas
  int width;
  /// The height of the canvas
  int height;
  /// Build the window
  void BuildWindow ();
  /// Build the toolbar
  void BuildToolbar ();

  /// Event callback
  static gint canvas_event (GnomeCanvasItem *item, GdkEvent *event, gpointer data);

  /// Main event handler
  gint CanvasEvent (GnomeCanvasItem *item, GdkEvent *event);

  /// Set the dimensions of the editing canvas
  void SetSize (int newWidth, int newHeight);

 public:
  /** Build a new timeline edit window
   *  @param src the source timeline to edit */
  TimelineWindow (Timeline *src);
  ~TimelineWindow ();
  /// Get back the number of pixels per frame
  inline double GetZoom ();

  /// Enlarge the display
  void ZoomIn ();
  /// Shrink the display
  void ZoomOut ();
  /// Cut selection -> clipboard
  void Cut ();
  /// Copy selection -> clipboard
  void Copy ();
  /// Copy clipboard -> selection
  void Paste ();
  /// Handle an app signal
  virtual void Signal (AppSignal *signal);
 private:
  /// Toolbar callbacks
  static void _ZoomIn  (GtkMenuItem *menuitem, gpointer user_data);
  static void _ZoomOut (GtkMenuItem *menuitem, gpointer user_data);
  static void _Cut  (GtkMenuItem *menuitem, gpointer user_data);
  static void _Copy (GtkMenuItem *menuitem, gpointer user_data);
  static void _Paste (GtkMenuItem *menuitem, gpointer user_data);
};

inline double TimelineWindow::GetZoom () {
  return zoom;
}

#endif
