#include <config.h>

#include <stdio.h>
#include <stdlib.h>

#include <gnome.h>

#include "toolbox.h"
#include "application.h"
#include "misc.h"
#include "singleimage.h"
#include "imagesequence.h"
#include "trisequence.h"
#include "colorclip.h"
#include "timeline.h"

// Get images for toolbar
#include "pixmaps/timeline.xpm"
#include "pixmaps/colorclip.xpm"

// Get images for menu
#include "pixmaps/timeline_m.xpm"
#include "pixmaps/colorclip_m.xpm"



GnomeUIInfo Toolbox::import_menu[] = {
  { GNOME_APP_UI_ITEM, N_("_Single image..."), N_("Import a single image"), 
    SingleImageClip::Import, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("_Image sequence..."), N_("Create a single image"),
    ImageSequenceClip::Import, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("_Tri sequence..."), N_("Create a single image"),
    TriSequenceClip::Import, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  GNOMEUIINFO_END
};

GnomeUIInfo Toolbox::file_menu[] = {
  { GNOME_APP_UI_ITEM, N_("_New project"), N_("Create a new project"), 
    New_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_NEW, 'n', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("_Open project..."), N_("Open an existing project"), 
    Open_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_OPEN, 'o', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("_Save project"), N_("Save the current project"), 
    Save_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_SAVE, 's', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Save project _as..."), N_("Save the current project under a new name"), 
    Save_project_as, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_SAVE_AS, 0, (GdkModifierType)0, NULL },
  { GNOME_APP_UI_ITEM, N_("_Close project"), N_("Close the current project"), 
    &Close_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_CLOSE, 0, (GdkModifierType)0, NULL },
  
  GNOMEUIINFO_SEPARATOR,

  GNOMEUIINFO_SUBTREE (N_("_Import"), import_menu),

  GNOMEUIINFO_SEPARATOR,
  { GNOME_APP_UI_ITEM, N_("_Preferences..."), N_("Edit the program setup"), 
    ShowAppPrefs, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_PREF, 0, (GdkModifierType)0, NULL },
  { GNOME_APP_UI_ITEM, N_("E_xit"), N_("Exit the program"), 
    &ProjectWindow::Quit, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_QUIT, 'q', GDK_CONTROL_MASK, NULL },

  GNOMEUIINFO_END
};

GnomeUIInfo Toolbox::edit_menu[] = {
  { GNOME_APP_UI_ITEM, N_("_Undo"), N_("Undo the last action"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_UNDO, 'z', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("_Redo"), N_("Redo the last Undo'ed action"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_REDO, 'y', GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_SEPARATOR,
  { GNOME_APP_UI_ITEM, N_("_Cut"), N_("Cut the selection to the clipboard"), 
    Cut, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_CUT, 'x', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("_Copy"), N_("Copy the selection to the clipboard"), 
    Copy, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_COPY, 'c', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("_Paste"), N_("Paste the contents of the clipboard into the app"), 
    Paste, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_PASTE, 'v', GDK_CONTROL_MASK, NULL },

  GNOMEUIINFO_END
};
  
GnomeUIInfo Toolbox::project_menu[] = {
  { GNOME_APP_UI_ITEM, N_("_New folder..."), N_("Make a new folder for organizing your clips"), 
    New_folder, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("_Rename folder..."), N_("Rename the current folder"),
    Rename_folder, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("_Delete folder"), N_("Delete the current folder and any media it contains"),
    Delete_folder, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  GNOMEUIINFO_SEPARATOR,
  { GNOME_APP_UI_ITEM, N_("_Output settings..."), N_("Edit the output settings for this project"),
    Delete_folder, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_PREF, 0,(GdkModifierType)0, NULL},
  GNOMEUIINFO_END
};

GnomeUIInfo Toolbox::tools_menu[] = {
  { GNOME_APP_UI_ITEM, N_("Add _color clip"), N_("Add a single color clip"), 
    ColorClip::Add, NULL, NULL, GNOME_APP_PIXMAP_DATA,
    colorclip_m_xpm, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("Add new _timeline"), N_("Add a new timeline"), 
    Timeline::Add, NULL, NULL, GNOME_APP_PIXMAP_DATA,
    timeline_m_xpm, 0,(GdkModifierType)0, NULL},
  GNOMEUIINFO_END
};

GnomeUIInfo Toolbox::help_menu[] = {
  { GNOME_APP_UI_ITEM, N_("_Help"), N_("Show the help for Trinity"), 
    //Help, 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_NONE, 
    NULL, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("_About"), N_("Show the author & version info for Trinity"), 
    About, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_MENU_ABOUT, 0,(GdkModifierType)0, NULL},
  GNOMEUIINFO_END
};

GnomeUIInfo Toolbox::main_menu[] = {
  GNOMEUIINFO_SUBTREE (N_("_File"), file_menu),
  GNOMEUIINFO_SUBTREE (N_("_Edit"), edit_menu),
  GNOMEUIINFO_SUBTREE (N_("_Project"), project_menu),
  GNOMEUIINFO_SUBTREE (N_("_Tools"), tools_menu),
  GNOMEUIINFO_SUBTREE (N_("_Help"), help_menu),
  GNOMEUIINFO_END
}; 

void Toolbox::CreateMenu (GnomeApp *app) {
  gnome_app_create_menus(app, main_menu);

  // right justify help menu
  gtk_menu_item_right_justify(GTK_MENU_ITEM(main_menu[sizeof(main_menu) / 
	sizeof(main_menu[0]) - 2].widget));
}

GnomeUIInfo Toolbox::toolbar[] = {
  { GNOME_APP_UI_ITEM, N_("New"), N_("Create a new project"), 
    New_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_NEW, 'n', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Open"), N_("Open an existing project"), 
    Open_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_OPEN, 'o', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Save"), N_("Save the current project"), 
    Save_project, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_SAVE, 's', GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_SEPARATOR,
  /*  { GNOME_APP_UI_ITEM, N_("Undo"), N_("Undo the last action"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_UNDO, 'z', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Redo"), N_("Redo the last Undo'ed action"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_REDO, 'y', GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_SEPARATOR,
  { GNOME_APP_UI_ITEM, N_("Cut"), N_("Cut the selection to the clipboard"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_CUT, 'x', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Copy"), N_("Copy the selection to the clipboard"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_COPY, 'c', GDK_CONTROL_MASK, NULL },
  { GNOME_APP_UI_ITEM, N_("Paste"), N_("Paste the contents of the clipboard into the app"), 
    NULL, NULL, NULL, GNOME_APP_PIXMAP_STOCK, 
    GNOME_STOCK_PIXMAP_PASTE, 'v', GDK_CONTROL_MASK, NULL },
  GNOMEUIINFO_SEPARATOR,
  */
  { GNOME_APP_UI_ITEM, N_("Solid"), N_("Add a solid color clip"), 
    ColorClip::Add, NULL, NULL, GNOME_APP_PIXMAP_DATA,
    colorclip_xpm, 0,(GdkModifierType)0, NULL},
  { GNOME_APP_UI_ITEM, N_("Timeline"), N_("Add a new timeline"), 
    Timeline::Add, NULL, NULL, GNOME_APP_PIXMAP_DATA,
    timeline_xpm, 0,(GdkModifierType)0, NULL},

  GNOMEUIINFO_END
};


void Toolbox::CreateToolbar (GnomeApp *app) {
  gnome_app_create_toolbar (app, toolbar);
}



// Main menu signal handlers
// file menu
void Toolbox::New_project (GtkMenuItem *menuitem, gpointer user_data) {
  if (!app->CloseProject ())
    return;
  app->NewProject ();
}

void Toolbox::Open_project (GtkMenuItem *menuitem, gpointer user_data) {
  app->OpenProject ();
}


void Toolbox::Save_project (GtkMenuItem *menuitem, gpointer user_data) {
  app->SaveProject ();
}


void Toolbox::Save_project_as (GtkMenuItem *menuitem, gpointer user_data) {
  app->SaveProjectAs ();
}

void Toolbox::Close_project (GtkMenuItem *menuitem, gpointer user_data) {
  if (!app->CloseProject ())
    return;
  app->NewProject ();
}

void Toolbox::Cut (GtkMenuItem *menuitem, gpointer user_data) {
  app->GetProjectWindow()->Cut ();
}
void Toolbox::Copy (GtkMenuItem *menuitem, gpointer user_data) {
  app->GetProjectWindow()->Copy ();
}
void Toolbox::Paste (GtkMenuItem *menuitem, gpointer user_data) {
  app->GetProjectWindow()->Paste ();
}
// New folder
void Toolbox::New_folder (GtkMenuItem *menuitem, gpointer user_data) {
  char *temp = "New Folder";
  if (ShowInputBox ("New folder", "Enter the name of the new folder", &temp))
    app->MakeNewFolder (temp);
}

// Rename folder
void Toolbox::Rename_folder (GtkMenuItem *menuitem, gpointer user_data) {
  char *temp = g_strdup (app->GetProjectWindow()->GetCurrentFolder()->GetName());
  if (ShowInputBox ("Rename folder", "Enter the new name of the folder", &temp))
    app->RenameCurrentFolder (temp);
}

// Delete folder
void Toolbox::Delete_folder (GtkMenuItem *menuitem, gpointer user_data) {
  if (ShowDialog ("Delete folder", "WARNING....\n\nYou are about to delete a folder from your project.\nIf you do this, all media in the folder will be deleted with it. \n\nAre you sure you wish to go ahead ?", Warning))
    return;
  app->EraseCurrentFolder ();
}
void Toolbox::ShowAppPrefs (GtkMenuItem *menuitem, gpointer user_data) {
  app->GetPrefs()->BuildEditWidget ();
}

void Toolbox::About (GtkMenuItem *menuitem, gpointer user_data) {
  static GtkWidget *about = NULL;

  if (about && GTK_IS_WIDGET(about))
    return;
  gchar *authors[] = {"Nicholas Francis - project leader", 
		      "Wim Taymans - compression/decompression",
		      "Sigsegv - makefiles", NULL};

  about = gnome_about_new (_("Trinity"), VERSION, 
				      _("Copyright Nicholas Francis 1999"), 
				      (const gchar**)authors,
				      "This program is copyrighted under the GNU General\nPublic License (Open Source). For more info, see the\nfile COPYING, or visit http://www.fsf.org.\nTrinity's homepage has is located at \nhttp://members.xoom.com/NicholasF/ - check in for updates and download the source", 
				      "icons/mainicon.xpm");
  gtk_widget_show (about);
}




