/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */

#include "codeccentral.h"
#include "trisequence.h"
#include "application.h"
#include "color.h"
#include "misc.h"
#include "assert.h"

#include <stdlib.h>
#include <stdio.h>
#include <sys/time.h>
#include <gtk/gtk.h>

GtkWidget *label17;
GtkWidget *label18;
GtkWidget *progressbar1;

/// File/import/image seq.
void TriSequenceClip::Import (GtkMenuItem *menuitem, gpointer user_data) {
  
  char *fileName = ShowFileSelector ("Select the sequence (.tri)", "");

  if (fileName) {
    TriSequenceClip *temp = new TriSequenceClip ();
    temp->SetName (g_basename (fileName));
    
    if (temp->SetFileName (fileName)) {
      delete temp;
      return;
    }
    if (app->GetProjectWindow()->GetCurrentFolder()->AddClip (temp)) {
      delete temp;   
      return;
    }
    app->GetProjectWindow()->BuildProjectListView ();
  }
}

void TriSequenceClip::BuildEditWidget (GtkNotebook *parent) {
}

void TriSequenceClip::BuildEditWidget (GtkWidget *parent) {
}

TriSequenceClip::TriSequenceClip () {
  startFrame = length = 0;
  prefix = postfix = NULL;
  streamFile = NULL;
  frameData = NULL;
  frameData2 = NULL;
  restoredData = NULL;
  resolution = 1;
  imported = false;
}

TriSequenceClip::~TriSequenceClip () {
  if (prefix)
    delete prefix;
  if (postfix)
    delete postfix;

  if (streamFile) 
    fclose(streamFile);
  if (frameData) 
    delete frameData;
  if (frameData2) 
    delete frameData2;
  if (restoredData) 
    delete restoredData;
  printf("deleting temp %s\n", importedTempFile);
  if (imported) unlink(importedTempFile);
}

char *TriSequenceClip::GetTypeName () {
  return "Tri sequence";
}
int TriSequenceClip::Uses (MediaClip *clip) {
  if (clip == this)
    return 1;
  return 0;
}

int TriSequenceClip::Erase () {
  return 0;
}

void TriSequenceClip::Save (XmlToken *dest) {
  MediaClip::Save (dest);

  dest->AppendInt ("Decimals", decimals);
  dest->AppendInt ("StartFrame", startFrame);
  dest->AppendInt ("Length", length);
  printf ("%p\n", prefix);
  if (prefix && strlen (prefix))
    dest->AppendString ("Prefix", prefix);
  if (prefix && strlen (postfix))
    dest->AppendString ("Postfix", postfix);

  dest->SetKey ("TriSequence");
}

void TriSequenceClip::Load (XmlToken *src) {
  MediaClip::Load (src);

  decimals = src->GetInt ("Decimals");
  startFrame = src->GetInt ("StartFrame");
  length = src->GetInt ("Length");
  prefix = src->GetString ("Prefix");
  if (!prefix)
    prefix = g_strdup ("");
  postfix = src->GetString ("PostFix");
  if (!postfix)
    postfix = g_strdup ("");
}

int TriSequenceClip::GetLength (int fps) {
  return length * fps / clipFps;
}

int TriSequenceClip::GetSize () {
  return streamFileLength;
}
char *TriSequenceClip::GetFileLocation () {
  return "";
}
int TriSequenceClip::GetCacheStatus () {
  return 2;
}

void TriSequenceClip::Render (int frame, int fps, FrameBuffer *dest) {
  int width, height;

  frame *= clipFps / fps;

  assert (frame >= 0 && frame < length);

  CodecFrameInfo *info=codec->GetFrameInfo(codecInfo, frame);

  fseek(streamFile, info->position, SEEK_SET);

  frameData = new unsigned char[info->size];
  fread(frameData, 1, info->size, streamFile); 
  //printf("reading  from %8x for frame %d %8x bytes, data %8x\n", info->position, frame, info->size, frameData);

  codec->data = frameData;
  codec->fb = dest;
  codec->Decompress(codecInfo);
  delete frameData;

}

GtkWidget *TriSequenceClip::BuildConvertWidget (GtkWidget *parent) {


   codec = CodecCentral::GetCodecWithID("TRC0000");

   return codec->GetSettingsWidget(NULL, parent);

}


GtkWidget* TriSequenceClip::BuildConvertStatusWidget (GtkWidget *parent)
{
  GtkWidget *window2;
  GtkWidget *vbox4;
  GtkWidget *frame3;
  GtkWidget *vbox5;
  GtkWidget *label15;
  GtkWidget *label16;
  GtkWidget *hbuttonbox2;
  GtkWidget *button4;

  window2 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
  gtk_widget_ref (window2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "window2", window2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_window_set_title (GTK_WINDOW (window2), _("window2"));

  vbox4 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox4);
  gtk_object_set_data_full (GTK_OBJECT (window2), "vbox4", vbox4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox4);
  gtk_container_add (GTK_CONTAINER (window2), vbox4);
  gtk_container_set_border_width (GTK_CONTAINER (vbox4), 6);

  frame3 = gtk_frame_new (_("Conversion"));
  gtk_widget_ref (frame3);
  gtk_object_set_data_full (GTK_OBJECT (window2), "frame3", frame3,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (frame3);
  gtk_box_pack_start (GTK_BOX (vbox4), frame3, TRUE, TRUE, 0);

  vbox5 = gtk_vbox_new (FALSE, 0);
  gtk_widget_ref (vbox5);
  gtk_object_set_data_full (GTK_OBJECT (window2), "vbox5", vbox5,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (vbox5);
  gtk_container_add (GTK_CONTAINER (frame3), vbox5);

  label15 = gtk_label_new (_("Input:"));
  gtk_widget_ref (label15);
  gtk_object_set_data_full (GTK_OBJECT (window2), "label15", label15,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label15);
  gtk_box_pack_start (GTK_BOX (vbox5), label15, FALSE, FALSE, 0);
  gtk_label_set_justify (GTK_LABEL (label15), GTK_JUSTIFY_RIGHT);

  label17 = gtk_label_new ("reading images : ");
  gtk_widget_ref (label17);
  gtk_object_set_data_full (GTK_OBJECT (window2), "label17", label17,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label17);
  gtk_box_pack_start (GTK_BOX (vbox5), label17, TRUE, TRUE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label17), TRUE);

  label16 = gtk_label_new (_("Output:"));
  gtk_widget_ref (label16);
  gtk_object_set_data_full (GTK_OBJECT (window2), "label16", label16,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label16);
  gtk_box_pack_start (GTK_BOX (vbox5), label16, FALSE, FALSE, 0);

  label18 = gtk_label_new ("writing stream : ");
  gtk_widget_ref (label18);
  gtk_object_set_data_full (GTK_OBJECT (window2), "label18", label18,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (label18);
  gtk_box_pack_start (GTK_BOX (vbox5), label18, TRUE, TRUE, 0);
  gtk_label_set_line_wrap (GTK_LABEL (label18), TRUE);

  progressbar1 = gtk_progress_bar_new ();
  gtk_widget_ref (progressbar1);
  gtk_object_set_data_full (GTK_OBJECT (window2), "progressbar1", progressbar1,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (progressbar1);
  gtk_box_pack_start (GTK_BOX (vbox4), progressbar1, FALSE, FALSE, 0);
  gtk_progress_set_show_text (GTK_PROGRESS (progressbar1), TRUE);

  hbuttonbox2 = gtk_hbutton_box_new ();
  gtk_widget_ref (hbuttonbox2);
  gtk_object_set_data_full (GTK_OBJECT (window2), "hbuttonbox2", hbuttonbox2,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (hbuttonbox2);
  gtk_box_pack_start (GTK_BOX (vbox4), hbuttonbox2, FALSE, TRUE, 0);

  button4 = gnome_stock_button (GNOME_STOCK_BUTTON_CANCEL);
  gtk_widget_ref (button4);
  gtk_object_set_data_full (GTK_OBJECT (window2), "button4", button4,
                            (GtkDestroyNotify) gtk_widget_unref);
  gtk_widget_show (button4);
  gtk_container_add (GTK_CONTAINER (hbuttonbox2), button4);
  GTK_WIDGET_SET_FLAGS (button4, GTK_CAN_DEFAULT);

  return window2;
}


int TriSequenceClip::ConvertFrom (MediaClip *clip, char *fileName, int showGUI, FrameBuffer *buf) {

	unsigned char *framebuf;
	int i;
	int compressedBytes;
	int totalCompressedBytes;
	char propertyBuf[128];
	char inputStatus[128];
	char outputStatus[128];
	
   // save this to destroy the temp file
   strcpy(importedTempFile,fileName);

	// get this codecs info
   codec = CodecCentral::GetCodecWithID("TRC0000");

   if (!codec) {
     printf("ERROR: cannot find codec\n");
	  return 1;
   }

   // set the framebuffer
	codec->fb = buf;
	//framebuf = new unsigned char[IMPORTWIDTH*IMPORTHEIGHT*3];
	framebuf = codec->fb->GetVideoData();

   sprintf(propertyBuf, "%d", clip->GetLength(50));
	codec->SetProperty(NULL, "numFrames", propertyBuf);

   streamFile = fopen(fileName, "wb");

   if (!streamFile) {
     printf("ERROR: cannot open file\n");
	  delete codec->fb;
	  delete framebuf;
	  return 1;
   }

   // reserve some space for the header TODO check for failure
	codec->data = new unsigned char[codec->headerSize];

   // prapre for compression, the header will be in the data member TODO check for failure
	void *codecData = codec->PrepareCompressionStream();	

	// TODO check for failure
	fwrite(codec->data, 1, codec->headerSize, streamFile);

   // its written so we can delete it
	delete codec->data;

	// TODO check for failure
	codec->data = new unsigned char[codec->GetCompressionBufferSize(buf->GetWidth(), buf->GetHeight())];

   totalCompressedBytes = 0;

   GtkWidget *statusWidget = BuildConvertStatusWidget(NULL);
   
	// TODO move the input status generation to the source clip
	sprintf(inputStatus, "\nReading images : %d/%d\n\n", 0, clip->GetLength(50));
	gtk_label_set_text(GTK_LABEL(label17), inputStatus); 
	sprintf(outputStatus, "\nWriting stream : %d\n\n", totalCompressedBytes);
	gtk_label_set_text(GTK_LABEL(label18), outputStatus);

   if (showGUI) gtk_widget_show(statusWidget);

   // make sure the widget shows
	while (gtk_events_pending()) gtk_main_iteration();


	for (i=0; i<clip->GetLength(50); i++) {

		clip->Render(i, 50, codec->fb);

	   // TODO check for failure
		compressedBytes = codec->Compress(codecData);

		fwrite(codec->data, 1, compressedBytes, streamFile);

	   sprintf(inputStatus, "\nReading images : %d/%d\n\n", i, clip->GetLength(50));
	   gtk_label_set_text(GTK_LABEL(label17), inputStatus); 

		// TODO make the codec report on this
	   sprintf(outputStatus, "\nWriting stream : %d\n\n", totalCompressedBytes);
	   gtk_label_set_text(GTK_LABEL(label18), outputStatus);

		gtk_progress_bar_update( GTK_PROGRESS_BAR(progressbar1), i/((gfloat)clip->GetLength(50)));

		totalCompressedBytes += compressedBytes;

	   while (gtk_events_pending()) gtk_main_iteration();

	}

   codec->CloseStream(codecData);
	fclose(streamFile);
	streamFile = NULL;

   if (showGUI) gtk_widget_destroy(statusWidget);

	SetFileName(fileName);
	imported = true;

	// we don't need these...
	delete codec->fb;
	delete framebuf;
	
	return (0);

}

int TriSequenceClip::SetFileName (char *newFileName) {

  if (prefix)
    delete prefix;

  if (postfix)
    delete postfix;

  if (streamFile) fclose (streamFile);

  codec = CodecCentral::GetCodecWithID("TRC0000");

  if (!codec) {
    printf("ERROR: CODEC not found\n");
	 return 0;
  }

  codecInfo = codec->OpenStream(newFileName);

  if (!codecInfo) {
    printf("ERROR: CODEC cannot open file\n");
	 return 0;
  }
  
  streamFile = fopen(newFileName, "rb");

  if (!streamFile) {
    printf("ERROR: cannot open file\n");
	 return 0;
  }


  length = atoi(codec->GetProperty(codecInfo, "numFrames"));

  inPoint = 0;
  outPoint = length;
  
  startFrame = 0;

  return 0;
}

