/*
 * Trinity.
 * 
 * Copyright (C) 1999 by Nicholas Francis
 * 
 * This software comes under the GPL (GNU Public License)
 * You may freely copy,distribute etc. this as long as the source code
 * is made available for FREE.
 * 
 * No warranty is made or implied. You use this program at your own risk.
 */
#ifndef __TRISEQUENCE_H__
#define __TRISEQUENCE_H__

#include <stdio.h>
#include "mclip.h"
#include "codec.h"


/** This is a native clip format loosly based on avi. It can contain
 *  both video and audio.
 *  The format is contructed in such a way that it allows full image and
 *  preview playback from the same stream without needing extra disk space.
 *  The disk bandwidth is also depending on the size (quality) of the image. */
class TriSequenceClip : public MediaClip {
  /** The number of digits in the numbering.
   *  if 0, no padding is done. If -1 no numbering is used (we are dealing with 1 image)or a stream*/
  int decimals;
  /// The start frame number of this imported clip.
  int startFrame;
  /// The length (in frames) of this clip
  int length;
  /// The resolution to draw 
  int resolution;
  /// the pre & postfix of the filename string.
  char *prefix, *postfix;

  /// the file associated with this stream
  FILE *streamFile;

  /// the length of this stream in bytes
  long streamFileLength;

  /// is this clip imported in a temporary file
  int imported;
  /// and this is the filename
  char importedTempFile[128];

  /// a buffer to hold the frame pixels
  unsigned char *frameData;
  unsigned char *frameData2;
  unsigned char *restoredData;

  Codec *codec;
  void *codecInfo;


 protected:
  virtual void BuildEditWidget (GtkWidget *parent);
 public:
  TriSequenceClip ();
  virtual ~TriSequenceClip ();
  /** Return a textual description of this clip type */
  virtual char *GetTypeName ();

  /** Query if this clip makes use of another clip.
   *  @return 1 if [this clip] = [clip], otherwise 0; */
  virtual int Uses (MediaClip *clip);
  /// Save this clip into a token
  virtual void Save (XmlToken *dest);
  /// Load this clip from a token
  virtual void Load (XmlToken *src);

  /** Erase the dummy clip
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int Erase ();

  /** Get the length of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetLength (int fps);

  /** Get the disk size of this clip.
   *  This function is a stub. It doesn't do anything, and always returns 0 */
  virtual int GetSize ();

  /** Get the location of this clips file(s)
   *  This function is a stub. It doesn't do anything, and always returns "" */ 
  virtual char *GetFileLocation ();

  /** Get cache status of this clip
   *  This function is a stub. It doesn't do anything, and always returns 2 (N/A) */
  virtual int GetCacheStatus ();

  /* Render this clip's data into the current stream.
   * This function is a stud. It doesnt do anything. */
  virtual void Render (int frame, int fps, FrameBuffer *dest);

  /// Build the notebook page for this widget
  virtual void BuildEditWidget (GtkNotebook *parent);

  /// Build the notebook page for this widget
  virtual GtkWidget *BuildConvertWidget (GtkWidget *parent);

  /// Build the notebook page for this widget
  virtual GtkWidget *BuildConvertStatusWidget (GtkWidget *parent);

  /** Set the first image file used by this clip.
   *  This function calculates the files used by this clip & sets up all neccessary vars.
   *  @param newFileName the new file to use 
   *  @return error flag: 0 = this file is ok, 1 = Imlib can not load this file*/
  int SetFileName (char *newFileName);

  // takes the data from clip and store the stream in newFileName, 
  // optionally showing a progressBar.
  // the image size to store the images in the stream is taken from buf.
  int ConvertFrom (MediaClip *clip, char *newFileName, int showGUI, FrameBuffer *buf);

  /// Import menu callback
  static void Import (GtkMenuItem *menuitem, gpointer user_data);
};

#endif

